searchData={"items":[{"type":"module","title":"Oban","doc":"Oban is a robust job processing library which uses PostgreSQL or SQLite3 for\nstorage and coordination.","ref":"Oban.html"},{"type":"module","title":"Installation - Oban","doc":"See the [installation guide](https://hexdocs.pm/oban/installation.html) for\ndetails on installing and configuring Oban in your application.","ref":"Oban.html#module-installation"},{"type":"module","title":"Requirements - Oban","doc":"Oban requires Elixir 1.12+, Erlang 22+, and PostgreSQL 12.0+ or SQLite3 3.37.0+.","ref":"Oban.html#module-requirements"},{"type":"module","title":"Testing - Oban","doc":"Find testing setup, helpers, and strategies in the [testing guide](https://hexdocs.pm/oban/testing.html).","ref":"Oban.html#module-testing"},{"type":"module","title":"Oban Web+Pro - Oban","doc":"A web dashboard for managing Oban, along with an official set of extensions,\nplugins, and workers are available as licensed packages:\n\n* [🧭 Web Overview](https://getoban.pro#oban-web)\n* [🌟 Pro Overview](https://getoban.pro#oban-pro)\n\nLearn more about prices and licensing Oban Web+Pro at [getoban.pro][pro].\n\n[pro]: https://getoban.pro","ref":"Oban.html#module-oban-web-pro"},{"type":"module","title":"Running with SQLite3 - Oban","doc":"Oban ships with engines for PostgreSQL and SQLite3. Both engines support the\nsame core functionality for a single node, while the Postgres engine is more\nadvanced and designed to run in a distributed environment.\n\nRunning with SQLite3 requires adding `ecto_sqlite3` to your app's dependencies\nand setting the `Oban.Engines.Lite` engine:\n\n```elixir\nconfig :my_app, Oban,\n  engine: Oban.Engines.Lite,\n  queues: [default: 10],\n  repo: MyApp.Repo\n```\n\n_Please note that SQLite3 may not be suitable for high-concurrency systems or for systems that need\nto handle large amounts of data. If you expect your background jobs to generate high loads, it\nwould be better to use a more robust database solution that supports horizontal scalability, like\nPostgres._","ref":"Oban.html#module-running-with-sqlite3"},{"type":"module","title":"Configuring Queues - Oban","doc":"Queues are specified as a keyword list where the key is the name of the queue\nand the value is the maximum number of concurrent jobs. The following\nconfiguration would start four queues with concurrency ranging from 5 to 50:\n\n```elixir\nconfig :my_app, Oban,\n  queues: [default: 10, mailers: 20, events: 50, media: 5],\n  repo: MyApp.Repo\n```\n\nYou may also use an expanded form to configure queues with individual overrides:\n\n```elixir\nqueues: [\n  default: 10,\n  events: [limit: 50, paused: true]\n]\n```\n\nThe `events` queue will now start in a paused state, which means it won't\nprocess anything until `Oban.resume_queue/2` is called to start it.\n\nThere isn't a limit to the number of queues or how many jobs may execute\nconcurrently in each queue. Some additional guidelines:\n\n#### Caveats & Guidelines\n\n* Each queue will run as many jobs as possible concurrently, up to the\n  configured limit. Make sure your system has enough resources (i.e. database\n  connections) to handle the concurrent load.\n\n* Queue limits are local (per-node), not global (per-cluster). For example,\n  running a queue with a local limit of one on three separate nodes is\n  effectively a global limit of three. If you require a global limit you must\n  restrict the number of nodes running a particular queue.\n\n* Only jobs in the configured queues will execute. Jobs in any other queue will\n  stay in the database untouched.\n\n* Be careful how many concurrent jobs make expensive system calls (i.e. FFMpeg,\n  ImageMagick). The BEAM ensures that the system stays responsive under load,\n  but those guarantees don't apply when using ports or shelling out commands.","ref":"Oban.html#module-configuring-queues"},{"type":"module","title":"Defining Workers - Oban","doc":"Worker modules do the work of processing a job. At a minimum they must define a\n`perform/1` function, which is called with an `%Oban.Job{}` struct.\n\nNote that the `args` field of the job struct will _always_ have string keys, regardless of the key\ntype when the job was enqueued. The `args` are stored as JSON and the serialization process\nautomatically stringifies all keys. Also, because `args` are always encoded as JSON, you must\nensure that all values are serializable, otherwise you'll have encoding errors when inserting jobs.\n\nDefine a worker to process jobs in the `events` queue:\n\n```elixir\ndefmodule MyApp.Business do\n  use Oban.Worker, queue: :events\n\n  @impl Oban.Worker\n  def perform(%Oban.Job{args: %{\"id\" => id} = args}) do\n    model = MyApp.Repo.get(MyApp.Business.Man, id)\n\n    case args do\n      %{\"in_the\" => \"business\"} ->\n        IO.inspect(model)\n\n      %{\"vote_for\" => vote} ->\n        IO.inspect([vote, model])\n\n      _ ->\n        IO.inspect(model)\n    end\n\n    :ok\n  end\nend\n```\n\nThe `use` macro also accepts options to customize `max_attempts`, `priority`, `tags`, and `unique`\noptions:\n\n```elixir\ndefmodule MyApp.LazyBusiness do\n  use Oban.Worker,\n    queue: :events,\n    priority: 3,\n    max_attempts: 3,\n    tags: [\"business\"],\n    unique: [period: 30]\n\n  @impl Oban.Worker\n  def perform(_job) do\n    # do business slowly\n\n    :ok\n  end\nend\n```\n\nLike all `use` macros, options are defined at compile time. Avoid using `Application.get_env/2` to\ndefine worker options. Instead, pass dynamic options at runtime by passing them to\n`MyWorker.new/2`:\n\n```elixir\nMyApp.MyWorker.new(args, queue: dynamic_queue)\n```\n\nSuccessful jobs should return `:ok` or an `{:ok, value}` tuple. The value returned from\n`perform/1` is used to control whether the job is treated as a success, a failure, cancelled or\ndeferred for retrying later.\n\nSee the `Oban.Worker` docs for more details on failure conditions and `Oban.Telemetry` for details\non job reporting.","ref":"Oban.html#module-defining-workers"},{"type":"module","title":"Enqueueing Jobs - Oban","doc":"Jobs are simply Ecto structs and are enqueued by inserting them into the\ndatabase. For convenience and consistency all workers provide a `new/2`\nfunction that converts an args map into a job changeset suitable for insertion:\n\n```elixir\n%{id: 1, in_the: \"business\", of_doing: \"business\"}\n|> MyApp.Business.new()\n|> Oban.insert()\n```\n\nThe worker's defaults may be overridden by passing options:\n\n```elixir\n%{id: 1, vote_for: \"none of the above\"}\n|> MyApp.Business.new(queue: :special, max_attempts: 5)\n|> Oban.insert()\n```\n\nUnique jobs can be configured in the worker, or when the job is built:\n\n```elixir\n%{email: \"brewster@example.com\"}\n|> MyApp.Mailer.new(unique: [period: 300, fields: [:queue, :worker])\n|> Oban.insert()\n```\n\nJob priority can be specified using an integer from 0 to 9, with 0 being the\ndefault and highest priority:\n\n```elixir\n%{id: 1}\n|> MyApp.Backfiller.new(priority: 2)\n|> Oban.insert()\n```\n\nAny number of tags can be added to a job dynamically, at the time it is\ninserted:\n\n```elixir\nid = 1\n\n%{id: id}\n|> MyApp.OnboardMailer.new(tags: [\"mailer\", \"record-#{id}\"])\n|> Oban.insert()\n```\n\nMultiple jobs can be inserted in a single transaction:\n\n```elixir\nEcto.Multi.new()\n|> Oban.insert(:b_job, MyApp.Business.new(%{id: 1}))\n|> Oban.insert(:m_job, MyApp.Mailer.new(%{email: \"brewser@example.com\"}))\n|> Repo.transaction()\n```\n\nOccasionally you may need to insert a job for a worker that exists in another\napplication. In that case you can use `Oban.Job.new/2` to build the changeset\nmanually:\n\n```elixir\n%{id: 1, user_id: 2}\n|> Oban.Job.new(queue: :default, worker: OtherApp.Worker)\n|> Oban.insert()\n```\n\n`Oban.insert/2,4` is the preferred way of inserting jobs as it provides some of\nOban's advanced features (i.e., unique jobs). However, you can use your\napplication's `Repo.insert/2` function if necessary.\n\nSee `Oban.Job.new/2` for a full list of job options.","ref":"Oban.html#module-enqueueing-jobs"},{"type":"module","title":"Scheduling Jobs - Oban","doc":"Jobs may be scheduled down to the second any time in the future:\n\n```elixir\n%{id: 1}\n|> MyApp.Business.new(schedule_in: 5)\n|> Oban.insert()\n```\n\nJobs may also be scheduled at a specific datetime in the future:\n\n```elixir\n%{id: 1}\n|> MyApp.Business.new(scheduled_at: ~U[2020-12-25 19:00:56.0Z])\n|> Oban.insert()\n```\n\nScheduling is _always_ in UTC. You'll have to shift timestamps in other zones to\nUTC before scheduling:\n\n```elixir\n%{id: 1}\n|> MyApp.Business.new(scheduled_at: DateTime.shift_zone!(datetime, \"Etc/UTC\"))\n|> Oban.insert()\n```\n\n#### Caveats & Guidelines\n\nUsually, scheduled job management operates in `global` mode and notifies queues\nof available jobs via PubSub to minimize database load. However, when PubSub\nisn't available, staging switches to a `local` mode where each queue polls\nindependently.\n\nLocal mode is less efficient and will only happen if you're running in an\nenvironment where neither `Postgres` nor `PG` notifications work. That situation\nshould be rare and limited to the following conditions:\n\n1. Running with a connection pooler, i.e., `pg_bouncer`, in transaction mode.\n2. Running without clustering, i.e., without Distributed Erlang\n\nIf **both** of those criteria apply and PubSub notifications won't work, then\nstaging will switch to polling in `local` mode.","ref":"Oban.html#module-scheduling-jobs"},{"type":"module","title":"Prioritizing Jobs - Oban","doc":"Normally, all available jobs within a queue are executed in the order they were scheduled. You can\noverride the normal behavior and prioritize or de-prioritize a job by assigning a numerical\n`priority`.\n\n* Priorities from 0-9 are allowed, where 0 is the highest priority and 9 is the lowest.\n\n* The default priority is 0, unless specified all jobs have an equally high priority.\n\n* All jobs with a higher priority will execute before any jobs with a lower priority. Within a\n  particular priority jobs are executed in their scheduled order.\n\n#### Caveats & Guidelines\n\nThe default priority is defined in the jobs table. The least intrusive way to change it for all\njobs is to change the column default:\n\n```elixir\nalter table(\"oban_jobs\") do\n  modify :priority, :integer, default: 1, from: {:integer, default: 0}\nend\n```","ref":"Oban.html#module-prioritizing-jobs"},{"type":"module","title":"Unique Jobs - Oban","doc":"The unique jobs feature lets you specify constraints to prevent enqueueing duplicate jobs.\nUniqueness is based on a combination of `args`, `queue`, `worker`, `state` and insertion time. It\nis configured at the worker or job level using the following options:\n\n* `:period` — The number of seconds until a job is no longer considered duplicate. You should\n  always specify a period, otherwise Oban will default to 60 seconds. `:infinity` can be used to\n  indicate the job be considered a duplicate as long as jobs are retained.\n\n* `:fields` — The fields to compare when evaluating uniqueness. The available fields are `:args`,\n  `:queue`, `:worker`, and `:meta`. By default, fields is set to `[:worker, :queue, :args]`.\n\n* `:keys` — A specific subset of the `:args` or `:meta` to consider when comparing against\n  historic jobs. This allows a job with multiple key/value pairs in the args to be compared using\n  only a subset of them.\n\n* `:states` — The job states that are checked for duplicates. The available states are\n  `:available`, `:scheduled`, `:executing`, `:retryable`, `:completed`, `:cancelled` and\n  `:discarded`. By default all states except for `:discarded` and `:cancelled` are checked, which\n  prevents duplicates even if the previous job has been completed.\n\n* `:timestamp` — Which timestamp to check the period against. The available timestamps are\n  `:inserted_at` or `:scheduled_at`, and it defaults to `:inserted_at` for legacy reasons.\n\nFor example, configure a worker to be unique across all fields and states for 60\nseconds:\n\n```elixir\nuse Oban.Worker, unique: [period: 60]\n```\n\nConfigure the worker to be unique only by `:worker` and `:queue`:\n\n```elixir\nuse Oban.Worker, unique: [fields: [:queue, :worker], period: 60]\n```\n\nCheck the `:scheduled_at` timestamp instead of `:inserted_at` for uniqueness:\n\n```elixir\nuse Oban.Worker, unique: [period: 120, timestamp: :scheduled_at]\n```\n\nOr, configure a worker to be unique until it has executed:\n\n```elixir\nuse Oban.Worker, unique: [period: 300, states: [:available, :scheduled, :executing]]\n```\n\nOnly consider the `:url` key rather than the entire `args`:\n\n```elixir\nuse Oban.Worker, unique: [fields: [:args, :worker], keys: [:url]]\n```\n\nYou can use `Oban.Job.states/0` to specify uniqueness across _all_ states,\nincluding `:discarded`:\n\n```elixir\nuse Oban.Worker, unique: [period: 300, states: Oban.Job.states()]\n```\n\n#### Detecting Unique Conflicts\n\nWhen unique settings match an existing job, the return value of `Oban.insert/2`\nis still `{:ok, job}`. However, you can detect a unique conflict by checking the\njobs' `:conflict?` field. If there was an existing job, the field is `true`;\notherwise it is `false`.\n\nYou can use the `:conflict?` field to customize responses after insert:\n\n```elixir\nwith {:ok, %Job{conflict?: true}} <- Oban.insert(changeset) do\n  {:error, :job_already_exists}\nend\n```\n\nNote that conflicts are only detected for jobs enqueued through `Oban.insert/2,3`.\nJobs enqueued through `Oban.insert_all/2` _do not_ use per-job unique\nconfiguration.\n\n#### Replacing Values\n\nIn addition to detecting unique conflicts, passing options to `replace` can\nupdate any job field when there is a conflict. Any of the following fields can\nbe replaced per _state_:  `args`, `max_attempts`, `meta`, `priority`, `queue`,\n`scheduled_at`, `tags`, `worker`.\n\nFor example, to change the `priority` and increase `max_attempts` when there is\na conflict with a job in a `scheduled` state:\n\n```elixir\nBusinessWorker.new(\n  args,\n  max_attempts: 5,\n  priority: 0,\n  replace: [scheduled: [:max_attempts, :priority]]\n)\n```\n\nAnother example is bumping the scheduled time on conflict. Either `scheduled_at`\nor `schedule_in` values will work, but the replace option is always\n`scheduled_at`.\n\n```elixir\nUrgentWorker.new(args, schedule_in: 1, replace: [scheduled: [:scheduled_at]])\n```\n\nNOTE: If you use this feature to replace a field (e.g. `args`) in the\n`executing` state by doing something like: `UniqueWorker.new(new_args, replace:\n[executing: [:args]])` Oban will update the `args`, but the job will continue\nexecuting with the original value.\n\n#### Strong Guarantees\n\nUnique jobs are guaranteed through transactional locks and database queries:\nthey _do not_ rely on unique constraints in the database. This makes uniqueness\nentirely configurable by application code, without the need for database\nmigrations.","ref":"Oban.html#module-unique-jobs"},{"type":"module","title":"Pruning Historic Jobs - Oban","doc":"Job stats and queue introspection are built on keeping job rows in the database\nafter they have completed. This allows administrators to review completed jobs\nand build informative aggregates, at the expense of storage and an unbounded\ntable size. To prevent the `oban_jobs` table from growing indefinitely, Oban\nprovides active pruning of `completed`, `cancelled` and `discarded` jobs.\n\nBy default, the `Pruner` plugin retains jobs for 60 seconds. You can configure a\nlonger retention period by providing a `max_age` in seconds to the `Pruner`\nplugin.\n\n```elixir\n# Set the max_age for 5 minutes\nconfig :my_app, Oban,\n  plugins: [{Oban.Plugins.Pruner, max_age: 300}]\n  ...\n```\n\n#### Caveats & Guidelines\n\n* Pruning is best-effort and performed out-of-band. This means that all limits\n  are soft; jobs beyond a specified age may not be pruned immediately after jobs\n  complete.\n\n* Pruning is only applied to jobs that are `completed`, `cancelled` or\n  `discarded`. It'll never delete a new job, a scheduled job or a job that\n  failed and will be retried.","ref":"Oban.html#module-pruning-historic-jobs"},{"type":"module","title":"Periodic Jobs - Oban","doc":"Oban's `Cron` plugin registers workers a cron-like schedule and enqueues jobs\nautomatically. Periodic jobs are declared as a list of `{cron, worker}` or\n`{cron, worker, options}` tuples:\n\n```elixir\nconfig :my_app, Oban,\n  repo: MyApp.Repo,\n  plugins: [\n    {Oban.Plugins.Cron,\n     crontab: [\n       {\"* * * * *\", MyApp.MinuteWorker},\n       {\"0 * * * *\", MyApp.HourlyWorker, args: %{custom: \"arg\"}},\n       {\"0 0 * * *\", MyApp.DailyWorker, max_attempts: 1},\n       {\"0 12 * * MON\", MyApp.MondayWorker, queue: :scheduled, tags: [\"mondays\"]},\n       {\"@daily\", MyApp.AnotherDailyWorker}\n     ]}\n  ]\n```\n\nThe crontab would insert jobs as follows:\n\n* `MyApp.MinuteWorker` — Inserted once every minute\n* `MyApp.HourlyWorker` — Inserted at the first minute of every hour with custom args\n* `MyApp.DailyWorker` — Inserted at midnight every day with no retries\n* `MyApp.MondayWorker` — Inserted at noon every Monday in the \"scheduled\" queue\n* `MyApp.AnotherDailyWorker` — Inserted at midnight every day with no retries\n\nThe crontab format respects all [standard rules][cron] and has one minute\nresolution. Jobs are considered unique for most of each minute, which prevents\nduplicate jobs with multiple nodes and across node restarts.\n\nLike other jobs, recurring jobs will use the `:queue` specified by the worker\nmodule (or `:default` if one is not specified).","ref":"Oban.html#module-periodic-jobs"},{"type":"module","title":"Cron Expressions - Oban","doc":"Standard Cron expressions are composed of rules specifying the minutes, hours,\ndays, months and weekdays. Rules for each field are comprised of literal values,\nwildcards, step values or ranges:\n\n* `*` — Wildcard, matches any value (0, 1, 2, ...)\n* `0` — Literal, matches only itself (only 0)\n* `*/15` — Step, matches any value that is a multiple (0, 15, 30, 45)\n* `0-5` — Range, matches any value within the range (0, 1, 2, 3, 4, 5)\n* `0-9/2` - Step values can be used in conjunction with ranges (0, 2, 4, 6, 8)\n\nEach part may have multiple rules, where rules are separated by a comma. The\nallowed values for each field are as follows:\n\n* `minute` — 0-59\n* `hour` — 0-23\n* `days` — 1-31\n* `month` — 1-12 (or aliases, `JAN`, `FEB`, `MAR`, etc.)\n* `weekdays` — 0-6 (or aliases, `SUN`, `MON`, `TUE`, etc.)\n\nThe following Cron extensions are supported:\n\n* `@hourly` — `0 * * * *`\n* `@daily` (as well as `@midnight`) — `0 0 * * *`\n* `@weekly` — `0 0 * * 0`\n* `@monthly` — `0 0 1 * *`\n* `@yearly` (as well as `@annually`) — `0 0 1 1 *`\n* `@reboot` — Run once at boot across the entire cluster\n\nSome specific examples that demonstrate the full range of expressions:\n\n* `0 * * * *` — The first minute of every hour\n* `*/15 9-17 * * *` — Every fifteen minutes during standard business hours\n* `0 0 * DEC *` — Once a day at midnight during December\n* `0 7-9,4-6 13 * FRI` — Once an hour during both rush hours on Friday the 13th\n\nFor more in depth information see the man documentation for `cron` and `crontab`\nin your system. Alternatively you can experiment with various expressions\nonline at [Crontab Guru][guru].\n\n#### Caveats & Guidelines\n\n* All schedules are evaluated as UTC unless a different timezone is provided.\n  See `Oban.Plugins.Cron` for information about configuring a timezone.\n\n* Workers can be used for regular _and_ scheduled jobs so long as they accept\n  different arguments.\n\n* Duplicate jobs are prevented through transactional locks and unique\n  constraints. Workers that are used for regular and scheduled jobs _must not_\n  specify `unique` options less than `60s`.\n\n* Long running jobs may execute simultaneously if the scheduling interval is\n  shorter than it takes to execute the job. You can prevent overlap by passing\n  custom `unique` opts in the crontab config:\n\n  ```elixir\n  custom_args = %{scheduled: true}\n\n  unique_opts = [\n    period: 60 * 60 * 24,\n    states: [:available, :scheduled, :executing]\n  ]\n\n  config :my_app, Oban,\n    repo: MyApp.Repo,\n    plugins: [\n      {Oban.Plugins.Cron,\n       crontab: [\n         {\"* * * * *\", MyApp.SlowWorker, args: custom_args, unique: unique_opts}\n       ]}\n    ]\n  ```\n\n[cron]: https://en.wikipedia.org/wiki/Cron#Overview\n[guru]: https://crontab.guru","ref":"Oban.html#module-cron-expressions"},{"type":"module","title":"Error Handling - Oban","doc":"When a job returns an error value, raises an error, or exits during execution the\ndetails are recorded within the `errors` array on the job. When the number of\nexecution attempts is below the configured `max_attempts` limit, the job will\nautomatically be retried in the future.\n\nThe retry delay has an exponential backoff, meaning the job's second attempt\nwill be after 16s, third after 31s, fourth after 1m 36s, etc.\n\nSee the `Oban.Worker` documentation on \"Customizing Backoff\" for alternative\nbackoff strategies.","ref":"Oban.html#module-error-handling"},{"type":"module","title":"Error Details - Oban","doc":"Execution errors are stored as a formatted exception along with metadata about\nwhen the failure occurred and which attempt caused it. Each error is stored with\nthe following keys:\n\n* `at` The UTC timestamp when the error occurred at\n* `attempt` The attempt number when the error occurred\n* `error` A formatted error message and stacktrace\n\nSee the [Instrumentation](#instrumentation-error-reporting-and-logging) docs for an example of\nintegrating with external error reporting systems.","ref":"Oban.html#module-error-details"},{"type":"module","title":"Limiting Retries - Oban","doc":"By default, jobs are retried up to 20 times. The number of retries is controlled\nby the `max_attempts` value, which can be set at the Worker or Job level. For\nexample, to instruct a worker to discard jobs after three failures:\n\n```elixir\nuse Oban.Worker, queue: :limited, max_attempts: 3\n```","ref":"Oban.html#module-limiting-retries"},{"type":"module","title":"Limiting Execution Time - Oban","doc":"By default, individual jobs may execute indefinitely. If this is undesirable you\nmay define a timeout in milliseconds with the `timeout/1` callback on your\nworker module.\n\nFor example, to limit a worker's execution time to 30 seconds:\n\n```elixir\ndef MyApp.Worker do\n  use Oban.Worker\n\n  @impl Oban.Worker\n  def perform(_job) do\n    something_that_may_take_a_long_time()\n\n    :ok\n  end\n\n  @impl Oban.Worker\n  def timeout(_job), do: :timer.seconds(30)\nend\n```\n\nThe `timeout/1` function accepts an `Oban.Job` struct, so you can customize the\ntimeout using any job attributes.\n\nDefine the `timeout` value through job args:\n\n```elixir\ndef timeout(%_{args: %{\"timeout\" => timeout}}), do: timeout\n```\n\nDefine the `timeout` based on the number of attempts:\n\n```elixir\ndef timeout(%_{attempt: attempt}), do: attempt * :timer.seconds(5)\n```","ref":"Oban.html#module-limiting-execution-time"},{"type":"module","title":"Instrumentation, Error Reporting, and Logging - Oban","doc":"Oban provides integration with [Telemetry][tele], a dispatching library for\nmetrics. It is easy to report Oban metrics to any backend by attaching to\n`:oban` events.\n\nHere is an example of a sample unstructured log handler:\n\n```elixir\ndefmodule MyApp.ObanLogger do\n  require Logger\n\n  def handle_event([:oban, :job, :start], measure, meta, _) do\n    Logger.warning(\"[Oban] :started #{meta.worker} at #{measure.system_time}\")\n  end\n\n  def handle_event([:oban, :job, event], measure, meta, _) do\n    Logger.warning(\"[Oban] #{event} #{meta.worker} ran in #{measure.duration}\")\n  end\nend\n```\n\nAttach the handler to success and failure events in `application.ex`:\n\n```elixir\nevents = [[:oban, :job, :start], [:oban, :job, :stop], [:oban, :job, :exception]]\n\n:telemetry.attach_many(\"oban-logger\", events, &MyApp.ObanLogger.handle_event/4, [])\n```\n\nThe `Oban.Telemetry` module provides a robust structured logger that handles all\nof Oban's telemetry events. As in the example above, attach it within your\n`application.ex` module:\n\n```elixir\n:ok = Oban.Telemetry.attach_default_logger()\n```\n\nFor more details on the default structured logger and information on event\nmetadata see docs for the `Oban.Telemetry` module.","ref":"Oban.html#module-instrumentation-error-reporting-and-logging"},{"type":"module","title":"Reporting Errors - Oban","doc":"Another great use of execution data is error reporting. Here is an example of\nintegrating with [Honeybadger][honeybadger] to report job failures:\n\n```elixir\ndefmodule MyApp.ErrorReporter do\n  def attach do\n    :telemetry.attach(\n      \"oban-errors\",\n      [:oban, :job, :exception],\n      &__MODULE__.handle_event/4,\n      []\n    )\n  end\n\n  def handle_event([:oban, :job, :exception], measure, meta, _) do\n    Honeybadger.notify(meta.reason, stacktrace: meta.stacktrace)\n  end\nend\n\nMyApp.ErrorReporter.attach()\n```\n\nYou can use exception events to send error reports to Sentry, AppSignal, Honeybadger, Rollbar, or any other application monitoring platform.\n\nSome of these services support reporting Oban errors out of the box:\n\n  - Sentry — [Oban integration documentation](https://docs.sentry.io/platforms/elixir/integrations/oban)\n  - AppSignal - [Oban integration documentation](https://docs.appsignal.com/elixir/integrations/oban.html)\n\n[tele]: https://hexdocs.pm/telemetry\n[honeybadger]: https://www.honeybadger.io","ref":"Oban.html#module-reporting-errors"},{"type":"module","title":"Instance and Database Isolation - Oban","doc":"You can run multiple Oban instances with different prefixes on the same system\nand have them entirely isolated, provided you give each supervisor a distinct\nid.\n\nHere we configure our application to start three Oban supervisors using the\n\"public\", \"special\" and \"private\" prefixes, respectively:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    Repo,\n    Endpoint,\n    {Oban, name: ObanA, repo: Repo},\n    {Oban, name: ObanB, repo: Repo, prefix: \"special\"},\n    {Oban, name: ObanC, repo: Repo, prefix: \"private\"}\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\nend\n```","ref":"Oban.html#module-instance-and-database-isolation"},{"type":"module","title":"Umbrella Apps - Oban","doc":"If you need to run Oban from an umbrella application where more than one of\nthe child apps need to interact with Oban, you may need to set the `:name` for\neach child application that configures Oban.\n\nFor example, your umbrella contains two apps: `MyAppA` and `MyAppB`. `MyAppA` is\nresponsible for inserting jobs, while only `MyAppB` actually runs any queues.\n\nConfigure Oban with a custom name for `MyAppA`:\n\n```elixir\nconfig :my_app_a, Oban,\n  name: MyAppA.Oban,\n  repo: MyApp.Repo\n```\n\nThen configure Oban for `MyAppB` with a different name:\n\n```elixir\nconfig :my_app_b, Oban,\n  name: MyAppB.Oban,\n  repo: MyApp.Repo,\n  queues: [default: 10]\n```\n\nNow, use the configured name when calling functions like `Oban.insert/2`,\n`Oban.insert_all/2`, `Oban.drain_queue/2`, etc., to reference the correct Oban\nprocess for the current application.\n\n```elixir\nOban.insert(MyAppA.Oban, MyWorker.new(%{}))\nOban.insert_all(MyAppB.Oban, multi, :multiname, [MyWorker.new(%{})])\nOban.drain_queue(MyAppB.Oban, queue: :default)\n```","ref":"Oban.html#module-umbrella-apps"},{"type":"module","title":"Database Prefixes - Oban","doc":"Oban supports namespacing through PostgreSQL schemas, also called \"prefixes\" in\nEcto. With prefixes your jobs table can reside outside of your primary schema\n(usually public) and you can have multiple separate job tables.\n\nTo use a prefix you first have to specify it within your migration:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedObanJobsTable do\n  use Ecto.Migration\n\n  def up do\n    Oban.Migrations.up(prefix: \"private\")\n  end\n\n  def down do\n    Oban.Migrations.down(prefix: \"private\")\n  end\nend\n```\n\nThe migration will create the \"private\" schema and all tables, functions and\ntriggers within that schema. With the database migrated you'll then specify the\nprefix in your configuration:\n\n```elixir\nconfig :my_app, Oban,\n  prefix: \"private\",\n  repo: MyApp.Repo,\n  queues: [default: 10]\n```\n\nNow all jobs are inserted and executed using the `private.oban_jobs` table. Note\nthat `Oban.insert/2,4` will write jobs in the `private.oban_jobs` table, you'll\nneed to specify a prefix manually if you insert jobs directly through a repo.\n\nNot only is the `oban_jobs` table isolated within the schema, but all\nnotification events are also isolated. That means that insert/update events will\nonly dispatch new jobs for their prefix.","ref":"Oban.html#module-database-prefixes"},{"type":"module","title":"Dynamic Repositories - Oban","doc":"Oban supports [Ecto dynamic repositories][dynamic] through the\n`:get_dynamic_repo` option. To make this work, you need to run a separate Oban\ninstance per each dynamic repo instance. Most often it's worth bundling each\nOban and repo instance under the same supervisor:\n\n```elixir\ndef start_repo_and_oban(instance_id) do\n  children = [\n    {MyDynamicRepo, name: nil, url: repo_url(instance_id)},\n    {Oban, name: instance_id, get_dynamic_repo: fn -> repo_pid(instance_id) end}\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one)\nend\n```\n\nThe function `repo_pid/1` must return the pid of the repo for the given\ninstance. You can use `Registry` to register the repo (for example in the repo's\n`init/2` callback) and discover it.\n\nIf your application exclusively uses dynamic repositories and doesn't specify\nall credentials upfront, you must implement an `init/1` callback in your Ecto\nRepo. Doing so provides the Postgres notifier with the correct credentials on\ninit, allowing jobs to process as expected.\n\n[dynamic]: https://hexdocs.pm/ecto/replicas-and-dynamic-repositories.html#dynamic-repositories","ref":"Oban.html#module-dynamic-repositories"},{"type":"module","title":"Ecto Multi-tenancy - Oban","doc":"If you followed the Ecto guide on setting up multi-tenancy with foreign keys, you need to add an\nexception for queries originating from Oban. All of Oban's queries have the custom option `oban:\ntrue` to help you identify them in `prepare_query/3` or other instrumentation:\n\n```elixir\n# Sample code, only relevant if you followed the Ecto guide on multi tenancy with foreign keys.\ndefmodule MyApp.Repo do\n  use Ecto.Repo, otp_app: :my_app\n\n  require Ecto.Query\n\n  @impl true\n  def prepare_query(_operation, query, opts) do\n    cond do\n      opts[:skip_org_id] || opts[:schema_migration] || opts[:oban] ->\n        {query, opts}\n\n      org_id = opts[:org_id] ->\n        {Ecto.Query.where(query, org_id: ^org_id), opts}\n\n      true ->\n        raise \"expected org_id or skip_org_id to be set\"\n    end\n  end\nend\n```","ref":"Oban.html#module-ecto-multi-tenancy"},{"type":"macro","title":"Oban.__using__/1","doc":"Creates a facade for `Oban` functions and automates fetching configuration from the application\nenvironment.\n\nFacade modules support configuration via the application environment under an OTP application\nkey. For example, the facade:\n\n    defmodule MyApp.Oban do\n      use Oban, otp_app: MyApp\n    end\n\nCould be configured with:\n\n    config :my_app, Oban, repo: MyApp.Repo\n\nThen you can include `MyApp.Oban` in your application's supervision tree without passing extra\noptions:\n\n    defmodule MyApp.Application do\n      use Application\n\n      def start(_type, _args) do\n        children = [\n          MyApp.Repo,\n          MyApp.Oban\n        ]\n\n        opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n        Supervisor.start_link(children, opts)\n      end\n    end","ref":"Oban.html#__using__/1"},{"type":"macro","title":"Calling Functions - Oban.__using__/1","doc":"Facade modules allow you to call `Oban` functions on instances with custom names, e.g. not\n`Oban`, without passing a `t:Oban.name/0` as the first argument.\n\nFor example, rather than calling `Oban.config/1` you'd call `MyOban.config/0`:\n\n    MyOban.config()\n\nIt also makes piping into Oban functions far more convenient:\n\n    %{some: :args}\n    |> MyWorker.new()\n    |> MyOban.insert()","ref":"Oban.html#__using__/1-calling-functions"},{"type":"macro","title":"Merging Configuration - Oban.__using__/1","doc":"All configuration can be provided through the `use` macro or application config, and options\nfrom the application supersedes those passed through `use`. Configuration is prioritized in\norder:\n\n1. Options passed through `use`\n2. Options pulled from the OTP app via `Application.get_env/3`\n3. Options passed through a child spec in the supervisor","ref":"Oban.html#__using__/1-merging-configuration"},{"type":"function","title":"Oban.cancel_all_jobs/2","doc":"Cancel many jobs based on a queryable and mark them as `cancelled` to prevent them from running.\nAny currently `executing` jobs are killed while the others are ignored.\n\nIf executing jobs happen to fail before cancellation then the state is set to `cancelled`.\nHowever, any that complete successfully will remain `completed`.\n\nOnly jobs with the statuses `executing`, `available`, `scheduled`, or `retryable` can be cancelled.","ref":"Oban.html#cancel_all_jobs/2"},{"type":"function","title":"Example - Oban.cancel_all_jobs/2","doc":"Cancel all jobs:\n\n    Oban.cancel_all_jobs(Oban.Job)\n    {:ok, 9}\n\nCancel all jobs for a specific worker:\n\n    Oban.Job\n    |> Ecto.Query.where(worker: \"MyApp.MyWorker\")\n    |> Oban.cancel_all_jobs()\n    {:ok, 2}","ref":"Oban.html#cancel_all_jobs/2-example"},{"type":"function","title":"Oban.cancel_job/2","doc":"Cancel an `executing`, `available`, `scheduled` or `retryable` job and mark it as `cancelled` to\nprevent it from running. If the job is currently `executing` it will be killed and otherwise it\nis ignored.\n\nIf an executing job happens to fail before it can be cancelled the state is set to `cancelled`.\nHowever, if it manages to complete successfully then the state will still be `completed`.","ref":"Oban.html#cancel_job/2"},{"type":"function","title":"Example - Oban.cancel_job/2","doc":"Cancel a job:\n\n    Oban.cancel_job(job)\n    :ok","ref":"Oban.html#cancel_job/2-example"},{"type":"function","title":"Oban.check_queue/2","doc":"Check the current state of a queue producer.\n\nThis allows you to introspect on a queue's health by retrieving key attributes of the producer's\nstate; values such as the current `limit`, the `running` job ids, and when the producer was\nstarted.","ref":"Oban.html#check_queue/2"},{"type":"function","title":"Options - Oban.check_queue/2","doc":"* `:queue` - a string or atom specifying the queue to check, required","ref":"Oban.html#check_queue/2-options"},{"type":"function","title":"Example - Oban.check_queue/2","doc":"Oban.check_queue(queue: :default)\n    %{\n      limit: 10,\n      node: \"me@local\",\n      paused: false,\n      queue: \"default\",\n      running: [100, 102],\n      started_at: ~D[2020-10-07 15:31:00],\n      updated_at: ~D[2020-10-07 15:31:00]\n    }","ref":"Oban.html#check_queue/2-example"},{"type":"function","title":"Oban.config/1","doc":"Retrieve the `Oban.Config` struct for a named Oban supervision tree.","ref":"Oban.html#config/1"},{"type":"function","title":"Example - Oban.config/1","doc":"Retrieve the default `Oban` instance config:\n\n    %Oban.Config{} = Oban.config()\n\nRetrieve the config for an instance started with a custom name:\n\n    %Oban.Config{} = Oban.config(MyCustomOban)","ref":"Oban.html#config/1-example"},{"type":"function","title":"Oban.drain_queue/2","doc":"Synchronously execute all available jobs in a queue.\n\nAll execution happens within the current process and it is guaranteed not to raise an error or\nexit.\n\nDraining a queue from within the current process is especially useful for testing. Jobs that are\nenqueued by a process when `Ecto` is in sandbox mode are only visible to that process. Calling\n`drain_queue/2` allows you to control when the jobs are executed and to wait synchronously for\nall jobs to complete.","ref":"Oban.html#drain_queue/2"},{"type":"function","title":"Failures & Retries - Oban.drain_queue/2","doc":"Draining a queue uses the same execution mechanism as regular job dispatch. That means that any\njob failures or crashes are captured and result in a retry. Retries are scheduled in the future\nwith backoff and won't be retried immediately.\n\nBy default jobs are executed in `safe` mode, just as they are in production. Safe mode catches\nany errors or exits and records the formatted error in the job's `errors` array.  That means\nexceptions and crashes are _not_ bubbled up to the calling process.\n\nIf you expect jobs to fail, would like to track failures, or need to check for specific errors\nyou can pass the `with_safety: false` flag. See the \"Options\" section below for more details.","ref":"Oban.html#drain_queue/2-failures-retries"},{"type":"function","title":"Scheduled Jobs - Oban.drain_queue/2","doc":"By default, `drain_queue/2` will execute all currently available jobs. In order to execute\nscheduled jobs, you may pass the `with_scheduled: true` which will cause all scheduled jobs to\nbe marked as `available` beforehand. To run jobs scheduled up to a specific point in time, pass\na `DateTime` instead.","ref":"Oban.html#drain_queue/2-scheduled-jobs"},{"type":"function","title":"Options - Oban.drain_queue/2","doc":"* `:queue` - a string or atom specifying the queue to drain, required\n\n* `:with_limit` — the maximum number of jobs to drain at once. When recursion is enabled this is\n  how many jobs are processed per-iteration.\n\n* `:with_recursion` — whether to keep draining a queue repeatedly when jobs insert _more_ jobs\n\n* `:with_safety` — whether to silently catch errors when draining, defaults to `true`. When\n  `false`, raised exceptions or unhandled exits are reraised (unhandled exits are wrapped in\n  `Oban.CrashError`).\n\n* `:with_scheduled` — whether to include any scheduled jobs when draining, default `false`.\n   When `true`, drains all scheduled jobs. When a `DateTime` is provided, drains all jobs\n   scheduled up to, and including, that point in time.","ref":"Oban.html#drain_queue/2-options"},{"type":"function","title":"Example - Oban.drain_queue/2","doc":"Drain a queue with three available jobs, two of which succeed and one of which fails:\n\n    Oban.drain_queue(queue: :default)\n    %{failure: 1, snoozed: 0, success: 2}\n\nDrain a queue including any scheduled jobs:\n\n    Oban.drain_queue(queue: :default, with_scheduled: true)\n    %{failure: 0, snoozed: 0, success: 1}\n\nDrain a queue including jobs scheduled up to a minute:\n\n    Oban.drain_queue(queue: :default, with_scheduled: DateTime.add(DateTime.utc_now(), 60, :second))\n\nDrain a queue and assert an error is raised:\n\n    assert_raise RuntimeError, fn -> Oban.drain_queue(queue: :risky, with_safety: false) end\n\nDrain a queue repeatedly until there aren't any more jobs to run. This is particularly useful\nfor testing jobs that enqueue other jobs:\n\n    Oban.drain_queue(queue: :default, with_recursion: true)\n    %{failure: 1, snoozed: 0, success: 2}\n\nDrain only the top (by scheduled time and priority) five jobs off a queue:\n\n    Oban.drain_queue(queue: :default, with_limit: 5)\n    %{failure: 0, snoozed: 0, success: 1}\n\nDrain a queue recursively, only one job at a time:\n\n    Oban.drain_queue(queue: :default, with_limit: 1, with_recursion: true)\n    %{failure: 0, snoozed: 0, success: 3}","ref":"Oban.html#drain_queue/2-example"},{"type":"function","title":"Oban.insert/3","doc":"Insert a new job into the database for execution.\n\nThis and the other `insert` variants are the recommended way to enqueue jobs because they\nsupport features like unique jobs.\n\nSee the section on \"Unique Jobs\" for more details.","ref":"Oban.html#insert/3"},{"type":"function","title":"Example - Oban.insert/3","doc":"Insert a single job:\n\n    {:ok, job} = Oban.insert(MyApp.Worker.new(%{id: 1}))\n\nInsert a job while ensuring that it is unique within the past 30 seconds:\n\n    {:ok, job} = Oban.insert(MyApp.Worker.new(%{id: 1}, unique: [period: 30]))\n\nInsert a job using a custom timeout:\n\n    {:ok, job} = Oban.insert(MyApp.Worker.new(%{id: 1}), timeout: 10_000)\n\nInsert a job using an alternative instance name:\n\n    {:ok, job} = Oban.insert(MyOban, MyApp.Worker.new(%{id: 1}))","ref":"Oban.html#insert/3-example"},{"type":"function","title":"Oban.insert/5","doc":"Put a job insert operation into an `Ecto.Multi`.\n\nLike `insert/2`, this variant is recommended over `Ecto.Multi.insert` because it supports all of\nOban's features, i.e. unique jobs.\n\nSee the section on \"Unique Jobs\" for more details.","ref":"Oban.html#insert/5"},{"type":"function","title":"Example - Oban.insert/5","doc":"Ecto.Multi.new()\n    |> Oban.insert(\"job-1\", MyApp.Worker.new(%{id: 1}))\n    |> Oban.insert(\"job-2\", fn _ -> MyApp.Worker.new(%{id: 2}) end)\n    |> MyApp.Repo.transaction()","ref":"Oban.html#insert/5-example"},{"type":"function","title":"Oban.insert!/3","doc":"Similar to `insert/3`, but raises an `Ecto.InvalidChangesetError` if the job can't be inserted.","ref":"Oban.html#insert!/3"},{"type":"function","title":"Example - Oban.insert!/3","doc":"Insert a single job:\n\n    job = Oban.insert!(MyApp.Worker.new(%{id: 1}))\n\nInsert a job using a custom timeout:\n\n    job = Oban.insert!(MyApp.Worker.new(%{id: 1}), timeout: 10_000)\n\nInsert a job using an alternative instance name:\n\n    job = Oban.insert!(MyOban, MyApp.Worker.new(%{id: 1}))","ref":"Oban.html#insert!/3-example"},{"type":"function","title":"Oban.insert_all/3","doc":"Insert multiple jobs into the database for execution.\n\nThere are a few important differences between this function and `c:Ecto.Repo.insert_all/3`:\n\n1. This function always returns a list rather than a tuple of `{count, records}`\n2. This function accepts a list of changesets rather than a list of maps or keyword lists\n\n#### Error Handling and Rollbacks\n\nIf `insert_all` encounters an issue, the function will raise an error based on your database\nadapter. This behavior is valuable in conjunction with `c:Ecto.Repo.transaction/2` because it\nallows for rollbacks.\n\nFor example, an invalid changeset raises:\n\n`* (Ecto.InvalidChangesetError) could not perform insert because changeset is invalid.`\n\n> #### 🌟 Unique Jobs and Batching {: .warning}\n>\n> Only the [Smart Engine](https://getoban.pro/docs/pro/Oban.Pro.Engines.Smart.html) in [Oban\n> Pro](https://getoban.pro) supports bulk unique jobs and automatic batching. With the basic\n> engine, you must use `insert/3` for unique support.","ref":"Oban.html#insert_all/3"},{"type":"function","title":"Options - Oban.insert_all/3","doc":"Accepts any of Ecto's \"Shared Options\" such as `timeout` and `log`.","ref":"Oban.html#insert_all/3-options"},{"type":"function","title":"Example - Oban.insert_all/3","doc":"Insert a list of 100 jobs at once:\n\n    1..100\n    |> Enum.map(&MyApp.Worker.new(%{id: &1}))\n    |> Oban.insert_all()\n\nInsert a stream of jobs at once (be sure the stream terminates!):\n\n    (fn -> MyApp.Worker.new(%{}))\n    |> Stream.repeatedly()\n    |> Stream.take(100)\n    |> Oban.insert_all()\n\nInsert with a custom timeout:\n\n    1..100\n    |> Enum.map(&MyApp.Worker.new(%{id: &1}))\n    |> Oban.insert_all(timeout: 10_000)\n\nInsert with an alternative instance name:\n\n    changesets = Enum.map(1..100, &MyApp.Worker.new(%{id: &1}))\n    jobs = Oban.insert_all(MyOban, changesets)","ref":"Oban.html#insert_all/3-example"},{"type":"function","title":"Oban.insert_all/5","doc":"Put an `insert_all` operation into an `Ecto.Multi`.\n\nThis function supports the same features and has the same caveats as `insert_all/2`.","ref":"Oban.html#insert_all/5"},{"type":"function","title":"Example - Oban.insert_all/5","doc":"Insert job changesets within a multi:\n\n    changesets = Enum.map(0..100, &MyApp.Worker.new(%{id: &1}))\n\n    Ecto.Multi.new()\n    |> Oban.insert_all(:jobs, changesets)\n    |> MyApp.Repo.transaction()\n\nInsert job changesets using a function:\n\n    Ecto.Multi.new()\n    |> Ecto.Multi.insert(:user, user_changeset)\n    |> Oban.insert_all(:jobs, fn %{user: user} ->\n      email_job = EmailWorker.new(%{id: user.id})\n      staff_job = StaffWorker.new(%{id: user.id})\n      [email_job, staff_job]\n    end)\n    |> MyApp.Repo.transaction()","ref":"Oban.html#insert_all/5-example"},{"type":"function","title":"Oban.pause_all_queues/2","doc":"Pause all running queues to prevent them from executing any new jobs.\n\nSee `pause_queue/2` for options and details.","ref":"Oban.html#pause_all_queues/2"},{"type":"function","title":"Example - Oban.pause_all_queues/2","doc":"Pause all queues:\n\n    Oban.pause_all_queues()\n\nPause all queues on the local node:\n\n    Oban.pause_all_queues(local_only: true)\n\nPause all queues on a specific node:\n\n    Oban.pause_all_queues(node: \"worker.1\")\n\nPause all queues for an alternative instance name:\n\n    Oban.pause_all_queues(MyOban)","ref":"Oban.html#pause_all_queues/2-example"},{"type":"function","title":"Oban.pause_queue/2","doc":"Pause a running queue, preventing it from executing any new jobs. All running jobs will remain\nrunning until they are finished.\n\nWhen shutdown begins all queues are paused.","ref":"Oban.html#pause_queue/2"},{"type":"function","title":"Options - Oban.pause_queue/2","doc":"* `:queue` - a string or atom specifying the queue to pause, required\n\n* `:local_only` - whether the queue will be paused only on the local node, default: `false`\n\n* `:node` - restrict pausing to a particular node\n\nNote: by default, Oban does not verify that the given queue exists unless `:local_only`\nis set to `true` as even if the queue does not exist locally, it might be running on\nanother node.","ref":"Oban.html#pause_queue/2-options"},{"type":"function","title":"Example - Oban.pause_queue/2","doc":"Pause the default queue:\n\n    Oban.pause_queue(queue: :default)\n    :ok\n\nPause the default queue, but only on the local node:\n\n    Oban.pause_queue(queue: :default, local_only: true)\n    :ok\n\nPause the default queue only on a particular node:\n\n    Oban.pause_queue(queue: :default, node: \"worker.1\")\n    :ok","ref":"Oban.html#pause_queue/2-example"},{"type":"function","title":"Oban.resume_all_queues/2","doc":"Resume executing jobs in all paused queues.\n\nSee `resume_queue/2` for options and details.","ref":"Oban.html#resume_all_queues/2"},{"type":"function","title":"Example - Oban.resume_all_queues/2","doc":"Resume all queues:\n\n    Oban.resume_all_queues()\n\nResume all queues on the local node:\n\n    Oban.resume_all_queues(local_only: true)\n\nResume all queues on a specific node:\n\n    Oban.resume_all_queues(node: \"worker.1\")\n\nResume all queues for an alternative instance name:\n\n    Oban.resume_all_queues(MyOban)","ref":"Oban.html#resume_all_queues/2-example"},{"type":"function","title":"Oban.resume_queue/2","doc":"Resume executing jobs in a paused queue.","ref":"Oban.html#resume_queue/2"},{"type":"function","title":"Options - Oban.resume_queue/2","doc":"* `:queue` - a string or atom specifying the queue to resume, required\n\n* `:local_only` - whether the queue will be resumed only on the local node, default: `false`\n\n* `:node` - restrict resuming to a particular node\n\nNote: by default, Oban does not verify that the given queue exists unless `:local_only`\nis set to `true` as even if the queue does not exist locally, it might be running on\nanother node.","ref":"Oban.html#resume_queue/2-options"},{"type":"function","title":"Example - Oban.resume_queue/2","doc":"Resume a paused default queue:\n\n    Oban.resume_queue(queue: :default)\n\nResume the default queue, but only on the local node:\n\n    Oban.resume_queue(queue: :default, local_only: true)\n\nResume the default queue only on a particular node:\n\n    Oban.resume_queue(queue: :default, node: \"worker.1\")","ref":"Oban.html#resume_queue/2-example"},{"type":"function","title":"Oban.retry_all_jobs/2","doc":"Retries all jobs that match on the given queryable.\n\nIf no queryable is given, Oban will retry all jobs that aren't currently `available` or\n`executing`. Note that regardless of constraints, it will never retry `available` or\n`executing` jobs.","ref":"Oban.html#retry_all_jobs/2"},{"type":"function","title":"Example - Oban.retry_all_jobs/2","doc":"Retries jobs in _any state_ other than `available` or `executing`:\n\n    Oban.retry_all_jobs(Oban.Job)\n    {:ok, 9}\n\nRetries jobs with the `retryable` state:\n\n    Oban.Job\n    |> Ecto.Query.where(state: \"retryable\")\n    |> Oban.retry_all_jobs()\n    {:ok, 3}\n\nRetries all inactive jobs with priority 0\n\n    Oban.Job\n    |> Ecto.Query.where(priority: 0)\n    |> Oban.retry_all_jobs()\n    {:ok, 5}","ref":"Oban.html#retry_all_jobs/2-example"},{"type":"function","title":"Oban.retry_job/2","doc":"Sets a job as `available`, adding attempts if already maxed out. Jobs currently `available`,\n`executing` or `scheduled` are ignored. The job is scheduled for immediate execution.","ref":"Oban.html#retry_job/2"},{"type":"function","title":"Example - Oban.retry_job/2","doc":"Retry a job:\n\n    Oban.retry_job(job)\n    :ok","ref":"Oban.html#retry_job/2-example"},{"type":"function","title":"Oban.scale_queue/2","doc":"Scale the concurrency for a queue.","ref":"Oban.html#scale_queue/2"},{"type":"function","title":"Options - Oban.scale_queue/2","doc":"* `:queue` - a string or atom specifying the queue to scale, required\n\n* `:limit` — the new concurrency limit, required\n\n* `:local_only` — whether the queue will be scaled only on the local node, default: `false`\n\n* `:node` - restrict scaling to a particular node\n\nIn addition, all engine-specific queue options are passed along after validation.\n\nNote: by default, Oban does not verify that the given queue exists unless `:local_only`\nis set to `true` as even if the queue does not exist locally, it might be running on\nanother node.","ref":"Oban.html#scale_queue/2-options"},{"type":"function","title":"Example - Oban.scale_queue/2","doc":"Scale a queue up, triggering immediate execution of queued jobs:\n\n    Oban.scale_queue(queue: :default, limit: 50)\n    :ok\n\nScale the queue back down, allowing executing jobs to finish:\n\n    Oban.scale_queue(queue: :default, limit: 5)\n    :ok\n\nScale the queue only on the local node:\n\n    Oban.scale_queue(queue: :default, limit: 10, local_only: true)\n    :ok\n\nScale the queue on a particular node:\n\n    Oban.scale_queue(queue: :default, limit: 10, node: \"worker.1\")\n    :ok","ref":"Oban.html#scale_queue/2-example"},{"type":"function","title":"Oban.start_link/1","doc":"Starts an `Oban` supervision tree linked to the current process.","ref":"Oban.html#start_link/1"},{"type":"function","title":"Options - Oban.start_link/1","doc":"These options are required; without them the supervisor won't start:\n\n* `:repo` — specifies the Ecto repo used to insert and retrieve jobs","ref":"Oban.html#start_link/1-options"},{"type":"function","title":"Primary Options - Oban.start_link/1","doc":"These options determine what the system does at a high level, i.e. which queues to run:\n\n* `:engine` — facilitates inserting, fetching, and otherwise managing jobs.\n\n  There are three built-in engines: `Oban.Engines.Basic` for Postgres databases,\n  `Oban.Engines.Lite` for SQLite3 databases, and `Oban.Engines.Inline` for simplified testing\n  (only available for `:inline` testing mode).\n\n  When the `Oban.Engines.Lite` engine is used the `:notifier` and `:peer` are automatically set\n  to `PG` and `isolated` mode, respectively.\n\n  Additional engines, such as Oban Pro's `SmartEngine` with advanced functionality for Postgres,\n  are also available as an add-on.\n\n  Defaults to the `Basic` engine for Postgres.\n\n* `:log` — either `false` to disable logging or a standard log level (`:error`, `:warning`,\n  `:info`, `:debug`, etc.). This determines whether queries are logged or not; overriding the\n  repo's configured log level. Defaults to `false`, where no queries are logged.\n\n* `:name` — used for supervisor registration, it must be unique across an entire VM instance.\n  Defaults to `Oban` when no name is provided.\n\n* `:node` — used to identify the node that the supervision tree is running in. If no value is\n  provided it will use the `node` name in a distributed system, or the `hostname` in an isolated\n  node. See \"Node Name\" below.\n\n* `:notifier` — used to relay messages between processes, nodes, and the Postgres database.\n\n  There are two built-in notifiers: `Oban.Notifiers.Postgres`, which uses Postgres PubSub; and\n  `Oban.Notifiers.PG`, which uses process groups with distributed erlang. Defaults to the\n  Postgres notifier.\n\n* `:peer` — used to specify which peer module to use for cluster leadership.\n\n  There are two built-in peers: `Oban.Peers.Postgres`, which uses table-based leadership through\n  the `oban_peers` table; and `Oban.Peers.Global`, which uses global locks through distributed Erlang.\n\n  Leadership can be disabled by setting `peer: false`, but note that centralized plugins like\n  `Cron` won't run without leadership.\n\n  Defaults to the `Postgres` peer.\n\n* `:plugins` — a list or modules or module/option tuples that are started as children of an Oban\n  supervisor. Any supervisable module is a valid plugin, i.e. a `GenServer` or an `Agent`.\n\n* `:prefix` — the query prefix, or schema, to use for inserting and executing jobs. An\n  `oban_jobs` table must exist within the prefix. See the \"Prefix Support\" section in the module\n  documentation for more details.\n\n* `:queues` — a keyword list where the keys are queue names and the values are the concurrency\n  setting or a keyword list of queue options. For example, setting queues to `[default: 10,\n  exports: 5]` would start the queues `default` and `exports` with a combined concurrency level\n  of 15. The concurrency setting specifies how many jobs _each queue_ will run concurrently.\n\n  Queues accept additional override options to customize their behavior, e.g. by setting\n  `paused` or `dispatch_cooldown` for a specific queue.\n\n* `:testing` — a mode that controls how an instance is configured for testing. When set to\n  `:inline` or `:manual` queues, peers, and plugins are automatically disabled. Defaults to\n  `:disabled`, no test mode.","ref":"Oban.html#start_link/1-primary-options"},{"type":"function","title":"Twiddly Options - Oban.start_link/1","doc":"Additional options used to tune system behaviour. These are primarily useful for testing or\ntroubleshooting and don't usually need modification.\n\n* `:dispatch_cooldown` — the minimum number of milliseconds a producer will wait before fetching\n  and running more jobs. A slight cooldown period prevents a producer from flooding with\n  messages and thrashing the database. The cooldown period _directly impacts_ a producer's\n  throughput: jobs per second for a single queue is calculated by `(1000 / cooldown) * limit`.\n  For example, with a `5ms` cooldown and a queue limit of `25` a single queue can run 5,000\n  jobs/sec.\n\n  The default is `5ms` and the minimum is `1ms`, which is likely faster than the database can\n  return new jobs to run.\n\n* `:insert_trigger` — whether to dispatch notifications to relevant queues as jobs are inserted\n  into the database. At high load, e.g. thousands or more job inserts per second, notifications\n  may become a bottleneck.\n\n  The trigger mechanism is designed to make jobs execute immediately after insert, rather than\n  up to `:stage_interval` (1 second) afterwards, and it can safely be disabled to improve insert\n  throughput.\n\n  Defaults to `true`, with triggering enabled.\n\n* `:shutdown_grace_period` — the amount of time a queue will wait for executing jobs to complete\n  before hard shutdown, specified in milliseconds. The default is `15_000`, or 15 seconds.\n\n* `:stage_interval` — the number of milliseconds between making scheduled jobs available and\n  notifying relevant queues that jobs are available. This is directly tied to the resolution of\n  `scheduled` or `retryable` jobs and how frequently the database is checked for jobs to run. To\n  minimize database load, only `5_000` jobs are staged at each interval.\n\n  Only the leader node stages jobs and notifies queues when the `:notifier's` pubsub\n  notifications are functional. If pubusb messages can't get through then staging switches to a\n  less efficient \"local\" mode in which all nodes poll for jobs to run.\n\n  Setting the interval to `:infinity` disables staging entirely. The default is `1_000ms`.","ref":"Oban.html#start_link/1-twiddly-options"},{"type":"function","title":"Example - Oban.start_link/1","doc":"Start a stand-alone `Oban` instance:\n\n    {:ok, pid} = Oban.start_link(repo: MyApp.Repo, queues: [default: 10])\n\nTo start an `Oban` instance within an application's supervision tree:\n\n    def start(_type, _args) do\n      children = [MyApp.Repo, {Oban, repo: MyApp.Repo, queues: [default: 10]}]\n\n      Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n    end\n\nStart multiple, named `Oban` supervisors within a supervision tree:\n\n      children = [\n        MyApp.Repo,\n        {Oban, name: Oban.A, repo: MyApp.Repo, queues: [default: 10]},\n        {Oban, name: Oban.B, repo: MyApp.Repo, queues: [special: 10]},\n      ]\n\n      Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n\nStart a local `Oban` instance for SQLite:\n\n    {:ok, pid} = Oban.start_link(engine: Oban.Engines.Lite, repo: MyApp.Repo)","ref":"Oban.html#start_link/1-example"},{"type":"function","title":"Node Name - Oban.start_link/1","doc":"When the `node` value hasn't been configured it is generated based on the environment:\n\n* In a distributed system the node name is used\n* In a Heroku environment the system environment's `DYNO` value is used\n* Otherwise, the system hostname is used","ref":"Oban.html#start_link/1-node-name"},{"type":"function","title":"Oban.start_queue/2","doc":"Start a new supervised queue.\n\nBy default this starts a new supervised queue across all nodes running Oban on the same database\nand prefix. You can pass the option `local_only: true` if you prefer to start the queue only on\nthe local node.","ref":"Oban.html#start_queue/2"},{"type":"function","title":"Options - Oban.start_queue/2","doc":"* `:queue` - a string or atom specifying the queue to start, required\n* `:local_only` - whether the queue will be started only on the local node, default: `false`\n* `:limit` - set the concurrency limit, required\n* `:paused` — set whether the queue starts in the \"paused\" state, optional\n\nIn addition, all engine-specific queue options are passed along after validation.","ref":"Oban.html#start_queue/2-options"},{"type":"function","title":"Example - Oban.start_queue/2","doc":"Start the `:priority` queue with a concurrency limit of 10 across the connected nodes.\n\n    Oban.start_queue(queue: :priority, limit: 10)\n    :ok\n\nStart the `:media` queue with a concurrency limit of 5 only on the local node.\n\n    Oban.start_queue(queue: :media, limit: 5, local_only: true)\n    :ok\n\nStart the `:media` queue on a particular node.\n\n    Oban.start_queue(queue: :media, limit: 5, node: \"worker.1\")\n    :ok\n\nStart the `:media` queue in a `paused` state.\n\n    Oban.start_queue(queue: :media, limit: 5, paused: true)\n    :ok","ref":"Oban.html#start_queue/2-example"},{"type":"function","title":"Oban.stop_queue/2","doc":"Shutdown a queue's supervision tree and stop running jobs for that queue.\n\nBy default this action will occur across all the running nodes. Still, if you prefer to stop the\nqueue's supervision tree and stop running jobs for that queue only on the local node, you can\npass the option: `local_only: true`\n\nThe shutdown process pauses the queue first and allows current jobs to exit gracefully, provided\nthey finish within the shutdown limit.\n\nNote: by default, Oban does not verify that the given queue exists unless `:local_only`\nis set to `true` as even if the queue does not exist locally, it might be running on\nanother node.","ref":"Oban.html#stop_queue/2"},{"type":"function","title":"Options - Oban.stop_queue/2","doc":"* `:queue` - a string or atom specifying the queue to stop, required\n\n* `:local_only` - whether the queue will be stopped only on the local node, default: `false`\n\n* `:node` - restrict stopping to a particular node","ref":"Oban.html#stop_queue/2-options"},{"type":"function","title":"Example - Oban.stop_queue/2","doc":"Stop a running queue on all nodes:\n\n    Oban.stop_queue(queue: :default)\n    :ok\n\nStop the queue only on the local node:\n\n    Oban.stop_queue(queue: :media, local_only: true)\n    :ok\n\nStop the queue only on a particular node:\n\n    Oban.stop_queue(queue: :media, node: \"worker.1\")\n    :ok","ref":"Oban.html#stop_queue/2-example"},{"type":"function","title":"Oban.whereis/1","doc":"Returns the pid of the root Oban process for the given name.","ref":"Oban.html#whereis/1"},{"type":"function","title":"Example - Oban.whereis/1","doc":"Find the default instance:\n\n    Oban.whereis(Oban)\n\nFind a dynamically named instance:\n\n    Oban.whereis({:oban, 1})","ref":"Oban.html#whereis/1-example"},{"type":"type","title":"Oban.changeset_or_fun/0","doc":"","ref":"Oban.html#t:changeset_or_fun/0"},{"type":"type","title":"Oban.changeset_wrapper/0","doc":"","ref":"Oban.html#t:changeset_wrapper/0"},{"type":"type","title":"Oban.changesets_or_wrapper/0","doc":"","ref":"Oban.html#t:changesets_or_wrapper/0"},{"type":"type","title":"Oban.changesets_or_wrapper_or_fun/0","doc":"","ref":"Oban.html#t:changesets_or_wrapper_or_fun/0"},{"type":"type","title":"Oban.drain_option/0","doc":"","ref":"Oban.html#t:drain_option/0"},{"type":"type","title":"Oban.drain_result/0","doc":"","ref":"Oban.html#t:drain_result/0"},{"type":"type","title":"Oban.multi/0","doc":"","ref":"Oban.html#t:multi/0"},{"type":"type","title":"Oban.multi_name/0","doc":"","ref":"Oban.html#t:multi_name/0"},{"type":"type","title":"Oban.name/0","doc":"The name of an Oban instance. This is used to identify instances in the internal registry for\nconfiguration lookup.","ref":"Oban.html#t:name/0"},{"type":"type","title":"Oban.oban_node/0","doc":"","ref":"Oban.html#t:oban_node/0"},{"type":"type","title":"Oban.option/0","doc":"","ref":"Oban.html#t:option/0"},{"type":"type","title":"Oban.queue_all_option/0","doc":"","ref":"Oban.html#t:queue_all_option/0"},{"type":"type","title":"Oban.queue_name/0","doc":"","ref":"Oban.html#t:queue_name/0"},{"type":"type","title":"Oban.queue_option/0","doc":"","ref":"Oban.html#t:queue_option/0"},{"type":"type","title":"Oban.queue_state/0","doc":"","ref":"Oban.html#t:queue_state/0"},{"type":"module","title":"Oban.Job","doc":"A Job is an Ecto schema used for asynchronous execution.\n\nJob changesets are created by your application code and inserted into the database for\nasynchronous execution. Jobs can be inserted along with other application data as part of a\ntransaction, which guarantees that jobs will only be triggered from a successful transaction.","ref":"Oban.Job.html"},{"type":"function","title":"Oban.Job.format_attempt/1","doc":"Normalize, blame, and format a job's `unsaved_error` into the stored error format.\n\nFormatted errors are stored in a job's `errors` field.","ref":"Oban.Job.html#format_attempt/1"},{"type":"function","title":"Oban.Job.new/2","doc":"Construct a new job changeset ready for insertion into the database.","ref":"Oban.Job.html#new/2"},{"type":"function","title":"Options - Oban.Job.new/2","doc":"* `:max_attempts` — the maximum number of times a job can be retried if there are errors\n    during execution\n\n  * `:meta` — a map containing additional information about the job\n\n  * `:priority` — a numerical indicator from 0 to 9 of how important this job is relative to\n    other jobs in the same queue. The lower the number, the higher priority the job.\n\n  * `:queue` — a named queue to push the job into. Jobs may be pushed into any queue, regardless\n    of whether jobs are currently being processed for the queue.\n\n  * `:replace` - a list of keys to replace per state on a unique conflict\n\n  * `:scheduled_at` - a time in the future after which the job should be executed\n\n  * `:schedule_in` - the number of seconds until the job should be executed or a tuple containing\n    a number and unit\n\n  * `:tags` — a list of tags to group and organize related jobs, i.e. to identify scheduled jobs\n\n  * `:unique` — a keyword list of options specifying how uniqueness will be calculated. The\n    options define which fields will be used, for how long, with which keys, and for which states.\n\n  * `:worker` — a module to execute the job in. The module must implement the `Oban.Worker`\n    behaviour.","ref":"Oban.Job.html#new/2-options"},{"type":"function","title":"Examples - Oban.Job.new/2","doc":"Insert a job with the `:default` queue:\n\n    %{id: 1, user_id: 2}\n    |> Oban.Job.new(queue: :default, worker: MyApp.Worker)\n    |> Oban.insert()\n\nGenerate a pre-configured job for `MyApp.Worker`:\n\n    MyApp.Worker.new(%{id: 1, user_id: 2})\n\nSchedule a job to run in 5 seconds:\n\n    MyApp.Worker.new(%{id: 1}, schedule_in: 5)\n\nSchedule a job to run in 5 minutes:\n\n    MyApp.Worker.new(%{id: 1}, schedule_in: {5, :minutes})\n\nInsert a job, ensuring that it is unique within the past minute:\n\n    MyApp.Worker.new(%{id: 1}, unique: [period: {1, :minute}])\n\nInsert a unique job where the period is compared to the `scheduled_at` timestamp rather than\n`inserted_at`:\n\n    MyApp.Worker.new(%{id: 1}, unique: [period: 60, timestamp: :scheduled_at])\n\nInsert a unique job based only on the worker field, and within multiple states:\n\n    fields = [:worker]\n    states = [:available, :scheduled, :executing, :retryable, :completed]\n\n    MyApp.Worker.new(%{id: 1}, unique: [fields: fields, period: 60, states: states])\n\nInsert a unique job considering only the worker and specified keys in the args:\n\n    keys = [:account_id, :url]\n    args = %{account_id: 1, url: \"https://example.com\"}\n\n    MyApp.Worker.new(args, unique: [fields: [:args, :worker], keys: keys])\n\nInsert a unique job considering only specified keys in the meta:\n\n    unique = [fields: [:meta], keys: [:slug]]\n\n    MyApp.Worker.new(%{id: 1}, meta: %{slug: \"unique-key\"}, unique: unique)","ref":"Oban.Job.html#new/2-examples"},{"type":"function","title":"Oban.Job.states/0","doc":"A canonical list of all possible job states.\n\nThis may be used to build up `:unique` options without duplicating states in application code.","ref":"Oban.Job.html#states/0"},{"type":"function","title":"Examples - Oban.Job.states/0","doc":"iex> Oban.Job.states() -- [:completed, :discarded]\n    [:scheduled, :available, :executing, :retryable, :cancelled]","ref":"Oban.Job.html#states/0-examples"},{"type":"function","title":"Job State Transitions - Oban.Job.states/0","doc":"* `:scheduled`—Jobs inserted with `scheduled_at` in the future are `:scheduled`. After the\n  `scheduled_at` time has ellapsed the `Oban.Plugins.Stager` will transition them to `:available`\n\n* `:available`—Jobs without a future `scheduled_at` timestamp are inserted as `:available` and may\n  execute immediately\n\n* `:executing`—Available jobs may be ran, at which point they are `:executing`\n\n* `:retryable`—Jobs that fail and haven't exceeded their max attempts are transitiond to\n  `:retryable` and rescheduled until after a backoff period. Once the backoff has ellapsed the\n  `Oban.Plugins.Stager` will transition them back to `:available`\n\n* `:completed`—Jobs that finish executing succesfully are marked `:completed`\n\n* `:discarded`—Jobs that fail and exhaust their max attempts, or return a `:discard` tuple during\n  execution, are marked `:discarded`\n\n* `:cancelled`—Jobs that are cancelled intentionally","ref":"Oban.Job.html#states/0-job-state-transitions"},{"type":"function","title":"Oban.Job.to_map/1","doc":"Convert a Job changeset into a map suitable for database insertion.","ref":"Oban.Job.html#to_map/1"},{"type":"function","title":"Examples - Oban.Job.to_map/1","doc":"Convert a worker generated changeset into a plain map:\n\n    %{id: 123}\n    |> MyApp.Worker.new()\n    |> Oban.Job.to_map()","ref":"Oban.Job.html#to_map/1-examples"},{"type":"type","title":"Oban.Job.args/0","doc":"","ref":"Oban.Job.html#t:args/0"},{"type":"type","title":"Oban.Job.changeset/0","doc":"","ref":"Oban.Job.html#t:changeset/0"},{"type":"type","title":"Oban.Job.changeset_fun/0","doc":"","ref":"Oban.Job.html#t:changeset_fun/0"},{"type":"type","title":"Oban.Job.changeset_list/0","doc":"","ref":"Oban.Job.html#t:changeset_list/0"},{"type":"type","title":"Oban.Job.changeset_list_fun/0","doc":"","ref":"Oban.Job.html#t:changeset_list_fun/0"},{"type":"type","title":"Oban.Job.errors/0","doc":"","ref":"Oban.Job.html#t:errors/0"},{"type":"type","title":"Oban.Job.option/0","doc":"","ref":"Oban.Job.html#t:option/0"},{"type":"type","title":"Oban.Job.replace_by_state_option/0","doc":"","ref":"Oban.Job.html#t:replace_by_state_option/0"},{"type":"type","title":"Oban.Job.replace_option/0","doc":"","ref":"Oban.Job.html#t:replace_option/0"},{"type":"type","title":"Oban.Job.schedule_in_option/0","doc":"","ref":"Oban.Job.html#t:schedule_in_option/0"},{"type":"type","title":"Oban.Job.t/0","doc":"","ref":"Oban.Job.html#t:t/0"},{"type":"type","title":"Oban.Job.tags/0","doc":"","ref":"Oban.Job.html#t:tags/0"},{"type":"type","title":"Oban.Job.time_unit/0","doc":"","ref":"Oban.Job.html#t:time_unit/0"},{"type":"type","title":"Oban.Job.unique_field/0","doc":"","ref":"Oban.Job.html#t:unique_field/0"},{"type":"type","title":"Oban.Job.unique_option/0","doc":"","ref":"Oban.Job.html#t:unique_option/0"},{"type":"type","title":"Oban.Job.unique_period/0","doc":"","ref":"Oban.Job.html#t:unique_period/0"},{"type":"type","title":"Oban.Job.unique_state/0","doc":"","ref":"Oban.Job.html#t:unique_state/0"},{"type":"behaviour","title":"Oban.Migration","doc":"Migrations create and modify the database tables Oban needs to function.","ref":"Oban.Migration.html"},{"type":"behaviour","title":"Usage - Oban.Migration","doc":"To use migrations in your application you'll need to generate an `Ecto.Migration` that wraps\ncalls to `Oban.Migration`:\n\n```bash\nmix ecto.gen.migration add_oban\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`Oban.Migration`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddOban do\n  use Ecto.Migration\n\n  def up, do: Oban.Migrations.up()\n\n  def down, do: Oban.Migrations.down()\nend\n```\n\nThis will run all of Oban's versioned migrations for your database.\n\nNow, run the migration to create the table:\n\n```bash\nmix ecto.migrate\n```\n\nMigrations between versions are idempotent. As new versions are released, you may need to run\nadditional migrations. To do this, generate a new migration:\n\n```bash\nmix ecto.gen.migration upgrade_oban_to_v11\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`Oban.Migration`, passing a version number:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.UpgradeObanToV11 do\n  use Ecto.Migration\n\n  def up, do: Oban.Migrations.up(version: 11)\n\n  def down, do: Oban.Migrations.down(version: 11)\nend\n```","ref":"Oban.Migration.html#module-usage"},{"type":"behaviour","title":"Isolation with Prefixes - Oban.Migration","doc":"Oban supports namespacing through PostgreSQL schemas, also called \"prefixes\" in Ecto. With\nprefixes your jobs table can reside outside of your primary schema (usually public) and you can\nhave multiple separate job tables.\n\nTo use a prefix you first have to specify it within your migration:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedObanJobsTable do\n  use Ecto.Migration\n\n  def up, do: Oban.Migrations.up(prefix: \"private\")\n\n  def down, do: Oban.Migrations.down(prefix: \"private\")\nend\n```\n\nThe migration will create the \"private\" schema and all tables, functions and triggers within\nthat schema. With the database migrated you'll then specify the prefix in your configuration:\n\n```elixir\nconfig :my_app, Oban,\n  prefix: \"private\",\n  ...\n```\n\nIn some cases, for example if your \"private\" schema already exists and your database user in\nproduction doesn't have permissions to create a new schema, trying to create the schema from the\nmigration will result in an error. In such situations, it may be useful to inhibit the creation\nof the \"private\" schema:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddPrefixedObanJobsTable do\n  use Ecto.Migration\n\n  def up, do: Oban.Migrations.up(prefix: \"private\", create_schema: false)\n\n  def down, do: Oban.Migrations.down(prefix: \"private\")\nend\n```","ref":"Oban.Migration.html#module-isolation-with-prefixes"},{"type":"behaviour","title":"Migrating Without Ecto - Oban.Migration","doc":"If your application uses something other than Ecto for migrations, be it an external system or\nanother ORM, it may be helpful to create plain SQL migrations for Oban database schema changes.\n\nThe simplest mechanism for obtaining the SQL changes is to create the migration locally and run\n`mix ecto.migrate --log-migrations-sql`. That will log all of the generated SQL, which you can\nthen paste into your migration system of choice.\n\nAlternatively, if you'd like a more automated approach, try using the [oban_migations_sql][sql]\nproject to generate `up` and `down` SQL migrations for you.\n\n[sql]: https://github.com/btwb/oban_migrations_sql","ref":"Oban.Migration.html#module-migrating-without-ecto"},{"type":"function","title":"Oban.Migration.down/1","doc":"Run the `down` changes for all migrations between the current version and the initial version.","ref":"Oban.Migration.html#down/1"},{"type":"function","title":"Example - Oban.Migration.down/1","doc":"Run all migrations from current version down to the first:\n\n    Oban.Migration.down()\n\nRun migrations down to and including a specified version:\n\n    Oban.Migration.down(version: 5)\n\nRun migrations in an alternate prefix:\n\n    Oban.Migration.down(prefix: \"payments\")","ref":"Oban.Migration.html#down/1-example"},{"type":"callback","title":"Oban.Migration.down/1","doc":"Migrates storage down to the previous version.","ref":"Oban.Migration.html#c:down/1"},{"type":"function","title":"Oban.Migration.migrated_version/1","doc":"Check the latest version the database is migrated to.","ref":"Oban.Migration.html#migrated_version/1"},{"type":"function","title":"Example - Oban.Migration.migrated_version/1","doc":"Oban.Migration.migrated_version()","ref":"Oban.Migration.html#migrated_version/1-example"},{"type":"callback","title":"Oban.Migration.migrated_version/1","doc":"Identifies the last migrated version.","ref":"Oban.Migration.html#c:migrated_version/1"},{"type":"function","title":"Oban.Migration.up/1","doc":"Run the `up` changes for all migrations between the initial version and the current version.","ref":"Oban.Migration.html#up/1"},{"type":"function","title":"Example - Oban.Migration.up/1","doc":"Run all migrations up to the current version:\n\n    Oban.Migration.up()\n\nRun migrations up to a specified version:\n\n    Oban.Migration.up(version: 2)\n\nRun migrations in an alternate prefix:\n\n    Oban.Migration.up(prefix: \"payments\")\n\nRun migrations in an alternate prefix but don't try to create the schema:\n\n    Oban.Migration.up(prefix: \"payments\", create_schema: false)","ref":"Oban.Migration.html#up/1-example"},{"type":"callback","title":"Oban.Migration.up/1","doc":"Migrates storage up to the latest version.","ref":"Oban.Migration.html#c:up/1"},{"type":"module","title":"Oban.Telemetry","doc":"Telemetry integration for event metrics, logging and error reporting.","ref":"Oban.Telemetry.html"},{"type":"module","title":"Initialization Events - Oban.Telemetry","doc":"Oban emits the following telemetry event when an Oban supervisor is started:\n\n* `[:oban, :supervisor, :init]` - when the Oban supervisor is started this will execute\n\nThe initialization event contains the following measurements:\n\n* `:system_time` - The system's time when Oban was started\n\nThe initialization event contains the following metadata:\n\n* `:conf` - The configuration used for the Oban supervisor instance\n* `:pid` - The PID of the supervisor instance","ref":"Oban.Telemetry.html#module-initialization-events"},{"type":"module","title":"Job Events - Oban.Telemetry","doc":"Oban emits the following telemetry events for each job:\n\n* `[:oban, :job, :start]` — at the point a job is fetched from the database and will execute\n* `[:oban, :job, :stop]` — after a job succeeds and the success is recorded in the database\n* `[:oban, :job, :exception]` — after a job fails and the failure is recorded in the database\n\nAll job events share the same details about the job that was executed. In addition, failed jobs\nprovide the error type, the error itself, and the stacktrace. The following chart shows which\nmetadata you can expect for each event:\n\n| event        | measures                   | metadata                                                                |\n| ------------ | -------------------------- | ----------------------------------------------------------------------- |\n| `:start`     | `:system_time`             | `:conf`, `:job`                                                         |\n| `:stop`      | `:duration`, `:queue_time` | `:conf`, `:job`, `:state`, `:result`                                    |\n| `:exception` | `:duration`, `:queue_time` | `:conf`, `:job`, `:state`, `:kind`, `:reason`, `:result`, `:stacktrace` |","ref":"Oban.Telemetry.html#module-job-events"},{"type":"module","title":"Metadata - Oban.Telemetry","doc":"* `:conf` — the executing Oban instance's config\n* `:job` — the executing `Oban.Job`\n* `:state` — one of `:success`, `:failure`, `:cancelled`, `:discard` or `:snoozed`\n* `:result` — the `perform/1` return value, always `nil` for an exception or crash\n\nFor `:exception` events the metadata also includes details about what caused the failure. The\n`:kind` value is determined by how an error occurred. Here are the possible kinds:\n\n* `:error` — from an `{:error, error}` return value. Some Erlang functions may also throw an\n  `:error` tuple, which will be reported as `:error`.\n* `:exit` — from a caught process exit\n* `:throw` — from a caught value, this doesn't necessarily mean that an error occurred and the\n  error value is unpredictable","ref":"Oban.Telemetry.html#module-metadata"},{"type":"module","title":"Engine Events - Oban.Telemetry","doc":"Oban emits telemetry span events for the following Engine operations:\n\n* `[:oban, :engine, :init, :start | :stop | :exception]`\n* `[:oban, :engine, :refresh, :start | :stop | :exception]`\n* `[:oban, :engine, :put_meta, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                              |\n| ------------ | -------------- | ----------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:engine`                                    |\n| `:stop`      | `:duration`    | `:conf`, `:engine`                                    |\n| `:exception` | `:duration`    | `:conf`, `:engine`, `:kind`, `:reason`, `:stacktrace` |\n\nEvents for bulk operations also include `:jobs` for the `:stop` event:\n\n* `[:oban, :engine, :cancel_all_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :fetch_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :insert_all_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :prune_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :retry_all_jobs, :start | :stop | :exception]`\n* `[:oban, :engine, :stage_jobs, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                              |\n| ------------ | -------------- | ----------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:engine`                                    |\n| `:stop`      | `:duration`    | `:conf`, `:engine`, `:jobs`                                    |\n| `:exception` | `:duration`    | `:conf`, `:engine`, `:kind`, `:reason`, `:stacktrace` |\n\nEvents for job-level Engine operations also include the `job`, with the exception of\n`:insert_job, :start`, because the `job` isn't available yet.\n\n* `[:oban, :engine, :cancel_job, :start | :stop | :exception]`\n* `[:oban, :engine, :complete_job, :start | :stop | :exception]`\n* `[:oban, :engine, :discard_job, :start | :stop | :exception]`\n* `[:oban, :engine, :error_job, :start | :stop | :exception]`\n* `[:oban, :engine, :insert_job, :start | :stop | :exception]`\n* `[:oban, :engine, :retry_job, :start | :stop | :exception]`\n* `[:oban, :engine, :snooze_job, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                                      |\n| ------------ | -------------- | ------------------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:engine`, `:job`                                    |\n| `:stop`      | `:duration`    | `:conf`, `:engine`, `:job`                                    |\n| `:exception` | `:duration`    | `:conf`, `:engine`, `:job`, `:kind`, `:reason`, `:stacktrace` |","ref":"Oban.Telemetry.html#module-engine-events"},{"type":"module","title":"Metadata - Oban.Telemetry","doc":"* `:conf` — the Oban supervisor's config\n* `:engine` — the module of the engine used\n* `:job` - the `Oban.Job` in question\n* `:jobs` — zero or more maps with the `queue`, `state` for each modified job\n* `:kind`, `:reason`, `:stacktrace` — see the explanation in job metadata above","ref":"Oban.Telemetry.html#module-metadata"},{"type":"module","title":"Notifier Events - Oban.Telemetry","doc":"Oban emits a telemetry span event each time the Notifier is triggered:\n\n* `[:oban, :notifier, :notify, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                                            |\n| ------------ | -------------- | ------------------------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:channel`, `:payload`                                     |\n| `:stop`      | `:duration`    | `:conf`, `:channel`, `:payload`                                     |\n| `:exception` | `:duration`    | `:conf`, `:channel`, `:payload`, `:kind`, `:reason`, `:stacktrace`  |","ref":"Oban.Telemetry.html#module-notifier-events"},{"type":"module","title":"Metadata - Oban.Telemetry","doc":"* `:conf` — the Oban supervisor's config\n* `:channel` — the channel on which the notification was sent\n* `:payload` - the decoded payload that was sent\n* `:kind`, `:reason`, `:stacktrace` — see the explanation in job metadata above\n\n\nOban also emits an event when the notifier's sonar, responsible for tracking notifier\nconnectivity, switches connectivity status:\n\n* `[:oban, :notifier, :switch]`\n\n| event        | measures  | metadata           |\n| ------------ | --------- | ------------------ |\n| `:switch`    |           | `:conf`, `:status` |","ref":"Oban.Telemetry.html#module-metadata"},{"type":"module","title":"Metadata - Oban.Telemetry","doc":"* `:conf` — see the explanation in metadata above\n* `:status` — one of `:isolated`, `:solitary`, or `:clustered`, see\n  `Oban.Notifier.status/1` for details","ref":"Oban.Telemetry.html#module-metadata"},{"type":"module","title":"Plugin Events - Oban.Telemetry","doc":"All the Oban plugins emit telemetry events under the `[:oban, :plugin, *]` pattern (where `*` is\neither `:init`, `:start`, `:stop`, or `:exception`). You can filter out for plugin events by\nlooking into the metadata of the event and checking the value of `:plugin`. The `:plugin` field\nis the plugin module that emitted the event. For example, to get `Oban.Plugins.Cron` specific\nevents, you can filter for telemetry events with a metadata key/value of `plugin:\nOban.Plugins.Cron`.\n\nOban emits the following telemetry event whenever a plugin executes (be sure to check the\ndocumentation for each plugin as each plugin can also add additional metadata specific to\nthe plugin):\n\n* `[:oban, :plugin, :init]` — when the plugin first initializes\n* `[:oban, :plugin, :start]` — when the plugin beings performing its work\n* `[:oban, :plugin, :stop]` —  after the plugin completes its work\n* `[:oban, :plugin, :exception]` — when the plugin encounters an error\n\nThe following chart shows which metadata you can expect for each event:\n\n| event        | measures        | metadata                                              |\n| ------------ | --------------- | ----------------------------------------------------- |\n| `:init`      |                 | `:conf`, `:plugin`                                    |\n| `:start`     | `:system_time`  | `:conf`, `:plugin`                                    |\n| `:stop`      | `:duration`     | `:conf`, `:plugin`                                    |\n| `:exception` | `:duration`     | `:conf`, `:plugin`, `:kind`, `:reason`, `:stacktrace` |","ref":"Oban.Telemetry.html#module-plugin-events"},{"type":"module","title":"Peer Events - Oban.Telemetry","doc":"Oban emits a telemetry span event each time an Oban Peer election occurs:\n\n* `[:oban, :peer, :election, :start | :stop | :exception]`\n\n| event        | measures       | metadata                                                       |\n| ------------ | -------------- | -------------------------------------------------------------- |\n| `:start`     | `:system_time` | `:conf`, `:leader`, `:peer`,                                   |\n| `:stop`      | `:duration`    | `:conf`, `:leader`, `:peer`,                                   |\n| `:exception` | `:duration`    | `:conf`, `:leader`, `:peer`, `:kind`, `:reason`, `:stacktrace` |","ref":"Oban.Telemetry.html#module-peer-events"},{"type":"module","title":"Metadata - Oban.Telemetry","doc":"* `:conf`, `:kind`, `:reason`, `:stacktrace` — see the explanation in notifier metadata above\n* `:leader` — whether the peer is the current leader\n* `:peer` — the module used for peering","ref":"Oban.Telemetry.html#module-metadata"},{"type":"module","title":"Stager Events - Oban.Telemetry","doc":"Oban emits an event any time the Stager switches between `local` and `global` modes:\n\n* `[:oban, :stager, :switch]`\n\n| event        | measures  | metadata         |\n| ------------ | --------- | ---------------- |\n| `:switch`    |           | `:conf`, `:mode` |","ref":"Oban.Telemetry.html#module-stager-events"},{"type":"module","title":"Metadata - Oban.Telemetry","doc":"* `:conf` — see the explanation in metadata above\n* `:mode` — either `local` for polling mode or `global` in the more efficient pub-sub mode","ref":"Oban.Telemetry.html#module-metadata"},{"type":"module","title":"Default Logger - Oban.Telemetry","doc":"A default log handler that emits structured JSON is provided, see `attach_default_logger/0` for\nusage. Otherwise, if you would prefer more control over logging or would like to instrument\nevents you can write your own handler.\n\nHere is an example of the JSON output for the `job:stop` event:\n\n```json\n{\n  \"args\":{\"action\":\"OK\",\"ref\":1},\n  \"attempt\":1,\n  \"duration\":4327295,\n  \"event\":\"job:stop\",\n  \"id\":123,\n  \"max_attempts\":20,\n  \"meta\":{},\n  \"queue\":\"alpha\",\n  \"queue_time\":3127905,\n  \"source\":\"oban\",\n  \"state\":\"success\",\n  \"tags\":[],\n  \"worker\":\"Oban.Integration.Worker\"\n}\n```\n\nAll timing measurements are recorded as native time units but logged in microseconds.","ref":"Oban.Telemetry.html#module-default-logger"},{"type":"module","title":"Examples - Oban.Telemetry","doc":"A handler that only logs a few details about failed jobs:\n\n```elixir\ndefmodule MicroLogger do\n  require Logger\n\n  def handle_event([:oban, :job, :exception], %{duration: duration}, meta, nil) do\n    Logger.warning(\"[#{meta.queue}] #{meta.worker} failed in #{duration}\")\n  end\nend\n\n:telemetry.attach(\"oban-logger\", [:oban, :job, :exception], &MicroLogger.handle_event/4, nil)\n```\n\nAnother great use of execution data is error reporting. Here is an example of integrating with\n[Honeybadger][honey], but only reporting jobs that have failed 3 times or more:\n\n```elixir\ndefmodule ErrorReporter do\n  def handle_event([:oban, :job, :exception], _, %{attempt: attempt} = meta, _) do\n    if attempt >= 3 do\n      context = Map.take(meta, [:id, :args, :queue, :worker])\n\n      Honeybadger.notify(meta.reason, metadata: context, stacktrace: meta.stacktrace)\n    end\n  end\nend\n\n:telemetry.attach(\"oban-errors\", [:oban, :job, :exception], &ErrorReporter.handle_event/4, [])\n```\n\n[honey]: https://honeybadger.io","ref":"Oban.Telemetry.html#module-examples"},{"type":"function","title":"Oban.Telemetry.attach_default_logger/1","doc":"Attaches a default structured JSON Telemetry handler for logging.\n\nThis function attaches a handler that outputs logs with the following fields for job events:\n\n* `args` — a map of the job's raw arguments\n* `attempt` — the job's execution atttempt\n* `duration` — the job's runtime duration, in the native time unit\n* `event` — `job:start`, `job:stop`, `job:exception` depending on reporting telemetry event\n* `error` — a formatted error banner, without the extended stacktrace\n* `id` — the job's id\n* `meta` — a map of the job's raw metadata\n* `queue` — the job's queue\n* `source` — always \"oban\"\n* `state` — the execution state, one of \"success\", \"failure\", \"cancelled\", \"discard\", or\n  \"snoozed\"\n* `system_time` — when the job started, in microseconds\n* `tags` — the job's tags\n* `worker` — the job's worker module\n\nFor stager events:\n\n* `event` — always `stager:switch`\n* `message` — information about the mode switch\n* `mode` — either `\"local\"` or `\"global\"`\n* `source` — always \"oban\"\n\nFor notifier events:\n\n* `event` — always `notifier:switch`\n* `message` — information about the status switch\n* `source` — always \"oban\"\n* `status` — either `\"isolated\"`, `\"solitary\"`, or `\"clustered\"`","ref":"Oban.Telemetry.html#attach_default_logger/1"},{"type":"function","title":"Options - Oban.Telemetry.attach_default_logger/1","doc":"* `:level` — The log level to use for logging output, defaults to `:info`\n* `:encode` — Whether to encode log output as JSON, defaults to `true`","ref":"Oban.Telemetry.html#attach_default_logger/1-options"},{"type":"function","title":"Examples - Oban.Telemetry.attach_default_logger/1","doc":"Attach a logger at the default `:info` level with JSON encoding:\n\n    :ok = Oban.Telemetry.attach_default_logger()\n\nAttach a logger at the `:debug` level:\n\n    :ok = Oban.Telemetry.attach_default_logger(level: :debug)\n\nAttach a logger with JSON logging disabled:\n\n    :ok = Oban.Telemetry.attach_default_logger(encode: false)","ref":"Oban.Telemetry.html#attach_default_logger/1-examples"},{"type":"function","title":"Oban.Telemetry.detach_default_logger/0","doc":"Undoes `Oban.Telemetry.attach_default_logger/1` by detaching the attached logger.","ref":"Oban.Telemetry.html#detach_default_logger/0"},{"type":"function","title":"Examples - Oban.Telemetry.detach_default_logger/0","doc":"Detach a previously attached logger:\n\n    :ok = Oban.Telemetry.attach_default_logger()\n    :ok = Oban.Telemetry.detach_default_logger()\n\nAttempt to detach when a logger wasn't attached:\n\n    {:error, :not_found} = Oban.Telemetry.detach_default_logger()","ref":"Oban.Telemetry.html#detach_default_logger/0-examples"},{"type":"module","title":"Oban.Testing","doc":"This module simplifies testing workers and making assertions about enqueued jobs when testing in\n`:manual` mode.\n\nAssertions may be made on any property of a job, but you'll typically want to check by `args`,\n`queue` or `worker`.","ref":"Oban.Testing.html"},{"type":"module","title":"Usage - Oban.Testing","doc":"The most convenient way to use `Oban.Testing` is to `use` the module:\n\n    use Oban.Testing, repo: MyApp.Repo\n\nThat will define the helper functions you'll use to make assertions on the jobs that should (or\nshould not) be inserted in the database while testing.\n\nIf you're using namespacing through Postgres schemas, also called \"prefixes\" in Ecto, you\nshould set the `prefix` option:\n\n    use Oban.Testing, repo: MyApp.Repo, prefix: \"business\"\n\nUnless overridden, the default `prefix` is `public`.","ref":"Oban.Testing.html#module-usage"},{"type":"module","title":"Adding to Case Templates - Oban.Testing","doc":"To include helpers in all of your tests you can add it to your case template:\n\n```elixir\ndefmodule MyApp.DataCase do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      use Oban.Testing, repo: MyApp.Repo\n\n      import Ecto\n      import Ecto.Changeset\n      import Ecto.Query\n      import MyApp.DataCase\n\n      alias MyApp.Repo\n    end\n  end\nend\n```","ref":"Oban.Testing.html#module-adding-to-case-templates"},{"type":"module","title":"Examples - Oban.Testing","doc":"After the test helpers are imported, you can make assertions about enqueued (available or\nscheduled) jobs in your tests.\n\nHere are a few examples that demonstrate what's possible:\n\n```elixir\n# Assert that a job was already enqueued\nassert_enqueued worker: MyWorker, args: %{id: 1}\n\n# Assert that a job was enqueued or will be enqueued in the next 100ms\nassert_enqueued [worker: MyWorker, args: %{id: 1}], 100\n\n# Refute that a job was already enqueued\nrefute_enqueued queue: \"special\", args: %{id: 2}\n\n# Refute that a job was already enqueued or would be enqueued in the next 100ms\nrefute_enqueued queue: \"special\", args: %{id: 2}, 100\n\n# Make assertions on a list of all jobs matching some options\nassert [%{args: %{\"id\" => 1}}] = all_enqueued(worker: MyWorker)\n\n# Assert that no jobs are enqueued in any queues\nassert [] = all_enqueued()\n```\n\nNote that the final example, using `all_enqueued/1`, returns a raw list of matching jobs and\ndoes not make an assertion by itself. This makes it possible to test using pattern matching at\nthe expense of being more verbose.\n\nSee the docs for `assert_enqueued/1,2`, `refute_enqueued/1,2`, and `all_enqueued/1` for more\nexamples.","ref":"Oban.Testing.html#module-examples"},{"type":"module","title":"Matching Timestamps - Oban.Testing","doc":"In order to assert a job has been scheduled at a certain time, you will need to match against\nthe `scheduled_at` attribute of the enqueued job.\n\n    in_an_hour = DateTime.add(DateTime.utc_now(), 3600, :second)\n    assert_enqueued worker: MyApp.Worker, scheduled_at: in_an_hour\n\nBy default, Oban will apply a 1 second delta to all timestamp fields of jobs, so that small\ndeviations between the actual value and the expected one are ignored. You may configure this\ndelta by passing a tuple of value and a `delta` option (in seconds) to corresponding keyword:\n\n    assert_enqueued worker: MyApp.Worker, scheduled_at: {in_an_hour, delta: 10}","ref":"Oban.Testing.html#module-matching-timestamps"},{"type":"function","title":"Oban.Testing.all_enqueued/1","doc":"Retrieve all currently enqueued jobs matching a set of options.\n\nOnly jobs matching all of the provided arguments will be returned. Additionally, jobs are\nreturned in descending order where the most recently enqueued job will be listed first.","ref":"Oban.Testing.html#all_enqueued/1"},{"type":"function","title":"Examples - Oban.Testing.all_enqueued/1","doc":"Assert based on only _some_ of a job's args:\n\n    assert [%{args: %{\"id\" => 1}}] = all_enqueued(worker: MyWorker)\n\nAssert that exactly one job was inserted for a queue:\n\n    assert [%Oban.Job{}] = all_enqueued(queue: :alpha)\n\nAssert that there aren't any jobs enqueued for any queues or workers:\n\n    assert [] = all_enqueued()","ref":"Oban.Testing.html#all_enqueued/1-examples"},{"type":"function","title":"Oban.Testing.assert_enqueued/1","doc":"Assert that a job with matching fields is enqueued.\n\nOnly values for the provided fields are checked. For example, an assertion made on `worker:\n\"MyWorker\"` will match _any_ jobs for that worker, regardless of every other field.","ref":"Oban.Testing.html#assert_enqueued/1"},{"type":"function","title":"Examples - Oban.Testing.assert_enqueued/1","doc":"Assert that a job is enqueued for a certain worker and args:\n\n    assert_enqueued worker: MyWorker, args: %{id: 1}\n\nAssert that a job is enqueued for a particular queue and priority:\n\n    assert_enqueued queue: :business, priority: 3\n\nAssert that a job's args deeply match:\n\n    assert_enqueued args: %{config: %{enabled: true}}\n\nUse the `:_` wildcard to assert that a job's meta has a key with any value:\n\n    assert_enqueued meta: %{batch_id: :_}","ref":"Oban.Testing.html#assert_enqueued/1-examples"},{"type":"function","title":"Oban.Testing.assert_enqueued/2","doc":"Assert that a job with particular options is or will be enqueued within a timeout period.\n\nSee `assert_enqueued/1` for additional details.","ref":"Oban.Testing.html#assert_enqueued/2"},{"type":"function","title":"Examples - Oban.Testing.assert_enqueued/2","doc":"Assert that a job will be enqueued in the next 100ms:\n\n    assert_enqueued [worker: MyWorker], 100","ref":"Oban.Testing.html#assert_enqueued/2-examples"},{"type":"function","title":"Oban.Testing.perform_job/3","doc":"Construct a job and execute it with a worker module.\n\nThis reduces boilerplate when constructing jobs for unit tests and checks for common pitfalls.\nFor example, it automatically converts `args` to string keys before calling `perform/1`,\nensuring that perform clauses aren't erroneously trying to match atom keys.\n\nThe helper makes the following assertions:\n\n* That the worker implements the `Oban.Worker` behaviour\n* That the options provided build a valid job\n* That the return is valid, e.g. `:ok`, `{:ok, value}`, `{:error, value}` etc.\n\nIf all of the assertions pass then the function returns the result of `perform/1` for you to\nmake additional assertions on.","ref":"Oban.Testing.html#perform_job/3"},{"type":"function","title":"Examples - Oban.Testing.perform_job/3","doc":"Successfully execute a job with some string arguments:\n\n    assert :ok = perform_job(MyWorker, %{\"id\" => 1})\n\nSuccessfully execute a job and assert that it returns an error tuple:\n\n    assert {:error, _} = perform_job(MyWorker, %{\"bad\" => \"arg\"})\n\nExecute a job with the args keys automatically stringified:\n\n    assert :ok = perform_job(MyWorker, %{id: 1})\n\nExercise custom attempt handling within a worker by passing options:\n\n    assert :ok = perform_job(MyWorker, %{}, attempt: 42)\n\nCause a test failure because the provided worker isn't real:\n\n    assert :ok = perform_job(Vorker, %{\"id\" => 1})","ref":"Oban.Testing.html#perform_job/3-examples"},{"type":"function","title":"Oban.Testing.refute_enqueued/1","doc":"Refute that a job with particular options has been enqueued.\n\nSee `assert_enqueued/1` for additional details.","ref":"Oban.Testing.html#refute_enqueued/1"},{"type":"function","title":"Examples - Oban.Testing.refute_enqueued/1","doc":"Refute that a job is enqueued for a certain worker:\n\n    refute_enqueued worker: MyWorker\n\nRefute that a job is enqueued for a certain worker and args:\n\n    refute_enqueued worker: MyWorker, args: %{id: 1}\n\nRefute that a job's nested args match:\n\n    refute_enqueued args: %{config: %{enabled: false}}\n\nUse the `:_` wildcard to refute that a job's meta has a key with any value:\n\n    refute_enqueued meta: %{batch_id: :_}","ref":"Oban.Testing.html#refute_enqueued/1-examples"},{"type":"function","title":"Oban.Testing.refute_enqueued/2","doc":"Refute that a job with particular options is or will be enqueued within a timeout period.\n\nSee `assert_enqueued/1` for additional details.","ref":"Oban.Testing.html#refute_enqueued/2"},{"type":"function","title":"Examples - Oban.Testing.refute_enqueued/2","doc":"Refute that a job will be enqueued in the next 100ms:\n\n    refute_enqueued [worker: MyWorker], 100","ref":"Oban.Testing.html#refute_enqueued/2-examples"},{"type":"function","title":"Oban.Testing.refute_enqueued/3","doc":"","ref":"Oban.Testing.html#refute_enqueued/3"},{"type":"function","title":"Oban.Testing.with_testing_mode/2","doc":"Change the testing mode within the context of a function.\n\nOnly `:manual` and `:inline` mode are supported, as `:disabled` implies that supervised queues\nand plugins are running and this function won't start any processes.","ref":"Oban.Testing.html#with_testing_mode/2"},{"type":"function","title":"Examples - Oban.Testing.with_testing_mode/2","doc":"Switch to `:manual` mode when an Oban instance is configured for `:inline` testing:\n\n    Oban.Testing.with_testing_mode(:manual, fn ->\n      Oban.insert(MyWorker.new(%{id: 123}))\n\n      assert_enqueued worker: MyWorker, args: %{id: 123}\n    end)\n\nVisa-versa, switch to `:inline` mode:\n\n    Oban.Testing.with_testing_mode(:inline, fn ->\n      {:ok, %Job{state: \"completed\"}} = Oban.insert(MyWorker.new(%{id: 123}))\n    end)","ref":"Oban.Testing.html#with_testing_mode/2-examples"},{"type":"type","title":"Oban.Testing.perform_opts/0","doc":"","ref":"Oban.Testing.html#t:perform_opts/0"},{"type":"behaviour","title":"Oban.Worker","doc":"Defines a behavior and macro to guide the creation of worker modules.\n\nWorker modules do the work of processing a job. At a minimum they must define a `c:perform/1`\nfunction, which is called with the full `Oban.Job` struct.","ref":"Oban.Worker.html"},{"type":"behaviour","title":"Defining Workers - Oban.Worker","doc":"Worker modules are defined by using `Oban.Worker`. A bare `use Oban.Worker` invocation sets a\nworker with these defaults:\n\n* `:max_attempts` — 20\n* `:priority` — 0\n* `:queue` — `:default`\n* `:tags` — no tags set\n* `:replace` — no replacement set\n* `:unique` — no uniqueness set\n\nTo create a minimum worker using the defaults, including the `default` queue:\n\n    defmodule MyApp.Workers.Basic do\n      use Oban.Worker\n\n      @impl Oban.Worker\n      def perform(%Oban.Job{args: args}) do\n        IO.inspect(args)\n        :ok\n      end\n    end\n\nThe following example defines a complex worker module to process jobs in the `events` queue.\nIt then dials down the priority from 0 to 1, limits retrying on failures to 10 attempts, adds\na \"business\" tag, and ensures that duplicate jobs aren't enqueued within a 30 second period:\n\n    defmodule MyApp.Workers.Business do\n      use Oban.Worker,\n        queue: :events,\n        priority: 1,\n        max_attempts: 10,\n        tags: [\"business\"],\n        replace: [scheduled: [:scheduled_at]],\n        unique: [period: 30]\n\n      @impl Oban.Worker\n      def perform(%Oban.Job{attempt: attempt}) when attempt > 3 do\n        IO.inspect(attempt)\n      end\n\n      def perform(job) do\n        IO.inspect(job.args)\n      end\n    end\n\nThe `c:perform/1` function receives an `Oban.Job` struct as an argument. This allows workers to\nchange the behavior of `c:perform/1` based on attributes of the job, e.g. the args, number of\nexecution attempts, or when it was inserted.\n\nThe value returned from `c:perform/1` can control whether the job is a success or a failure:\n\n* `:ok` or `{:ok, value}` — the job is successful and marked as `completed`.  The `value` from\n  success tuples is ignored.\n\n* `{:cancel, reason}` — cancel executing the job and stop retrying it. An error is recorded\n  using the provided `reason`. The job is marked as `cancelled`.\n\n* `{:error, error}` — the job failed, record the error. If `max_attempts` has not been reached\n  already, the job is marked as `retryable` and scheduled to run again. Otherwise, the job is\n  marked as `discarded` and won't be retried.\n\n* `{:snooze, seconds}` — mark the job as `snoozed` and schedule it to run again `seconds` in the\n  future. See [Snoozing](#module-snoozing-jobs) for more details.\n\nIn addition to explicit return values, any _unhandled exception_, _exit_ or _throw_ will fail\nthe job and schedule a retry under the same conditions as in the `{:error, error}` case.\n\nAs an example of error tuple handling, this worker will return an error tuple when the `value`\nis less than one:\n\n    defmodule MyApp.Workers.ErrorExample do\n      use Oban.Worker\n\n      @impl Worker\n      def perform(%{args: %{\"value\" => value}}) do\n        if value > 1 do\n          :ok\n        else\n          {:error, \"invalid value given: \" <> inspect(value)}\n        end\n      end\n    end\n\nThe error tuple is wrapped in an `Oban.PerformError` with a formatted message. The error tuple\nitself is available through the exception's `:reason` field.","ref":"Oban.Worker.html#module-defining-workers"},{"type":"behaviour","title":"Enqueuing Jobs - Oban.Worker","doc":"All workers implement a `c:new/2` function that converts an args map into a job changeset\nsuitable for inserting into the database for later execution:\n\n    %{in_the: \"business\", of_doing: \"business\"}\n    |> MyApp.Workers.Business.new()\n    |> Oban.insert()\n\nThe worker's defaults may be overridden by passing options:\n\n    %{vote_for: \"none of the above\"}\n    |> MyApp.Workers.Business.new(queue: \"special\", max_attempts: 5)\n    |> Oban.insert()\n\nUniqueness options may also be overridden by passing options:\n\n    %{expensive: \"business\"}\n    |> MyApp.Workers.Business.new(unique: [period: 120, fields: [:worker]])\n    |> Oban.insert()\n\nNote that `unique` options aren't merged, they are overridden entirely.\n\nSee `Oban.Job` for all available options.","ref":"Oban.Worker.html#module-enqueuing-jobs"},{"type":"behaviour","title":"Customizing Backoff - Oban.Worker","doc":"When jobs fail they may be retried again in the future using a backoff algorithm. By default the\nbackoff is exponential with a fixed padding of 15 seconds and a small amount of jitter. The\njitter helps to prevent jobs that fail simultaneously from consistently retrying at the same\ntime. With the default backoff behavior, the 20th attempt will occur around 12 days after the\nfirst attempt.\n\nIf the default strategy is too aggressive or otherwise unsuited to your app's workload you can\ndefine a custom backoff function using the `c:backoff/1` callback.\n\nThe following worker defines a `c:backoff/1` function that delays retries using a variant of the\nhistoric Resque/Sidekiq algorithm:\n\n    defmodule MyApp.SidekiqBackoffWorker do\n      use Oban.Worker\n\n      @impl Worker\n      def backoff(%Job{attempt: attempt}) do\n        trunc(:math.pow(attempt, 4) + 15 + :rand.uniform(30) * attempt)\n      end\n\n      @impl Worker\n      def perform(_job) do\n        :do_business\n      end\n    end\n\nHere are some alternative backoff strategies to consider:\n\n* **constant** — delay by a fixed number of seconds, e.g. 1→15, 2→15, 3→15\n* **linear** — delay for the same number of seconds as the current attempt, e.g. 1→1, 2→2, 3→3\n* **squared** — delay by attempt number squared, e.g. 1→1, 2→4, 3→9\n* **sidekiq** — delay by a base amount plus some jitter, e.g. 1→32, 2→61, 3→135","ref":"Oban.Worker.html#module-customizing-backoff"},{"type":"behaviour","title":"Contextual Backoff - Oban.Worker","doc":"Any error, catch or throw is temporarily recorded in the job's `unsaved_error` map. The unsaved\nerror map can be used by `c:backoff/1` to calculate a custom backoff based on the exact error\nthat failed the job. In this example the `c:backoff/1` callback checks to see if the error was\ndue to rate limiting and adjusts the backoff accordingly:\n\n    defmodule MyApp.ApiWorker do\n      use Oban.Worker\n\n      @five_minutes 5 * 60\n\n      @impl Worker\n      def perform(%{args: args}) do\n        MyApp.make_external_api_call(args)\n      end\n\n      @impl Worker\n      def backoff(%Job{attempt: attempt, unsaved_error: unsaved_error}) do\n        %{kind: _, reason: reason, stacktrace: _} = unsaved_error\n\n        case reason do\n          %MyApp.ApiError{status: 429} -> @five_minutes\n          _ -> trunc(:math.pow(attempt, 4))\n        end\n      end\n    end","ref":"Oban.Worker.html#module-contextual-backoff"},{"type":"behaviour","title":"Execution Timeout - Oban.Worker","doc":"By default, individual jobs may execute indefinitely. If this is undesirable you may define a\ntimeout in milliseconds with the `c:timeout/1` callback on your worker module.\n\nFor example, to limit a worker's execution time to 30 seconds:\n\n    def MyApp.Worker do\n      use Oban.Worker\n\n      @impl Oban.Worker\n      def perform(_job) do\n        something_that_may_take_a_long_time()\n\n        :ok\n      end\n\n      @impl Oban.Worker\n      def timeout(_job), do: :timer.seconds(30)\n    end\n\nThe `c:timeout/1` function accepts an `Oban.Job` struct, so you can customize the timeout using\nany job attributes.\n\nDefine the `timeout` value through job args:\n\n    def timeout(%_{args: %{\"timeout\" => timeout}}), do: timeout\n\nDefine the `timeout` based on the number of attempts:\n\n    def timeout(%_{attempt: attempt}), do: attempt * :timer.seconds(5)","ref":"Oban.Worker.html#module-execution-timeout"},{"type":"behaviour","title":"Snoozing Jobs - Oban.Worker","doc":"When returning `{:snooze, snooze_time}` in `c:perform/1`, the job is postponed for at least\n`snooze_time` seconds. Snoozing is done by incrementing the job's `max_attempts` field and\nscheduling execution for `snooze_time` seconds in the future.\n\nExecuting bumps a job's `attempt` count. Despite snooze incrementing the `max_attempts` to\npreserve total retries, the change to `attempt` will affect the default backoff retry\nalgorithm.\n\n> #### 🌟 Snoozes and Attempts {: .info}\n>\n> Oban Pro's [Smart Engine](https://getoban.pro/docs/pro/Oban.Pro.Engines.Smart.html) rolls back\n> the `attempt` and preserves the original `max_attempts` in order to differentiate between\n> \"real\" attempts and snoozes, which keeps backoff calculation accurate.\n>\n> Without attempt correction you may need a solution that compensates for snoozing, such as the\n> example below:\n\n    defmodule MyApp.SnoozingWorker do\n      @max_attempts 20\n\n      use Oban.Worker, max_attempts: @max_attempts\n\n      @impl Worker\n      def backoff(%Job{} = job) do\n        corrected_attempt = @max_attempts - (job.max_attempts - job.attempt)\n\n        Worker.backoff(%{job | attempt: corrected_attempt})\n      end\n\n      @impl Worker\n      def perform(job) do\n        if MyApp.something?(job), do: :ok, else: {:snooze, 60}\n      end\n    end","ref":"Oban.Worker.html#module-snoozing-jobs"},{"type":"callback","title":"Oban.Worker.backoff/1","doc":"Calculate the execution backoff.\n\nIn this context backoff specifies the number of seconds to wait before retrying a failed job.\n\nDefaults to an exponential algorithm with a minimum delay of 15 seconds and a small amount of\njitter.","ref":"Oban.Worker.html#c:backoff/1"},{"type":"function","title":"Oban.Worker.from_string/1","doc":"Resolve a module from a worker string.","ref":"Oban.Worker.html#from_string/1"},{"type":"function","title":"Examples - Oban.Worker.from_string/1","doc":"iex> Oban.Worker.from_string(\"Oban.Integration.Worker\")\n    {:ok, Oban.Integration.Worker}\n\n    iex> defmodule NotAWorker, do: []\n    ...> Oban.Worker.from_string(\"NotAWorker\")\n    {:error, %RuntimeError{message: \"module is not a worker: NotAWorker\"}}\n\n    iex> Oban.Worker.from_string(\"RandomWorker\")\n    {:error, %RuntimeError{message: \"unknown worker: RandomWorker\"}}","ref":"Oban.Worker.html#from_string/1-examples"},{"type":"callback","title":"Oban.Worker.new/2","doc":"Build a job changeset for this worker with optional overrides.\n\nSee `Oban.Job.new/2` for the available options.","ref":"Oban.Worker.html#c:new/2"},{"type":"callback","title":"Oban.Worker.perform/1","doc":"The `c:perform/1` function is called to execute a job.\n\nEach `c:perform/1` function should return `:ok` or a success tuple. When the return is an error\ntuple, an uncaught exception or a throw then the error is recorded and the job may be retried if\nthere are any attempts remaining.\n\nNote that the `args` map provided to `perform/1` will _always_ have string keys, regardless of\nthe key type when the job was enqueued. The `args` are stored as `jsonb` in PostgreSQL and the\nserialization process automatically stringifies all keys.","ref":"Oban.Worker.html#c:perform/1"},{"type":"callback","title":"Oban.Worker.timeout/1","doc":"Set a job's maximum execution time in milliseconds.\n\nJobs that exceed the time limit are considered a failure and may be retried.\n\nDefaults to `:infinity`.","ref":"Oban.Worker.html#c:timeout/1"},{"type":"function","title":"Oban.Worker.to_string/1","doc":"Return a string representation of a worker module.\n\nThis is particularly useful for normalizing worker names when building custom Ecto queries.","ref":"Oban.Worker.html#to_string/1"},{"type":"function","title":"Examples - Oban.Worker.to_string/1","doc":"iex> Oban.Worker.to_string(MyApp.SomeWorker)\n    \"MyApp.SomeWorker\"\n\n    iex> Oban.Worker.to_string(Elixir.MyApp.SomeWorker)\n    \"MyApp.SomeWorker\"\n\n    iex> Oban.Worker.to_string(\"Elixir.MyApp.SomeWorker\")\n    \"MyApp.SomeWorker\"","ref":"Oban.Worker.html#to_string/1-examples"},{"type":"type","title":"Oban.Worker.result/0","doc":"Return values control whether a job is treated as a success or a failure.\n\n- `:ok` - the job is successful and marked as `completed`.\n- `{:ok, ignored}` - the job is successful, marked as `completed`, and the return value is ignored.\n- `{:cancel, reason}` - the job is marked as `cancelled` for the provided reason and no longer retried.\n- `{:error, reason}` - the job is marked as `retryable` for the provided reason, or `discarded`\n  if it has  exhausted all attempts.\n- `{:snooze, seconds}` - mark the job as `scheduled` to run again `seconds` in the future.\n\n> #### Deprecated {: .warning}\n>\n> - `:discard` - deprecated, use `{:cancel, reason}` instead.\n> - `{:discard, reason}` - deprecated, use `{:cancel, reason}` instead.","ref":"Oban.Worker.html#t:result/0"},{"type":"type","title":"Oban.Worker.t/0","doc":"","ref":"Oban.Worker.html#t:t/0"},{"type":"module","title":"Oban.Plugins.Cron","doc":"Periodically enqueue jobs through CRON based scheduling.\n\nThis plugin registers workers a cron-like schedule and enqueues jobs automatically. Periodic\njobs are declared as a list of `{cron, worker}` or `{cron, worker, options}` tuples.\n\n\n> #### 🌟 DynamicCron {: .info}\n>\n> This plugin only loads the crontab statically, at boot time. To configure cron scheduling at\n> runtime, globally, across an entire cluster with scheduling guarantees and timezone overrides,\n> see the `DynamicCron` plugin in [Oban Pro](https://getoban.pro/docs/pro/Oban.Pro.Plugins.DynamicCron.html).","ref":"Oban.Plugins.Cron.html"},{"type":"module","title":"Using the Plugin - Oban.Plugins.Cron","doc":"Schedule various jobs using `{expr, worker}` and `{expr, worker, opts}` syntaxes:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    {Oban.Plugins.Cron,\n     crontab: [\n       {\"* * * * *\", MyApp.MinuteWorker},\n       {\"0 * * * *\", MyApp.HourlyWorker, args: %{custom: \"arg\"}},\n       {\"0 0 * * *\", MyApp.DailyWorker, max_attempts: 1},\n       {\"0 12 * * MON\", MyApp.MondayWorker, queue: :scheduled, tags: [\"mondays\"]},\n       {\"@daily\", MyApp.AnotherDailyWorker}\n     ]}\n  ]\n```","ref":"Oban.Plugins.Cron.html#module-using-the-plugin"},{"type":"module","title":"Options - Oban.Plugins.Cron","doc":"* `:crontab` — a list of cron expressions that enqueue jobs on a periodic basis. See [Periodic\n  Jobs][perjob] in the Oban module docs for syntax and details.\n\n* `:timezone` — which timezone to use when scheduling cron jobs. To use a timezone other than\n  the default of \"Etc/UTC\" you *must* have a timezone database like [tz][tz] installed and\n  configured.\n\n[tz]: https://hexdocs.pm/tz\n[perjob]: Oban.html#module-periodic-jobs","ref":"Oban.Plugins.Cron.html#module-options"},{"type":"module","title":"Identifying Cron Jobs - Oban.Plugins.Cron","doc":"Jobs inserted by the Cron plugin are marked with a `cron` flag and the _original_ expression is\nstored as `cron_expr` in the job's `meta` field. For example, the meta for a `@daily` cron job\nwould look like this:\n\n```elixir\n%Oban.Job{meta: %{\"cron\" => true, \"cron_expr\" => \"@daily\"}}\n```","ref":"Oban.Plugins.Cron.html#module-identifying-cron-jobs"},{"type":"module","title":"Instrumenting with Telemetry - Oban.Plugins.Cron","doc":"The `Oban.Plugins.Cron` plugin adds the following metadata to the `[:oban, :plugin, :stop]` event:\n\n* :jobs - a list of jobs that were inserted into the database","ref":"Oban.Plugins.Cron.html#module-instrumenting-with-telemetry"},{"type":"function","title":"Oban.Plugins.Cron.parse/1","doc":"Parse a crontab expression into a cron struct.\n\nThis is provided as a convenience for validating and testing cron expressions. As such, the cron\nstruct itself is opaque and the internals may change at any time.\n\nThe parser can handle common expressions that use minutes, hours, days, months and weekdays,\nalong with ranges and steps. It also supports common extensions, also called nicknames.\n\nReturns `{:error, %ArgumentError{}}` with a detailed error if the expression cannot be parsed.","ref":"Oban.Plugins.Cron.html#parse/1"},{"type":"function","title":"Nicknames - Oban.Plugins.Cron.parse/1","doc":"The following special nicknames are supported in addition to standard cron expressions:\n\n  * `@yearly`—Run once a year, \"0 0 1 1 *\"\n  * `@annually`—Same as `@yearly`\n  * `@monthly`—Run once a month, \"0 0 1 * *\"\n  * `@weekly`—Run once a week, \"0 0 * * 0\"\n  * `@daily`—Run once a day, \"0 0 * * *\"\n  * `@midnight`—Same as `@daily`\n  * `@hourly`—Run once an hour, \"0 * * * *\"\n  * `@reboot`—Run once at boot","ref":"Oban.Plugins.Cron.html#parse/1-nicknames"},{"type":"function","title":"Examples - Oban.Plugins.Cron.parse/1","doc":"iex> Oban.Plugins.Cron.parse(\"@hourly\")\n    {:ok, #Oban.Cron.Expression<...>}\n\n    iex> Oban.Plugins.Cron.parse(\"0 * * * *\")\n    {:ok, #Oban.Cron.Expression<...>}\n\n    iex> Oban.Plugins.Cron.parse(\"60 * * * *\")\n    {:error, %ArgumentError{message: \"expression field 60 is out of range 0..59\"}}","ref":"Oban.Plugins.Cron.html#parse/1-examples"},{"type":"type","title":"Oban.Plugins.Cron.cron_input/0","doc":"","ref":"Oban.Plugins.Cron.html#t:cron_input/0"},{"type":"opaque","title":"Oban.Plugins.Cron.expression/0","doc":"","ref":"Oban.Plugins.Cron.html#t:expression/0"},{"type":"type","title":"Oban.Plugins.Cron.option/0","doc":"","ref":"Oban.Plugins.Cron.html#t:option/0"},{"type":"module","title":"Oban.Plugins.Lifeline","doc":"Naively transition jobs stuck `executing` back to `available`.\n\nThe `Lifeline` plugin periodically rescues orphaned jobs, i.e. jobs that are stuck in the\n`executing` state because the node was shut down before the job could finish. Rescuing is\npurely based on time, rather than any heuristic about the job's expected execution time or\nwhether the node is still alive.\n\nIf an executing job has exhausted all attempts, the Lifeline plugin will mark it `discarded`\nrather than `available`.\n\n> #### 🌟 DynamicLifeline {: .info}\n>\n> This plugin may transition jobs that are genuinely `executing` and cause duplicate execution.\n> For more accurate rescuing or to rescue jobs that have exhausted retry attempts see the\n> `DynamicLifeline` plugin in [Oban Pro](https://getoban.pro/docs/pro/Oban.Pro.Plugins.DynamicLifeline.html).","ref":"Oban.Plugins.Lifeline.html"},{"type":"module","title":"Using the Plugin - Oban.Plugins.Lifeline","doc":"Rescue orphaned jobs that are still `executing` after the default of 60 minutes:\n\n    config :my_app, Oban,\n      plugins: [Oban.Plugins.Lifeline],\n      ...\n\nOverride the default period to rescue orphans after a more aggressive period of 5 minutes:\n\n    config :my_app, Oban,\n      plugins: [{Oban.Plugins.Lifeline, rescue_after: :timer.minutes(5)}],\n      ...","ref":"Oban.Plugins.Lifeline.html#module-using-the-plugin"},{"type":"module","title":"Options - Oban.Plugins.Lifeline","doc":"* `:interval` — the number of milliseconds between rescue attempts. The default is `60_000ms`.\n\n* `:rescue_after` — the maximum amount of time, in milliseconds, that a job may execute before\nbeing rescued. 60 minutes by default, and rescuing is performed once a minute.","ref":"Oban.Plugins.Lifeline.html#module-options"},{"type":"module","title":"Instrumenting with Telemetry - Oban.Plugins.Lifeline","doc":"The `Oban.Plugins.Lifeline` plugin adds the following metadata to the `[:oban, :plugin, :stop]`\nevent:\n\n* `:rescued_jobs` — a list of jobs transitioned back to `available`\n\n* `:discarded_jobs` — a list of jobs transitioned to `discarded`\n\n_Note: jobs only include `id`, `queue`, `state` fields._","ref":"Oban.Plugins.Lifeline.html#module-instrumenting-with-telemetry"},{"type":"type","title":"Oban.Plugins.Lifeline.option/0","doc":"","ref":"Oban.Plugins.Lifeline.html#t:option/0"},{"type":"module","title":"Oban.Plugins.Pruner","doc":"Periodically delete `completed`, `cancelled` and `discarded` jobs based on their age.\n\nPruning is critical for maintaining table size and continued responsive job processing. It\nis recommended for all production applications.\n\n> #### 🌟 DynamicPruner {: .info}\n>\n> This plugin limited to a fixed interval and a single `max_age` check for all jobs. To prune\n> on a cron-style schedule, retain jobs by a limit or age, or provide overrides for specific\n> queues, workers, and job states; see Oban Pro's\n> [DynamicPruner](https://getoban.pro/docs/pro/Oban.Pro.Plugins.DynamicPruner.html).","ref":"Oban.Plugins.Pruner.html"},{"type":"module","title":"Using the Plugin - Oban.Plugins.Pruner","doc":"The following example demonstrates using the plugin without any configuration, which will prune\njobs older than the default of 60 seconds:\n\n    config :my_app, Oban,\n      plugins: [Oban.Plugins.Pruner],\n      ...\n\nOverride the default options to prune jobs after 5 minutes:\n\n    config :my_app, Oban,\n      plugins: [{Oban.Plugins.Pruner, max_age: 300}],\n      ...","ref":"Oban.Plugins.Pruner.html#module-using-the-plugin"},{"type":"module","title":"Options - Oban.Plugins.Pruner","doc":"* `:interval` — the number of milliseconds between pruning attempts. The default is `30_000ms`.\n\n* `:limit` — the maximum number of jobs to prune at one time. The default is 10,000 to prevent\n  request timeouts. Applications that steadily generate more than 10k jobs a minute should\n  increase this value.\n\n* `:max_age` — the number of seconds after which a job may be pruned. Defaults to 60s.","ref":"Oban.Plugins.Pruner.html#module-options"},{"type":"module","title":"Instrumenting with Telemetry - Oban.Plugins.Pruner","doc":"The `Oban.Plugins.Pruner` plugin adds the following metadata to the `[:oban, :plugin, :stop]` event:\n\n* `:pruned_jobs` - the jobs that were deleted from the database\n\n_Note: jobs only include `id`, `queue`, `state` fields._","ref":"Oban.Plugins.Pruner.html#module-instrumenting-with-telemetry"},{"type":"type","title":"Oban.Plugins.Pruner.option/0","doc":"","ref":"Oban.Plugins.Pruner.html#t:option/0"},{"type":"module","title":"Oban.Plugins.Reindexer","doc":"Periodically rebuild indexes to minimize database bloat.\n\nOver time various Oban indexes may grow without `VACUUM` cleaning them up properly. When this\nhappens, rebuilding the indexes will release bloat.\n\nThe plugin uses `REINDEX` with the `CONCURRENTLY` option to rebuild without taking any locks\nthat prevent concurrent inserts, updates, or deletes on the table.\n\nNote: This plugin requires the `CONCURRENT` option, which is only available in Postgres 12 and\nabove.","ref":"Oban.Plugins.Reindexer.html"},{"type":"module","title":"Using the Plugin - Oban.Plugins.Reindexer","doc":"By default, the plugin will reindex once a day, at midnight UTC:\n\n    config :my_app, Oban,\n      plugins: [Oban.Plugins.Reindexer],\n      ...\n\nTo run on a different schedule you can provide a cron expression. For example, you could use the\n`\"@weekly\"` shorthand to run once a week on Sunday:\n\n    config :my_app, Oban,\n      plugins: [{Oban.Plugins.Reindexer, schedule: \"@weekly\"}],\n      ...","ref":"Oban.Plugins.Reindexer.html#module-using-the-plugin"},{"type":"module","title":"Options - Oban.Plugins.Reindexer","doc":"* `:indexes` — a list of indexes to reindex on the `oban_jobs` table. Defaults to only the\n    `oban_jobs_args_index` and `oban_jobs_meta_index`.\n\n  * `:schedule` — a cron expression that controls when to reindex. Defaults to `\"@midnight\"`.\n\n  * `:timeout` - time in milliseconds to wait for each query call to finish. Defaults to 15 seconds.\n\n  * `:timezone` — which timezone to use when evaluating the schedule. To use a timezone other than\n    the default of \"Etc/UTC\" you *must* have a timezone database like [tz][tz] installed and\n    configured.\n\n[tz]: https://hexdocs.pm/tz","ref":"Oban.Plugins.Reindexer.html#module-options"},{"type":"type","title":"Oban.Plugins.Reindexer.option/0","doc":"","ref":"Oban.Plugins.Reindexer.html#t:option/0"},{"type":"module","title":"Oban.Config","doc":"The Config struct validates and encapsulates Oban instance state.\n\nTypically, you won't use the Config module directly. Oban automatically creates a Config struct\non initialization and passes it through to all supervised children with the `:conf` key.\n\nTo fetch a running Oban supervisor's config, see `Oban.config/1`.","ref":"Oban.Config.html"},{"type":"function","title":"Oban.Config.new/1","doc":"Generate a Config struct after normalizing and verifying Oban options.\n\nSee `Oban.start_link/1` for a comprehensive description of available options.","ref":"Oban.Config.html#new/1"},{"type":"function","title":"Example - Oban.Config.new/1","doc":"Generate a minimal config with only a `:repo`:\n\n    Oban.Config.new(repo: Oban.Test.Repo)","ref":"Oban.Config.html#new/1-example"},{"type":"function","title":"Oban.Config.validate/1","doc":"Verify configuration options.\n\nThis helper is used by `new/1`, and therefore by `Oban.start_link/1`, to verify configuration\noptions when an Oban supervisor starts. It is provided publicly to aid in configuration testing,\nas `test` config may differ from `prod` config.\n\n# Example\n\nValidating top level options:\n\n    iex> Oban.Config.validate(name: Oban)\n    :ok\n\n    iex> Oban.Config.validate(name: Oban, log: false)\n    :ok\n\n    iex> Oban.Config.validate(node: {:not, :binary})\n    {:error, \"expected :node to be a binary, got: {:not, :binary}\"}\n\n    iex> Oban.Config.validate(plugins: true)\n    {:error, \"invalid value for :plugins, expected :plugins to be a list, got: true\"}\n\nValidating plugin options:\n\n    iex> Oban.Config.validate(plugins: [{Oban.Plugins.Pruner, max_age: 60}])\n    :ok\n\n    iex> Oban.Config.validate(plugins: [{Oban.Plugins.Pruner, max_age: 0}])\n    {:error, \"invalid value for :plugins, expected :max_age to be a positive integer, got: 0\"}","ref":"Oban.Config.html#validate/1"},{"type":"type","title":"Oban.Config.t/0","doc":"","ref":"Oban.Config.html#t:t/0"},{"type":"behaviour","title":"Oban.Engine","doc":"Defines an Engine for job orchestration.\n\nEngines are responsible for all non-plugin database interaction, from inserting through\nexecuting jobs.\n\nOban ships with three Engine implementations:\n\n1. `Basic` — The default engine for development, production, and manual testing mode.\n2. `Inline` — Designed specifically for testing, it executes jobs immediately, in-memory, as\n   they are inserted.\n3. `Lite` - The engine for running Oban using SQLite3.\n\n> #### 🌟 SmartEngine {: .info}\n>\n> The Basic engine lacks advanced functionality such as global limits, rate limits, and\n> unique bulk insert. For those features and more, see the [`Smart` engine in Oban\n> Pro](https://getoban.pro/docs/pro/Oban.Pro.Engines.Smart.html).","ref":"Oban.Engine.html"},{"type":"callback","title":"Oban.Engine.cancel_all_jobs/2","doc":"Mark many `executing`, `available`, `scheduled` or `retryable` job as `cancelled` to prevent them\nfrom running.","ref":"Oban.Engine.html#c:cancel_all_jobs/2"},{"type":"callback","title":"Oban.Engine.cancel_job/2","doc":"Mark an `executing`, `available`, `scheduled` or `retryable` job as `cancelled` to prevent it\nfrom running.","ref":"Oban.Engine.html#c:cancel_job/2"},{"type":"callback","title":"Oban.Engine.check_meta/3","doc":"Format engine meta in a digestible format for queue inspection.","ref":"Oban.Engine.html#c:check_meta/3"},{"type":"callback","title":"Oban.Engine.complete_job/2","doc":"Record that a job completed successfully.","ref":"Oban.Engine.html#c:complete_job/2"},{"type":"callback","title":"Oban.Engine.discard_job/2","doc":"Transition a job to `discarded` and record an optional reason that it shouldn't be ran again.","ref":"Oban.Engine.html#c:discard_job/2"},{"type":"callback","title":"Oban.Engine.error_job/3","doc":"Record an executing job's errors and either retry or discard it, depending on whether it has\nexhausted its available attempts.","ref":"Oban.Engine.html#c:error_job/3"},{"type":"callback","title":"Oban.Engine.fetch_jobs/3","doc":"Fetch available jobs for the given queue, up to configured limits.","ref":"Oban.Engine.html#c:fetch_jobs/3"},{"type":"callback","title":"Oban.Engine.init/2","doc":"Initialize metadata for a queue engine.\n\nQueue metadata is used to identify and track subsequent actions such as fetching or staging\njobs.","ref":"Oban.Engine.html#c:init/2"},{"type":"callback","title":"Oban.Engine.insert_all_jobs/3","doc":"Insert multiple jobs into the database.","ref":"Oban.Engine.html#c:insert_all_jobs/3"},{"type":"callback","title":"Oban.Engine.insert_all_jobs/5","doc":"","ref":"Oban.Engine.html#c:insert_all_jobs/5"},{"type":"callback","title":"Oban.Engine.insert_job/3","doc":"Insert a job into the database.","ref":"Oban.Engine.html#c:insert_job/3"},{"type":"callback","title":"Oban.Engine.insert_job/5","doc":"","ref":"Oban.Engine.html#c:insert_job/5"},{"type":"callback","title":"Oban.Engine.prune_jobs/3","doc":"Delete completed, cancelled and discarded jobs.","ref":"Oban.Engine.html#c:prune_jobs/3"},{"type":"callback","title":"Oban.Engine.put_meta/4","doc":"Store the given key/value pair in the engine meta.","ref":"Oban.Engine.html#c:put_meta/4"},{"type":"callback","title":"Oban.Engine.refresh/2","doc":"Refresh a queue to indicate that it is still alive.","ref":"Oban.Engine.html#c:refresh/2"},{"type":"callback","title":"Oban.Engine.retry_all_jobs/2","doc":"Mark many jobs as `available`, adding attempts if already maxed out. Any jobs currently\n`available`, `executing` or `scheduled` should be ignored.","ref":"Oban.Engine.html#c:retry_all_jobs/2"},{"type":"callback","title":"Oban.Engine.retry_job/2","doc":"Mark a job as `available`, adding attempts if already maxed out. If the job is currently\n`available`, `executing` or `scheduled` it should be ignored.","ref":"Oban.Engine.html#c:retry_job/2"},{"type":"callback","title":"Oban.Engine.shutdown/2","doc":"Prepare a queue engine for shutdown.\n\nThe queue process is expected to stop processing new jobs after shutdown starts, though it may\ncontinue executing jobs that are already running.","ref":"Oban.Engine.html#c:shutdown/2"},{"type":"callback","title":"Oban.Engine.snooze_job/3","doc":"Reschedule an executing job to run some number of seconds in the future.","ref":"Oban.Engine.html#c:snooze_job/3"},{"type":"callback","title":"Oban.Engine.stage_jobs/3","doc":"Transition scheduled or retryable jobs to available prior to execution.","ref":"Oban.Engine.html#c:stage_jobs/3"},{"type":"type","title":"Oban.Engine.conf/0","doc":"","ref":"Oban.Engine.html#t:conf/0"},{"type":"type","title":"Oban.Engine.job/0","doc":"","ref":"Oban.Engine.html#t:job/0"},{"type":"type","title":"Oban.Engine.meta/0","doc":"","ref":"Oban.Engine.html#t:meta/0"},{"type":"type","title":"Oban.Engine.opts/0","doc":"","ref":"Oban.Engine.html#t:opts/0"},{"type":"type","title":"Oban.Engine.queryable/0","doc":"","ref":"Oban.Engine.html#t:queryable/0"},{"type":"type","title":"Oban.Engine.running/0","doc":"","ref":"Oban.Engine.html#t:running/0"},{"type":"type","title":"Oban.Engine.seconds/0","doc":"","ref":"Oban.Engine.html#t:seconds/0"},{"type":"behaviour","title":"Oban.Notifier","doc":"The `Notifier` coordinates listening for and publishing notifications for events in predefined\nchannels.\n\nOban functions such as `pause_queue`, `scale_queue`, and `cancel_job` all **require a connected\nnotifier to operate**. Use `status/1` to check the notifier's connectivity status and diagnose\nissues.","ref":"Oban.Notifier.html"},{"type":"behaviour","title":"Notifiers - Oban.Notifier","doc":"Every Oban supervision tree contains a notifier process, registered as `Oban.Notifier`, which is\nan implementation of the `Oban.Notifier` behaviour. \n\n* `Oban.Notifiers.Postgres` — A Postgres notifier that uses `LISTEN/NOTIFY` to broadcast\n  messages. This is the default.\n\n* `Oban.Notifiers.PG` — A process groups notifier that relies on Distributed Erlang to broadcast\n  messages.\n\n* [`Oban.Notifiers.Phoenix`](https://github.com/sorentwo/oban_notifiers_phoenix) — A notifier\n  that uses `Phoenix.PubSub` to broadcast messages. In addition to centralizing PubSub\n  communications, it opens up the possible transports to all PubSub adapters.","ref":"Oban.Notifier.html#module-notifiers"},{"type":"behaviour","title":"Channels - Oban.Notifier","doc":"All incoming notifications are relayed through the notifier to any processes listening on a\ngiven channel. Internally, Oban uses a variety of predefined channels with distinct\nresponsibilities:\n\n* `insert` — as jobs are inserted an event is published on the `insert` channel. Processes such\n  as queue producers use this as a signal to dispatch new jobs.\n\n* `leader` — messages regarding node leadership exchanged between peers\n\n* `signal` — instructions to take action, such as scale a queue or kill a running job, are sent\n  through the `signal` channel\n\n* `sonar` — periodic notification checks to monitor pubsub health and determine connectivity","ref":"Oban.Notifier.html#module-channels"},{"type":"behaviour","title":"Examples - Oban.Notifier","doc":"Broadcasting after a job is completed:\n\n    defmodule MyApp.Worker do\n      use Oban.Worker\n\n      @impl Oban.Worker\n      def perform(job) do\n        :ok = MyApp.do_work(job.args)\n\n        Oban.Notifier.notify(Oban, :my_app_jobs, %{complete: job.id})\n\n        :ok\n      end\n    end\n\nListening for job complete events from another process:\n\n    def insert_and_listen(args) do\n      :ok = Oban.Notifier.listen([:my_app_jobs])\n\n      {:ok, %{id: job_id} = job} =\n        args\n        |> MyApp.Worker.new()\n        |> Oban.insert()\n\n      receive do\n        {:notification, :my_app_jobs, %{\"complete\" => ^job_id}} ->\n          IO.puts(\"Other job complete!\")\n      after\n        30_000 ->\n          IO.puts(\"Other job didn't finish in 30 seconds!\")\n      end\n    end","ref":"Oban.Notifier.html#module-examples"},{"type":"function","title":"Oban.Notifier.listen/2","doc":"Register the current process to receive relayed messages for the provided channels.\n\nAll messages are received as `JSON` and decoded _before_ they are relayed to registered\nprocesses. Each registered process receives a three element notification tuple in the following\nformat:\n\n    {:notification, channel :: channel(), decoded :: map()}","ref":"Oban.Notifier.html#listen/2"},{"type":"function","title":"Example - Oban.Notifier.listen/2","doc":"Register to listen for all `:gossip` channel messages:\n\n    Oban.Notifier.listen(:gossip)\n\nListen for messages on multiple channels:\n\n    Oban.Notifier.listen([:gossip, :insert, :leader, :signal])\n\nListen for messages when using a custom Oban name:\n\n    Oban.Notifier.listen(MyApp.MyOban, [:gossip, :signal])","ref":"Oban.Notifier.html#listen/2-example"},{"type":"callback","title":"Oban.Notifier.listen/2","doc":"Register the current process to receive messages from one or more channels.","ref":"Oban.Notifier.html#c:listen/2"},{"type":"function","title":"Oban.Notifier.notify/3","doc":"Broadcast a notification to listeners on all nodes.\n\nNotifications are scoped to the configured `prefix`. For example, if there are instances running\nwith the `public` and `private` prefixes, a notification published in the `public` prefix won't\nbe picked up by processes listening with the `private` prefix.","ref":"Oban.Notifier.html#notify/3"},{"type":"function","title":"Example - Oban.Notifier.notify/3","doc":"Broadcast a gossip message:\n\n    Oban.Notifier.notify(:my_channel, %{message: \"hi!\"})\n\nBroadcast multiple messages at once:\n\n    Oban.Notifier.notify(:my_channel, [%{message: \"hi!\"}, %{message: \"there\"}])\n\nBroadcast using a custom instance name:\n\n    Oban.Notifier.notify(MyOban, :my_channel, %{message: \"hi!\"})","ref":"Oban.Notifier.html#notify/3-example"},{"type":"callback","title":"Oban.Notifier.notify/3","doc":"Broadcast a notification to all subscribers of a channel.","ref":"Oban.Notifier.html#c:notify/3"},{"type":"callback","title":"Oban.Notifier.start_link/1","doc":"Starts a notifier instance.","ref":"Oban.Notifier.html#c:start_link/1"},{"type":"function","title":"Oban.Notifier.status/1","doc":"Check a notifier's connectivity level to see whether it's able to publish or receive messages\nfrom other nodes.\n\nOban functions such as `pause_queue`, `scale_queue`, and `cancel_job` all require a connected\nnotifier to operate. Each Oban instance runs a persistent process to monitor connectivity,\nwhich is exposed by this function.","ref":"Oban.Notifier.html#status/1"},{"type":"function","title":"Statuses - Oban.Notifier.status/1","doc":"* `:unknown` — This is the default state on start before the notifier has time to determine the\n  appropriate status.\n\n* `:isolated` — The notifier isn't receiving any messages.\n\n  The notifier may be connected to a database but `:isolated` and unable to receive other\n  message and unable to receive outside messages. Typically, this is the case for the default\n  `Postgres` notifier while testing or behind a connection pooler.\n\n* `:solitary` — The notifier is only receiving messages from itself. This may be the case for\n  the `PG` notifier when Distributed Erlang nodes aren't connected, in development, or in\n  production deployments that only run a single node. If you're running multiple nodes in production\n  and the status is `:solitary`, there's a connectivity issue.\n\n* `:clustered` — The notifier is connected and able to receive messages from other nodes. The\n  `Postgres` notifier is considered clustered if it can receive notifications, while the PG\n  notifier requires a functional Distributed Erlang cluster.","ref":"Oban.Notifier.html#status/1-statuses"},{"type":"function","title":"Examples - Oban.Notifier.status/1","doc":"Check the notifier's pubsub status:\n\n    Oban.Notifier.status()\n\nCheck the status for a custom instance:\n\n    Oban.Notifier.status(MyOban)","ref":"Oban.Notifier.html#status/1-examples"},{"type":"function","title":"Oban.Notifier.unlisten/2","doc":"Unregister the current process from receiving relayed messages on provided channels.","ref":"Oban.Notifier.html#unlisten/2"},{"type":"function","title":"Example - Oban.Notifier.unlisten/2","doc":"Stop listening for messages on the `:gossip` channel:\n\n    Oban.Notifier.unlisten(:gossip)\n\nStop listening for messages on multiple channels:\n\n    Oban.Notifier.unlisten([:insert, :gossip])\n\nStop listening for messages when using a custom Oban name:\n\n    Oban.Notifier.unlisten(MyApp.MyOban, [:gossip])","ref":"Oban.Notifier.html#unlisten/2-example"},{"type":"callback","title":"Oban.Notifier.unlisten/2","doc":"Unregister current process from channels.","ref":"Oban.Notifier.html#c:unlisten/2"},{"type":"type","title":"Oban.Notifier.channel/0","doc":"","ref":"Oban.Notifier.html#t:channel/0"},{"type":"type","title":"Oban.Notifier.name_or_conf/0","doc":"","ref":"Oban.Notifier.html#t:name_or_conf/0"},{"type":"type","title":"Oban.Notifier.payload/0","doc":"","ref":"Oban.Notifier.html#t:payload/0"},{"type":"type","title":"Oban.Notifier.pubsub_status/0","doc":"","ref":"Oban.Notifier.html#t:pubsub_status/0"},{"type":"behaviour","title":"Oban.Peer","doc":"The `Peer` module maintains leadership for a particular Oban instance within a cluster.\n\nLeadership is used by plugins, primarily, to prevent duplicate work across nodes. For example,\nonly the leader's `Cron` plugin will try inserting new jobs. You can use peer leadership to\nextend Oban with custom plugins, or even within your own application.\n\nNote a few important details about how peer leadership operates:\n\n* Each peer checks for leadership at a 30 second interval. When the leader exits it broadcasts a\n  message to all other peers to encourage another one to assume leadership.\n\n* Each Oban instance supervises a distinct `Oban.Peer` instance. That means that with multiple\n  Oban instances on the same node one instance may be the leader, while the others aren't.\n\n* Without leadership, global plugins (Cron, Lifeline, Stager, etc.), will not run on any node.","ref":"Oban.Peer.html"},{"type":"behaviour","title":"Available Peer Implementations - Oban.Peer","doc":"There are two built-in peering modules:\n\n* `Oban.Peers.Postgres` — uses table-based leadership through the `oban_peers` table and works\n  in any environment, with or without clustering. Only one node (per instance name) will have a\n  row in the peers table, that node is the leader. This is the default.\n\n* `Oban.Peers.Global` — coordinates global locks through distributed Erlang, requires\n  distributed Erlang.\n\nYou can specify the peering module to use in your Oban configuration:\n\n    config :my_app, Oban,\n      peer: Oban.Peers.Postgres, # default value\n      ...\n\nIf in doubt, you can call `Oban.config()` to see which module is being used.","ref":"Oban.Peer.html#module-available-peer-implementations"},{"type":"behaviour","title":"Examples - Oban.Peer","doc":"Check leadership for the default Oban instance:\n\n    Oban.Peer.leader?()\n    # => true\n\nThat is identical to using the name `Oban`:\n\n    Oban.Peer.leader?(Oban)\n    # => true\n\nCheck leadership for a couple of instances:\n\n    Oban.Peer.leader?(Oban.A)\n    # => true\n\n    Oban.Peer.leader?(Oban.B)\n    # => false","ref":"Oban.Peer.html#module-examples"},{"type":"callback","title":"Oban.Peer.leader?/1","doc":"Check whether the current peer instance leads the cluster.","ref":"Oban.Peer.html#c:leader?/1"},{"type":"function","title":"Oban.Peer.leader?/2","doc":"Check whether the current instance leads the cluster.","ref":"Oban.Peer.html#leader?/2"},{"type":"function","title":"Example - Oban.Peer.leader?/2","doc":"Check leadership for the default Oban instance:\n\n    Oban.Peer.leader?()\n    # => true\n\nCheck leadership for an alternate instance named `Oban.Private`:\n\n    Oban.Peer.leader?(Oban.Private)\n    # => true","ref":"Oban.Peer.html#leader?/2-example"},{"type":"callback","title":"Oban.Peer.start_link/1","doc":"Starts a peer instance.","ref":"Oban.Peer.html#c:start_link/1"},{"type":"type","title":"Oban.Peer.option/0","doc":"","ref":"Oban.Peer.html#t:option/0"},{"type":"behaviour","title":"Oban.Plugin","doc":"Defines a shared behaviour for Oban plugins.\n\nIn addition to implementing the Plugin behaviour, all plugins **must** be a `GenServer`, `Agent`, or\nanother OTP compliant module.","ref":"Oban.Plugin.html"},{"type":"behaviour","title":"Example - Oban.Plugin","doc":"Defining a basic plugin that satisfies the minimum behaviour:\n\n    defmodule MyPlugin do\n      @behaviour Oban.Plugin\n\n      use GenServer\n\n      @impl Oban.Plugin\n      def start_link(opts) do\n        GenServer.start_link(__MODULE__, opts, name: opts[:name])\n      end\n\n      @impl Oban.Plugin\n      def validate(opts) do\n        if is_atom(opts[:mode])\n          :ok\n        else\n          {:error, \"expected opts to have a :mode key\"}\n        end\n      end\n\n      @impl GenServer\n      def init(opts) do\n        case validate(opts) do\n          :ok -> {:ok, opts}\n          {:error, reason} -> {:stop, reason}\n        end\n      end\n    end","ref":"Oban.Plugin.html#module-example"},{"type":"callback","title":"Oban.Plugin.start_link/1","doc":"Starts a Plugin process linked to the current process.\n\nPlugins are typically started as part of an Oban supervision tree and will receive the current\nconfiguration as `:conf`, along with a `:name` and any other provided options.","ref":"Oban.Plugin.html#c:start_link/1"},{"type":"callback","title":"Oban.Plugin.validate/1","doc":"Validate the structure, presence, or values of keyword options.","ref":"Oban.Plugin.html#c:validate/1"},{"type":"type","title":"Oban.Plugin.option/0","doc":"","ref":"Oban.Plugin.html#t:option/0"},{"type":"module","title":"Oban.Registry","doc":"Local process storage for Oban instances.","ref":"Oban.Registry.html"},{"type":"function","title":"Oban.Registry.config/1","doc":"Fetch the config for an Oban supervisor instance.","ref":"Oban.Registry.html#config/1"},{"type":"function","title":"Example - Oban.Registry.config/1","doc":"Get the default instance config:\n\n    Oban.Registry.config(Oban)\n\nGet config for a custom named instance:\n\n    Oban.Registry.config(MyApp.Oban)","ref":"Oban.Registry.html#config/1-example"},{"type":"function","title":"Oban.Registry.lookup/2","doc":"Find the `{pid, value}` pair for a registered Oban process.","ref":"Oban.Registry.html#lookup/2"},{"type":"function","title":"Example - Oban.Registry.lookup/2","doc":"Get the default instance config:\n\n    Oban.Registry.lookup(Oban)\n\nGet a supervised module's pid:\n\n    Oban.Registry.lookup(Oban, Oban.Notifier)","ref":"Oban.Registry.html#lookup/2-example"},{"type":"function","title":"Oban.Registry.via/3","doc":"Build a via tuple suitable for calls to a supervised Oban process.","ref":"Oban.Registry.html#via/3"},{"type":"function","title":"Example - Oban.Registry.via/3","doc":"For an Oban supervisor:\n\n    Oban.Registry.via(Oban)\n\nFor a supervised module:\n\n    Oban.Registry.via(Oban, Oban.Notifier)\n\nFor a plugin:\n\n    Oban.Registry.via(Oban, {:plugin, Oban.Plugins.Cron})","ref":"Oban.Registry.html#via/3-example"},{"type":"function","title":"Oban.Registry.whereis/2","doc":"Returns the pid of a supervised Oban process, or `nil` if the process can't be found.","ref":"Oban.Registry.html#whereis/2"},{"type":"function","title":"Example - Oban.Registry.whereis/2","doc":"Get the Oban supervisor's pid:\n\n    Oban.Registry.whereis(Oban)\n\nGet a supervised module's pid:\n\n    Oban.Registry.whereis(Oban, Oban.Notifier)\n\nGet the pid for a plugin:\n\n    Oban.Registry.whereis(Oban, {:plugin, MyApp.Oban.Plugin})\n\nGet the pid for a queue's producer:\n\n    Oban.Registry.whereis(Oban, {:producer, \"default\"})","ref":"Oban.Registry.html#whereis/2-example"},{"type":"type","title":"Oban.Registry.key/0","doc":"","ref":"Oban.Registry.html#t:key/0"},{"type":"type","title":"Oban.Registry.role/0","doc":"","ref":"Oban.Registry.html#t:role/0"},{"type":"type","title":"Oban.Registry.value/0","doc":"","ref":"Oban.Registry.html#t:value/0"},{"type":"module","title":"Oban.Repo","doc":"Wrappers around `Ecto.Repo` and `Ecto.Adapters.SQL` callbacks.\n\nEach function resolves the correct repo instance and sets options such as `prefix` and `log`\naccording to `Oban.Config`.\n\n> #### Meant for Extending Oban {: .warning}\n>\n> These functions should only be used when working with a repo inside engines, plugins, or other\n> extensions for Oban. Favor using your application's repo directly when querying `Oban.Job`\n> from your workers.","ref":"Oban.Repo.html"},{"type":"module","title":"Examples - Oban.Repo","doc":"The first argument for every function must be an `Oban.Config` struct. Many functions pass\nconfiguration around as a `conf` key, and it can always be fetched with `Oban.config/1`. This\ndemonstrates fetching the default instance config and querying all jobs:\n\n    Oban\n    |> Oban.config()\n    |> Oban.Repo.all(Oban.Job)","ref":"Oban.Repo.html#module-examples"},{"type":"function","title":"Oban.Repo.aggregate/4","doc":"Wraps `c:Ecto.Repo.aggregate/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#aggregate/4"},{"type":"function","title":"Oban.Repo.all/3","doc":"Wraps `c:Ecto.Repo.all/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#all/3"},{"type":"function","title":"Oban.Repo.checkout/3","doc":"Wraps `c:Ecto.Repo.checkout/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#checkout/3"},{"type":"function","title":"Oban.Repo.config/1","doc":"Wraps `c:Ecto.Repo.config/0` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#config/1"},{"type":"function","title":"Oban.Repo.default_options/1","doc":"The default values extracted from `Oban.Config` for use in all queries with options.","ref":"Oban.Repo.html#default_options/1"},{"type":"function","title":"Oban.Repo.default_options/2","doc":"Wraps `c:Ecto.Repo.default_options/1` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#default_options/2"},{"type":"function","title":"Oban.Repo.delete/3","doc":"Wraps `c:Ecto.Repo.delete/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#delete/3"},{"type":"function","title":"Oban.Repo.delete!/3","doc":"Wraps `c:Ecto.Repo.delete!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#delete!/3"},{"type":"function","title":"Oban.Repo.delete_all/3","doc":"Wraps `c:Ecto.Repo.delete_all/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#delete_all/3"},{"type":"function","title":"Oban.Repo.exists?/3","doc":"Wraps `c:Ecto.Repo.exists?/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#exists?/3"},{"type":"function","title":"Oban.Repo.get/4","doc":"Wraps `c:Ecto.Repo.get/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get/4"},{"type":"function","title":"Oban.Repo.get!/4","doc":"Wraps `c:Ecto.Repo.get!/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get!/4"},{"type":"function","title":"Oban.Repo.get_by/4","doc":"Wraps `c:Ecto.Repo.get_by/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get_by/4"},{"type":"function","title":"Oban.Repo.get_by!/4","doc":"Wraps `c:Ecto.Repo.get_by!/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get_by!/4"},{"type":"function","title":"Oban.Repo.get_dynamic_repo/1","doc":"Wraps `c:Ecto.Repo.get_dynamic_repo/0` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#get_dynamic_repo/1"},{"type":"function","title":"Oban.Repo.in_transaction?/1","doc":"Wraps `c:Ecto.Repo.in_transaction?/0` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#in_transaction?/1"},{"type":"function","title":"Oban.Repo.insert/3","doc":"Wraps `c:Ecto.Repo.insert/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert/3"},{"type":"function","title":"Oban.Repo.insert!/3","doc":"Wraps `c:Ecto.Repo.insert!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert!/3"},{"type":"function","title":"Oban.Repo.insert_all/4","doc":"Wraps `c:Ecto.Repo.insert_all/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert_all/4"},{"type":"function","title":"Oban.Repo.insert_or_update/3","doc":"Wraps `c:Ecto.Repo.insert_or_update/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert_or_update/3"},{"type":"function","title":"Oban.Repo.insert_or_update!/3","doc":"Wraps `c:Ecto.Repo.insert_or_update!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#insert_or_update!/3"},{"type":"function","title":"Oban.Repo.load/3","doc":"Wraps `c:Ecto.Repo.load/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#load/3"},{"type":"function","title":"Oban.Repo.one/3","doc":"Wraps `c:Ecto.Repo.one/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#one/3"},{"type":"function","title":"Oban.Repo.one!/3","doc":"Wraps `c:Ecto.Repo.one!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#one!/3"},{"type":"function","title":"Oban.Repo.preload/4","doc":"Wraps `c:Ecto.Repo.preload/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#preload/4"},{"type":"function","title":"Oban.Repo.put_dynamic_repo/2","doc":"Wraps `c:Ecto.Repo.put_dynamic_repo/1` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#put_dynamic_repo/2"},{"type":"function","title":"Oban.Repo.query/4","doc":"Wraps `Ecto.Adapters.SQL.Repo.query/4` with an added `Oban.Config` argument.","ref":"Oban.Repo.html#query/4"},{"type":"function","title":"Oban.Repo.query!/4","doc":"Wraps `Ecto.Adapters.SQL.Repo.query!/4` with an added `Oban.Config` argument.","ref":"Oban.Repo.html#query!/4"},{"type":"function","title":"Oban.Repo.reload/3","doc":"Wraps `c:Ecto.Repo.reload/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#reload/3"},{"type":"function","title":"Oban.Repo.reload!/3","doc":"Wraps `c:Ecto.Repo.reload!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#reload!/3"},{"type":"function","title":"Oban.Repo.rollback/2","doc":"Wraps `c:Ecto.Repo.rollback/1` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#rollback/2"},{"type":"function","title":"Oban.Repo.stream/3","doc":"Wraps `c:Ecto.Repo.stream/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#stream/3"},{"type":"function","title":"Oban.Repo.to_sql/3","doc":"Wraps `Ecto.Adapters.SQL.Repo.to_sql/2` with an added `Oban.Config` argument.","ref":"Oban.Repo.html#to_sql/3"},{"type":"function","title":"Oban.Repo.transaction/3","doc":"Wraps `c:Ecto.Repo.transaction/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#transaction/3"},{"type":"function","title":"Oban.Repo.update/3","doc":"Wraps `c:Ecto.Repo.update/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#update/3"},{"type":"function","title":"Oban.Repo.update!/3","doc":"Wraps `c:Ecto.Repo.update!/2` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#update!/3"},{"type":"function","title":"Oban.Repo.update_all/4","doc":"Wraps `c:Ecto.Repo.update_all/3` with an additional `Oban.Config` argument.","ref":"Oban.Repo.html#update_all/4"},{"type":"module","title":"Oban.Notifiers.PG","doc":"A [PG (Process Groups)][pg] based notifier implementation that runs with Distributed Erlang.\n\n> #### Distributed Erlang {: .info}\n>\n> PG requires a functional [Distributed Erlang][de] cluster to broadcast between nodes. If your\n> application isn't clustered, then you should consider an alternative notifier such as\n> `Oban.Notifiers.Postgres`","ref":"Oban.Notifiers.PG.html"},{"type":"module","title":"Usage - Oban.Notifiers.PG","doc":"Specify the `PG` notifier in your Oban configuration:\n\n```elixir\nconfig :my_app, Oban,\n  notifier: Oban.Notifiers.PG,\n  ...\n```\n\nBy default, all Oban instances using the same `prefix` option will receive notifications from\neach other. You can use the `namespace` option to separate instances that are in the same\ncluster _without_ changing the prefix:\n\n```elixir\nconfig :my_app, Oban,\n  notifier: {Oban.Notifiers.PG, namespace: :custom_namespace}\n  ...\n```\n\nThe namespace may be any term.\n\n[pg]: https://www.erlang.org/doc/man/pg.html\n[de]: https://elixir-lang.org/getting-started/mix-otp/distributed-tasks.html#our-first-distributed-code","ref":"Oban.Notifiers.PG.html#module-usage"},{"type":"function","title":"Oban.Notifiers.PG.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Oban.Notifiers.PG.html#child_spec/1"},{"type":"module","title":"Oban.Notifiers.Postgres","doc":"A Postgres LISTEN/NOTIFY based Notifier.\n\n> #### Connection Pooling {: .info}\n>\n> Postgres PubSub is fine for most applications, but it doesn't work with connection poolers\n> like [PgBouncer][pgb] when configured in _transaction_ or _statement_ mode, which is\n> standard. Notifications are required for some core Oban functionality, and you should\n> consider using an alternative notifier such as `Oban.Notifiers.PG`.","ref":"Oban.Notifiers.Postgres.html"},{"type":"module","title":"Usage - Oban.Notifiers.Postgres","doc":"Specify the `Postgres` notifier in your Oban configuration:\n\n    config :my_app, Oban,\n      notifier: Oban.Notifiers.Postgres,\n      ...","ref":"Oban.Notifiers.Postgres.html#module-usage"},{"type":"module","title":"Transactions and Testing - Oban.Notifiers.Postgres","doc":"The notifications system is built on PostgreSQL's `LISTEN/NOTIFY` functionality. Notifications\nare only delivered **after a transaction completes** and are de-duplicated before publishing.\nTypically, applications run Ecto in sandbox mode while testing, but sandbox mode wraps each test\nin a separate transaction that's rolled back after the test completes. That means the\ntransaction is never committed, which prevents delivering any notifications.\n\nTo test using notifications you must run Ecto without sandbox mode enabled, or use\n`Oban.Notifiers.PG` instead.\n\n[pgb]: https://www.pgbouncer.org/","ref":"Oban.Notifiers.Postgres.html#module-transactions-and-testing"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_call/3","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_call/3"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_connect/1","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_connect/1"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_disconnect/1","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_disconnect/1"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_info/2","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_info/2"},{"type":"function","title":"Oban.Notifiers.Postgres.handle_result/2","doc":"","ref":"Oban.Notifiers.Postgres.html#handle_result/2"},{"type":"function","title":"Oban.Notifiers.Postgres.init/1","doc":"","ref":"Oban.Notifiers.Postgres.html#init/1"},{"type":"module","title":"Oban.Peers.Global","doc":"A cluster based peer that coordinates through a distributed registry.\n\nLeadership is coordinated through global locks. It requires a functional distributed Erlang\ncluster, without one global plugins (Cron, Lifeline, etc.) will not function correctly.","ref":"Oban.Peers.Global.html"},{"type":"module","title":"Usage - Oban.Peers.Global","doc":"Specify the `Global` peer in your Oban configuration.\n\n    config :my_app, Oban,\n      peer: Oban.Peers.Global,\n      ...","ref":"Oban.Peers.Global.html#module-usage"},{"type":"module","title":"Oban.Peers.Postgres","doc":"A Postgres based peer that coordinates centrally through a database table.\n\nPostgres peers don't require clustering through distributed Erlang or any other\ninterconnectivity between nodes. Leadership is coordinated through the `oban_peers` table in\nyour database. With a standard Oban config the `oban_peers` table will only have one row, and\nthat node is the leader.\n\nApplications that run multiple Oban instances will have one row per instance. For example, an\numbrella application that runs `Oban.A` and `Oban.B` will have two rows in `oban_peers`.","ref":"Oban.Peers.Postgres.html"},{"type":"module","title":"Usage - Oban.Peers.Postgres","doc":"Specify the `Postgres` peer in your Oban configuration.\n\n    config :my_app, Oban,\n      peer: Oban.Peers.Postgres,\n      ...","ref":"Oban.Peers.Postgres.html#module-usage"},{"type":"module","title":"Oban.Engines.Basic","doc":"The default engine for use with Postgres databases.","ref":"Oban.Engines.Basic.html"},{"type":"module","title":"Usage - Oban.Engines.Basic","doc":"This is the default engine, no additional configuration is necessary:\n\n    Oban.start_link(repo: MyApp.Repo, queues: [default: 10])","ref":"Oban.Engines.Basic.html#module-usage"},{"type":"module","title":"Oban.Engines.Inline","doc":"A testing-specific engine that's used when Oban's started with `testing: :inline`.","ref":"Oban.Engines.Inline.html"},{"type":"module","title":"Usage - Oban.Engines.Inline","doc":"This is meant for testing and shouldn't be configured directly:\n\n    Oban.start_link(repo: MyApp.Repo, testing: :inline)","ref":"Oban.Engines.Inline.html#module-usage"},{"type":"module","title":"Oban.Engines.Lite","doc":"An engine for running Oban with SQLite3.","ref":"Oban.Engines.Lite.html"},{"type":"module","title":"Usage - Oban.Engines.Lite","doc":"Start an `Oban` instance using the `Lite` engine:\n\n    Oban.start_link(\n      engine: Oban.Engines.Lite,\n      queues: [default: 10],\n      repo: MyApp.Repo\n    )","ref":"Oban.Engines.Lite.html#module-usage"},{"type":"exception","title":"Oban.CrashError","doc":"Wraps unhandled exits and throws that occur during job execution.","ref":"Oban.CrashError.html"},{"type":"exception","title":"Oban.PerformError","doc":"Wraps the reason returned by `{:error, reason}`, `{:cancel, reason}`, or `{:discard, reason}` in\na proper exception.\n\nThe original return tuple is available in the `:reason` key.","ref":"Oban.PerformError.html"},{"type":"exception","title":"Oban.TimeoutError","doc":"Returned when a job is terminated early due to a custom timeout.","ref":"Oban.TimeoutError.html"},{"type":"extras","title":"Changelog","doc":"# Changelog for Oban v2.17\n\n_🌟 Looking for changes to Web or Pro? Check the [Oban.Pro Changelog][opc] or the [Oban.Web\nChangelog][owc]. 🌟_\n\nThis release includes an optional database migration to disable triggers and relax priority\nchecks. See the [v2.17 upgrade guide](v2-17.html) for step-by-step instructions.","ref":"changelog.html"},{"type":"extras","title":"📟 Universal Insert Notifications - Changelog","doc":"Historically, Oban used database triggers to emit a notification after a job is inserted. That\nallowed jobs to execute sooner, without waiting up to a second until the next poll event. Those\ntriggers and subsequent notifications added some overhead to database operations bulk inserts into\nthe same queue, despite deduplication logic in the trigger. Even worse, trigger notifications\ndidn't work behind connection poolers and were restricted to the Postgres notifier.\n\nNow insert notifications have moved out of the database and into application code, so it's\npossible to disable triggers without running database migrations, and they work for _any_\nnotifier, not just Postgres.\n\nDisable notifications with the `insert_trigger` option if sub-second job execution isn't important\nor you'd like to reduce PubSub chatter:\n\n```elixir\nconfig :my_app, Oban,\n  insert_trigger: false,\n  ...\n```","ref":"changelog.html#universal-insert-notifications"},{"type":"extras","title":"🧑‍🏭 Worker Conveniences - Changelog","doc":"Workers received a few quality of life improvements to make defining `unique` behaviour more\nexpressive and intuitive.\n\nFirst, it's now possible to define a job's unique period with time units like `{1, :minute}` or\n`{2, :hours}`, just like a job's `:schedule_in` option:\n\n```elixir\nuse Oban.Worker, unique: [period: {5, :minutes}]\n```\n\nSecond, you can set the `replace` option in `use Oban.Worker` rather than in an overridden `new/2`\nor as a runtime option. For example, to enable updating a job's `scheduled_at` value on unique\nconflict:\n\n```elixir\nuse Oban.Worker, unique: [period: 60], replace: [scheduled: [:scheduled_at]]\n```","ref":"changelog.html#worker-conveniences"},{"type":"extras","title":"🐦‍🔥 Oban Phoenix Notifier - Changelog","doc":"The new [`oban_notifiers_phoenix` package][onp] allows Oban to share a Phoenix application's\nPubSub for notifications. In addition to centralizing PubSub communications, it opens up the\npossible transports to all PubSub adapters. As Oban already provides `Postgres` and `PG`\n(Distributed Erlang) notifiers, the new package primarily enables Redis notifications.\n\n```elixir\nconfig :my_app, Oban,\n  notifier: {Oban.Notifiers.Phoenix, pubsub: MyApp.PubSub},\n  ...\n```\n\n[onp]: https://github.com/sorentwo/oban_notifiers_phoenix","ref":"changelog.html#oban-phoenix-notifier"},{"type":"extras","title":"🎚️ Ten Levels of Job Priority - Changelog","doc":"Job priority may now be set to values between 0 (highest) and 9 (lowest). This increases the\nrange from 4 to 10 possible priorities, giving applications much finer control over execution\norder.\n\n```elixir\nargs\n|> MyApp.PrioritizedWorker.new(priority: 9)\n|> Oban.insert()\n```","ref":"changelog.html#ten-levels-of-job-priority"},{"type":"extras","title":"v2.17.12 — 2024-06-28 - Changelog","doc":"","ref":"changelog.html#v2-17-12-2024-06-28"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Basic] Return `{:ok, job}` tuple on unique advisory lock conflict.\n\n  The advisory lock clause only returned the job, not a success tuple.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.11 — 2024-06-25 - Changelog","doc":"","ref":"changelog.html#v2-17-11-2024-06-25"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Oban] Handle deprecation warnings from Elixir 1.17\n\n- [Notifier] Prevent noisy logging about switching between modes.\n\n  There's an apparent race condition in Sonar between pruning stale nodes on `:ping` and updating\n  the status after a notification. This primarily happens in development for two reasons:\n\n  1. Development laptops are most prone to time warp because of system sleep.\n  2. Apps only run a single node in development.\n\n  Using `monotonic_time/1` instead of `system_time/1` guards against clock drift/time warp\n  effects.\n\n- [Stager] Prevent notification status timeouts from bubbling into the Stager.\n\n  A clogged Ecto pool could cause cascading errors on startup due to a sequence of calls between\n  the `Notifier`, `Sonar`, and `Stager`.\n\n  1. `Sonar` sends a notification in `handle_continue` on startup.\n  2. The notification is blocked while the `Notifier` waits for a connection from the Ecto pool.\n  3. `Stager` checks for the connection status on startup, which would eventually time out because\n     the `Sonar` hadn't finished initializing.\n  4. The `Stager` crashes from the timeout error.\n\n  This makes the following changes to prevent this sequence of events:\n\n  1. The `Stager` no longer gets the sonar status during startup.\n  2. The `Notifier` catches timeout errors from `Sonar` checks, warns about it, then returns an\n     `:unknown` status.\n\n- [Engine] Defensively check the process dictionary during inline testing.\n\n  Not all processes are guaranteed to return a value for the process dictionary. Sometimes a value\n  was missing during inline testing, which would crash the test.\n\n- [Basic] Set `conflict?` flag when encountering a unique advisory lock.\n\n  The `conflict?` flag wasn't set when inserting a unique job was blocked by an advisory lock. Now\n  the flag is set on either a fetched duplicate, or when the advisory lock is set.\n\n- [Job] Correct `replace_by_state_option` type by switching from keyword to tuples.\n\n- [Config] Correctly type `shutdown_grace_period` as an `integer` rather than a `timeout`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.10 — 2024-04-26 - Changelog","doc":"","ref":"changelog.html#v2-17-10-2024-04-26"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Oban] Make all generated functions from `use Oban` overridable.\n\n  Now the functions generated by `use Oban` are all marked with `defoverridable` for\n  extensibility.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Testing] Use `$callers` rather than `$ancestors` for ancestry tree check.\n\n  We care about Tasks for inline testing checks, not normal supervision tree ancestry. The\n  `$callers` entry is the appropriate mechanism to find the [trail of calling processes][trail]:\n\n  [trail]: https://hexdocs.pm/elixir/1.16.2/Task.html#module-ancestor-and-caller-tracking","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.9 — 2024-04-20 - Changelog","doc":"","ref":"changelog.html#v2-17-9-2024-04-20"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Testing] Check process ancestry tree for `with_testing_mode` override.\n\n  Cascade the `with_testing_mode` block to nested processes that make use of `:$ancestry` in the\n  process dictionary, i.e. tasks. Now enqueuing a job within spawned processes like `Task.async`\n  or `Task.async_stream` will honor the testing mode specified in `with_testing_mode/2`.\n\n- [PG] Support alternative namespacing in `PG` notifier\n\n  By default, all Oban instances using the same `prefix` option would receive notifications from\n  each other. Now you can use the `namespace` option to separate instances that are in the same\n  cluster _without_ changing the `prefix`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Oban] Restore zero arity version of `pause_all_queues/0`\n\n  Both pause and resume variants lost their default argument in a refactor that shifted around\n  guard clauses.\n\n- [Oban] Add `:oban_draining` to process dict while draining\n\n  The flag marks the test process while draining to give hints to the executor and engines. It\n  fixes an incompatibility between `Oban.drain_queue/2` and Pro's `Testing.drain_jobs/2`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.8 — 2024-04-07 - Changelog","doc":"","ref":"changelog.html#v2-17-8-2024-04-07"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Backoff] Backoff retry on DBConnection and Postgrex errors from GenServer calls.\n\n  GenServer calls that result in a `ConnectionError` or `Postgrex.Error` should also be caught and\n  retried rather than crashing on the first attempt.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Notifier] Check for a live notifier process and propagate notify errors.\n\n  The `Notifier.notify/1` spec showed it would always return `:ok`, but that wasn't the case when\n  the notifier was disconnected or the process was no longer running. Now an error tuple is\n  returned when a notifier process isn't running.\n\n  This situation happened most frequently during shutdown, particularly from external usage of the\n  Notifier like an application or the `oban_met` package.\n\n  In addition, the errors bubble up through top level `Oban` functions like `scale_queue/1`,\n  `pause_queue/1`, etc. to indicate that the operation can't actually succeed.\n\n- [Peers.Postgres] Rescue `DBConnection.ConnectionError` in peer leadership check.\n\n  Previously, only `Postgrex.Error` exceptions were rescued and other standard connection errors\n  were ignored, crashing the Peer. Because leadership is checked immediately after the peer\n  initializes, any connection issues would trigger a crash loop that could bring down the rest of\n  the supervision tree.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.7 — 2024-03-25 - Changelog","doc":"","ref":"changelog.html#v2-17-7-2024-03-25"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Notifier] Prevent Sonar from running in `:testing` modes.\n\n  Sonar has no purpose during tests, and it can cause sandbox issues when tests run with the\n  Postgres notifier.\n\n- [Oban] Correctly handle pause and resume all with opts.\n\n  The primary clause had two default arguments and it was impossible to call `pause_all_queues/1`\n  or `resume_all_queues/1` with opts and no name.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.6 — 2024-03-01 - Changelog","doc":"","ref":"changelog.html#v2-17-6-2024-03-01"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Cron] Include cron indicator and original cron expression in meta.\n\n  When the Cron plugin inserts jobs the original, unnormalized cron expression is now stored in a\n  job's meta under the `cron_expr` key. For compatibility with Pro's `DynamicCron`, meta also has\n  `cron: true` injected.\n\n- [Worker] Change `backoff/1` spec to allow immediate rescheduling by returning `0`\n\n  The callback now specifies a `non_neg_integer` to allow retrying 0 seconds into the future. This\n  matches the abilitiy to use `{:snooze, 0}`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Notifier] Revert using single connection to deliver notices.\n\n  Production use of the single notifier connection in various environments showed timeouts from\n  bottlenecks in the single connection could crash the connection and start new ones, leaving\n  extra idle processes. In addition, Use interpolated NOTIFY instead of pg_notify with a JSON\n  argument added parsing load because it could no longer use prepared statements.\n\n- [Worker] Apply custom backoff on timeout and unhandled exit\n\n  A worker's custom `backoff/1` wasn't applied after a `TimeoutError` or unhandled exit. That's\n  because the producer stores the executor struct _before_ resolving the worker module. Now the\n  module is resolved again to ensure custom `backoff/1` are used.\n\n- [Job] Revert setting the default priority key in Job schema.\n\n  The Ecto default was removed to allow overriding the default in the database and it was\n  purposefully removed.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.5 — 2024-02-25 - Changelog","doc":"","ref":"changelog.html#v2-17-5-2024-02-25"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Notifier] Use the Postgres notifier's connection to deliver notifications.\n\n  The Postgres notifier holds a single connection for listening and relaying messages. However, it\n  wasn't used to dispatch messages; that was left to queries through the Ecto pool. Those queries\n  were noisy and put unnecessary load on the pool, particularly from insert notifications.\n\n  Now notifications are delivered through the notifier's connection—they don't require a pool\n  checkout, and they won't clutter Ecto logs or telemetry.\n\n- [Engine] Emit insert trigger notification directly from `Engine` callbacks.\n\n  Notifications are now sent from the engine, within the `insert_*` telemetry block, so the timing\n  impact is visible. In addition, notifications aren't emitted for `scheduled` jobs, as there's\n  nothing ready for producers to fetch.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Notifier] Track and compare sonar pings using the correct time unit.\n\n  The notifier's status tracker pruned stale nodes using mismatched time units, causing constant\n  status change events despite nothing changing. This ensures the recorded and compared times are\n  both milliseconds, not a mixture of seconds and native time.\n\n- [Cron] Retain `@reboot` cron entries until node becomes leader.\n\n  With rolling deploys it is frequent that a node isn't the leader the first time cron evaluates.\n  However, `@reboot` expressions were discarded after the first run, which prevented reboots from\n  being inserted when the node acquired leadership.\n\n- [Oban] Require Ecto v3.10 to support `materialized` flag added in the previous patch.\n\n  The `materialized` option wasn't supported by Ecto until v3.10. Compiling with an earlier\n  version causes a compilation error.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.4 — 2024-02-16 - Changelog","doc":"","ref":"changelog.html#v2-17-4-2024-02-16"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Notifier] Add `Notifier.status/1` to expose cluster connectivty status for diagnostics.\n\n  Notifier gains `status/1`, backed by an internal `Sonar` process, for access to cluster\n  diagnostics. This will help identify whether the current node can communicate with other nodes\n  for vital functionality such as staging jobs.\n\n  A new `[:oban, :notifier, :switch]` telemetry event is emitted for diagnostic instrumentation\n  and switch events are displayed by the default logger.\n\n- [Stager] Use notifier status and leadership checks to switch between `global` and `local` mode.\n\n  The sonar-backed notifier status is more accurate than the ad-hoc stager checks previously used\n  to check connectivity. Now the staging mode is determined by a combination of sonar and\n  leadership—isolated follower nodes will correctly switch to `:local` (polling) mode.\n\n  The new detection mechanism prevents an infrequent and difficult to spot bug where isolated\n  follower nodes wouldn't process jobs.\n\n- [Backoff] Catch `GenServer` timeouts in `with_retry/2`.\n\n  Backoff's `with_retry/2` function previously rescued and retried known database exceptions, and\n  it didn't catch exits. Now it also catches timeout errors to accommodate synchronized `call`\n  based acks like those used in Pro's Smart engine in sync mode.\n\n- [Stager] Avoid transactions on follower nodes.\n\n  Only the leader needs to execute a transaction while staging. This change prevents pointless\n  empty transactions from each follower node every second. Now there are 60 staging transactions\n  per minute across the whole cluster, rather than 60 from each node.\n\n- [Notifier] Unify public function type signatures and accept a `name` or `conf` in all public\n  functions.\n\n- [Plugins] Always validate and build state in `start_link/1`\n\n  Error messages and stack traces are clearer when errors come from `start_link/1` rather than\n  `init/1`, and it avoids starting processes that will immediately crash.\n\n  This also hides `child_spec/1` in all public modules to keep it out of documentation.\n\n- [Basic] Skip taking advisory locks in testing modes.\n\n  Advisory locks are global and apply across transactions. That can break async tests with\n  overlapping unique jobs because the lock is held in a concurrent, sandboxed test.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Engine] Ensure uniqueness across args when no keys are specified regardless of insertion order.\n\n- [Basic] Force materializing the CTE when fetching jobs.\n\n  The CTE used to prevent optimizations in the Basic engine's fetch query is only referenced once,\n  which may allow the Postgres optimizer to inline it. Inlining can negate the CTE \"optimization\n  fence\", so we force the CTE to be materialized.\n\n- [Job] Restore default `priority` in the schema.\n\n  A default was still set in the database, but it was erroneously removed from the Job schema.\n\n- [Job] Fix unique typespec by unwrapping the `field` type.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.3 — 2024-01-23 - Changelog","doc":"","ref":"changelog.html#v2-17-3-2024-01-23"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Stager] Rescue and report staging errors with telemetry\n\n  Staging errors from queue contention or other database issues would cause the top level stager\n  process to crash. Eventually that could shut down the entire Oban supervision tree. Now we\n  rescue standard database connectivity issues instead and report them as errors in telemetry.\n\n- [Telemetry] Include result in job exception telemetry\n\n  Returning an `{:error, reason}` tuple triggers an :exception telemetry event, but there's still\n  a return value. Exception events for crashes, raises, timeouts, and kills will have a `nil`\n  result value.\n\n- [Queue] Add producer `handle_call` clause for engine `put_meta` calls.\n\n  Previously, the only way to put meta was through a non-blocking notification to `handle_info`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v2.17.2 — 2024-01-11 - Changelog","doc":"","ref":"changelog.html#v2-17-2-2024-01-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Oban] Support passing changeset streams to `insert_all`.\n\n  Accepting streams makes `Oban.insert_all` more flexible and may, in some circumstances, make it\n  possible to reduce memory usage for streams of large resources.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Config] Validate `:repo` option without checking for `Ecto.Repo` behaviour.\n\n  Repo wrappers that don't implement all functions of the `Ecto.Repo` behaviour are still viable and\n  shouldn't be validated with a behaviour check. This changes repo validation back to the way it\n  was done in older versions, by checking that it's a valid module that exports `config/0`.\n\n- [Peer] Handle rollback during `Oban.Peers.Postgres` peer election\n\n  Infrequently, the postgres peer election transaction returns `{:error, :rollback}`. Now that\n  return value is handled to prevent a match error.\n\n  The peer maintains its current `leader?` status on rollback—this may cause inconsistency if the\n  leader encounters an error and multiple rollbacks happen in sequence. That tradeoff is\n  acceptable because the situation is unlikely and less of an issue than crashing the peer.\n\n- [Oban] Skip queue existence check for `pause_all_queues` and `resume_all_queues` when the\n  `local_only` option is passed.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.1 — 2023-12-11 - Changelog","doc":"","ref":"changelog.html#v2-17-1-2023-12-11"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Validation] Restore validation helpers still used externally\n\n  Some of the internal validation helpers are needed by external packages that can't easily change\n  to schema validation. This restores those essential validation functions.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v2.17.0 — 2023-12-08 - Changelog","doc":"","ref":"changelog.html#v2-17-0-2023-12-08"},{"type":"extras","title":"Enhancements - Changelog","doc":"- [Oban] Add `Oban.pause_all_queues/2` and `Oban.resume_all_queues/2`.\n\n  Pause and resume all queues with a single function call and a single notification signal, rather\n  than manually looping through all queues and issuing separate calls.\n\n- [Cron] Add non-raising `Expression.parse/2` for use in `Cron.parse/2` and shared validations.\n\n  Multiple locations used `parse!` and converted a raised exception into an error tuple. That was\n  inefficient, repetitive, and violated the common practice of avoiding exceptions for flow\n  control.\n\n- [Validation] Use schema based validation for workers, plugins, and config.\n\n  Validations are now simpler and more consistent, and behaviour based notifiers such as Engine,\n  Repo, and Peer are more descriptive.\n\n- [Engine] Expand telemetry meta for all engine callbacks events.\n\n  All callbacks now include every argument in telemetry event metadata. In some situations, e.g.\n  `:init`, this simplifies testing and can be used to eliminate the need to poll a supervision\n  tree to see which queues started.\n\n- [Notifier] Add `Isolated` notifier for local use and simplified testing.\n\n  Using PG for async tests has occasional flakes due to its eventually consistent nature. In tests\n  and single node systems, we don't need to broadcast messages between instances or nodes, and a\n  simplified \"isolated\" mechanism is ideal.\n\n- [Repo] Add `Repo.query!/4` for `Ecto.Repo` parity\n\n- [Migration] Configure a third-party engine's migrator using the repo's `config` map.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- [Cron] Guard against invalid cron range expressions where the left side is greater than the\n  right, e.g. `SAT-FRI`.\n\n- [Testing] Disable the `prefix` by default in generated testing helpers.\n\n  A prefix is only necessary when it's not the standard \"public\" prefix, which is rarely the case\n  in testing helpers. This makes it easier to use testing helpers with the `Lite` engine.\n\n- [Testing] Remove `prefix` segment from `assert_enqueued` error messages.\n\n  Not all engines support a prefix and the assert/refute message in testing helpers is confusing\n  when the prefix is `nil`.","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Deprecations - Changelog","doc":"- [Gossip] The Gossip plugin is no longer needed, and shouldn't be used, by applications running\n  Oban Web v2.10 or above.\n\nFor changes prior to v2.17 see the [v2.16][prv] docs.\n\n[opc]: https://getoban.pro/docs/pro/changelog.html\n[owc]: https://getoban.pro/docs/web/changelog.html\n[prv]: https://hexdocs.pm/oban/2.16.3/changelog.html","ref":"changelog.html#deprecations"},{"type":"extras","title":"Installation","doc":"# Installation\n\nOban is published on [Hex](https://hex.pm/packages/oban). Add it to your list of dependencies in\n`mix.exs`:\n\n```elixir\n# mix.exs\ndef deps do\n  [\n    {:oban, \"~> 2.17\"}\n  ]\nend\n```\n\nThen run `mix deps.get` to install Oban and its dependencies, including [Ecto][ecto] and\n[Jason][jason]. You'll optionally need to include either [Postgrex][postgrex] for use with\nPostgres, or [EctoSQLite3][ecto_sqlite3] for SQLite3.\n\nAfter the packages are installed you must create a database migration to add the `oban_jobs` table\nto your database:\n\n```bash\nmix ecto.gen.migration add_oban_jobs_table\n```\n\nOpen the generated migration in your editor and call the `up` and `down` functions on\n`Oban.Migration`:\n\n```elixir\ndefmodule MyApp.Repo.Migrations.AddObanJobsTable do\n  use Ecto.Migration\n\n  def up do\n    Oban.Migration.up(version: 12)\n  end\n\n  # We specify `version: 1` in `down`, ensuring that we'll roll all the way back down if\n  # necessary, regardless of which version we've migrated `up` to.\n  def down do\n    Oban.Migration.down(version: 1)\n  end\nend\n```\n\nThis will run all of Oban's versioned migrations for your database. Migrations between versions\nare idempotent and rarely† change after a release. As new versions are released you may need to\nrun additional migrations.\n\n_† The only exception is the removal of `oban_beats`. That table is no longer created or modified\nin any migrations_\n\nNow, run the migration to create the table:\n\n```bash\nmix ecto.migrate\n```\n\nBefore you can run an Oban instance you must provide some base configuration:\n\n<!-- tabs-open -->","ref":"installation.html"},{"type":"extras","title":"Postgres - Installation","doc":"Running with Postgres requires using the `Oban.Engines.Basic` engine:\n\n```elixir\n# config/config.exs\nconfig :my_app, Oban,\n  engine: Oban.Engines.Basic,\n  queues: [default: 10],\n  repo: MyApp.Repo\n```","ref":"installation.html#postgres"},{"type":"extras","title":"SQLite3 - Installation","doc":"Running with SQLite3 requires using the `Oban.Engines.Lite` engine:\n\n \n```elixir\n# config/config.exs\nconfig :my_app, Oban,\n  engine: Oban.Engines.Lite,\n  queues: [default: 10],\n  repo: MyApp.Repo\n```\n\n<!-- tabs-close -->\n\nTo prevent Oban from running jobs and plugins during test runs, enable `:testing` mode in\n`test.exs`:\n\n```elixir\n# config/test.exs\nconfig :my_app, Oban, testing: :inline\n```\n\nOban instances are isolated supervision trees and must be included in your application's\nsupervisor to run. Use the application configuration you've just set and include Oban in the list\nof supervised children:\n\n```elixir\n# lib/my_app/application.ex\ndef start(_type, _args) do\n  children = [\n    MyApp.Repo,\n    {Oban, Application.fetch_env!(:my_app, Oban)}\n  ]\n\n  Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\nend\n```\n\nFinally, verify that Oban is configured and running properly. Within a new `iex -S mix` session:\n\n```elixir\niex(1)> Oban.config()\n#=> %Oban.Config{repo: MyApp.Repo}\n```\n\nYou're all set! Get started creating jobs and configuring queues in [Usage][use], or head to the\n[testing guide][test] to learn how to test with Oban.\n\n[use]: Oban.html#Usage\n[test]: testing.md\n[ecto]: https://hex.pm/packages/ecto\n[jason]: https://hex.pm/packages/jason\n[postgrex]: https://hex.pm/packages/postgrex\n[ecto_sqlite3]: https://hex.pm/packages/ecto_sqlite3","ref":"installation.html#sqlite3"},{"type":"extras","title":"Preparing for Production","doc":"# Preparing for Production\n\nThere are a few additional bits of configuration to consider before you're ready to run Oban in\nproduction. In development and test environments, job data is short lived and there's no scale to\ncontend with. Now we'll dig into enabling introspection, external observability, and maintaining\ndatabase health.","ref":"preparing_for_production.html"},{"type":"extras","title":"Logging - Preparing for Production","doc":"Oban heavily utilizes Telemetry for instrumentation at every level. From job execution, plugin\nactivity, through to every database call there's a telemetry event to hook into.\n\nThe simplest way to leverage Oban's telemetry usage is through the default logger, available with\n`Oban.Telemetry.attach_default_logger/1`. Attach the logger in your `application.ex`:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  @impl Application\n  def start(_type, _args) do\n    Oban.Telemetry.attach_default_logger()\n\n    children = [\n      ...\n    ]\n  end\nend\n```\n\nBy default, the logger emits JSON encoded logs at the `:info` level. You can disable encoding and\nfall back to structured logging with `encode: false`, or change the log level with the `:level`\noption.\n\nFor example, to log without encoding at the `:debug` level:\n\n```elixir\nOban.Telemetry.attach_default_logger(encode: false, level: :debug)\n```","ref":"preparing_for_production.html#logging"},{"type":"extras","title":"Pruning Jobs - Preparing for Production","doc":"Job introspection and uniqueness relies on keeping job rows in the database after they have\nexecuted. To prevent the `oban_jobs` table from growing indefinitely, the `Oban.Plugins.Pruner`\nplugin provides out-of-band deletion of `completed`, `cancelled` and `discarded` jobs.\n\nRetaining jobs for 7 days is a good starting point, but depending on throughput, you may wish to\nkeep jobs for even longer. Include `Pruner` in the list of plugins and configure it to retain jobs\nfor 7 days, specified in seconds:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    {Oban.Plugins.Pruner, max_age: 60 * 60 * 24 * 7},\n  ...\n```","ref":"preparing_for_production.html#pruning-jobs"},{"type":"extras","title":"Rescuing Jobs - Preparing for Production","doc":"During deployment or unexpected node restarts jobs may be left in an executing state indefinitely.\nWe call these jobs \"orphans\", but orphaning isn't a bad thing. It means that the job wasn't lost\nand it may be retried again when the system comes back online.\n\nThere are two mechanisms to mitigate orphans:\n\n1. Use the `Oban.Plugins.Lifeline` plugin to automatically move those jobs back to available so\n   they can run again.\n2. Increase the `shutdown_grace_period` to allow the system more time to finish executing before\n   shutdown.\n\nEven with a higher `shutdown_grace_period` it's possible to have orphans if there is an unexpected\ncrash or extra long running jobs.\n\nConsider adding the `Lifeline` plugin and configure it to rescue after a generous period of time,\nlike 30 minutes:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    {Oban.Plugins.Lifeline, rescue_after: :timer.minutes(30)},\n  ...\n```","ref":"preparing_for_production.html#rescuing-jobs"},{"type":"extras","title":"Error Handling - Preparing for Production","doc":"Telemetry events can be used to report issues externally to services like Sentry or AppSignal.\nWrite a handler that sends error notifications to a third party (use a mock, or something that\nsends a message back to the test process).\n\nYou can use exception events to send error reports to Honeybadger, Rollbar, AppSignal or any other\napplication monitoring platform.\n\nHere's an example reporter module for [Sentry](https://hex.pm/packages/sentry):\n\n```elixir\ndefmodule MyApp.ObanReporter do\n  def attach do\n    :telemetry.attach(\"oban-errors\", [:oban, :job, :exception], &__MODULE__.handle_event/4, [])\n  end\n\n  def handle_event([:oban, :job, :exception], measure, meta, _) do\n    extra =\n      meta.job\n      |> Map.take([:id, :args, :meta, :queue, :worker])\n      |> Map.merge(measure)\n\n    Sentry.capture_exception(meta.reason, stacktrace: meta.stacktrace, extra: extra)\n  end\nend\n```\n\nAttach the handler when your application boots:\n\n```elixir\n# application.ex\n@impl Application\ndef start(_type, _args) do\n  MyApp.ObanReporter.attach()\nend\n```","ref":"preparing_for_production.html#error-handling"},{"type":"extras","title":"Ship It! - Preparing for Production","doc":"Now you're ready to ship to production with essential logging, error reporting, and baseline\njob maintenance.\n\nFor additional observability and introspection, consider integrating with one of these external\ntools built on `Oban.Telemetry`:\n\n* [Oban Web](https://getoban.pro)—an official Oban package, it's a view of jobs, queues, and\n  metrics that you host directly within your application. Powered by Phoenix Live View and Oban\n  Metrics, it is extremely lightweight and continuously updated.\n\n* [PromEx](https://hex.pm/packages/prom_ex)—Prometheus metrics and Grafana dashboards based on\n  metrics from job events, producer events, and also from internal polling jobs to monitor queue\n  sizes.\n\n* [AppSignal](https://docs.appsignal.com/elixir/integrations/oban.html)—The AppSignal for Elixir\n  package instruments jobs performed by Oban workers, and collects metrics about your jobs'\n  performance.","ref":"preparing_for_production.html#ship-it"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting","ref":"troubleshooting.html"},{"type":"extras","title":"Jobs Stuck Executing Forever - Troubleshooting","doc":"During deployment or unexpected node restarts, jobs may be left in an `executing` state\nindefinitely. We call these jobs \"orphans\", but orphaning isn't a bad thing. It means that the job\nwasn't lost and it may be retried again when the system comes back online.\n\nThere are two mechanisms to mitigate orphans:\n\n1. Increase the [shutdown_grace_period](Oban.html#start_link/1-twiddly-options) to allow the\n   system more time to finish executing before shutdown. During shutdown each queue stops\n   fetching more jobs, but executing jobs have up to the grace period to complete. The default\n   value is `15000ms`, or 15 seconds.\n\n2. Use the [Lifeline plugin](Oban.Plugins.Lifeline.html) to automatically move those jobs back to\n   available so they can run again.\n\n```elixir\nconfig :my_app, Oban,\n    plugins: [Oban.Plugins.Lifeline],\n    shutdown_grace_period: :timer.seconds(60),\n    ...\n```","ref":"troubleshooting.html#jobs-stuck-executing-forever"},{"type":"extras","title":"Jobs or Plugins aren't Running - Troubleshooting","doc":"Sometimes `Cron` or `Pruner` plugins appear to stop working unexpectedly. Typically, this happens in\nsystems with multi-node setups where \"web\" nodes only enqueue jobs while \"worker\" nodes are\nconfigured to run queues and plugins. Most plugins require leadership to function, so When a \"web\"\nnode becomes leader the plugins go dormant.\n\nThe solution is to disable leadership with `peer: false` on any node that doesn't run plugins:\n\n```elixir\nconfig :my_app, Oban, peer: false, ...\n```","ref":"troubleshooting.html#jobs-or-plugins-aren-t-running"},{"type":"extras","title":"No Notifications with PgBouncer - Troubleshooting","doc":"Using PgBouncer's \"Transaction Pooling\" setup disables all of PostgreSQL's `LISTEN` and `NOTIFY`\nactivity. Some functionality, such as triggering job execution, scaling queues, canceling jobs,\netc. rely on those notifications.\n\nThere are several options available to ensure functional notifications:\n\n1. Switch to the `Oban.Notifiers.PG` notifier. This alternative notifier relies on Distributed\n   Erlang and exchanges messages within a cluster. The only drawback to the PG notifier is that it\n   doesn't trigger job insertion events.\n\n2. Switch `PgBouncer` to \"Session Pooling\". Session pooling isn't as resource efficient as\n   transaction pooling, but it retains all Postgres functionality.\n\n3. Use a dedicated Repo that connects directly to the database, bypassing `PgBouncer`.\n\nIf none of those options work, the [Stager][stag] will switch to `local` polling mode to ensure\nthat queues keep processing jobs.\n\n[stag]: Oban.Plugins.Stager.html","ref":"troubleshooting.html#no-notifications-with-pgbouncer"},{"type":"extras","title":"Unexpectedly Re-running All Migrations - Troubleshooting","doc":"Without a version comment on the `oban_jobs` table, it will rerun all of the migrations. This can\nhappen when comments are stripped when restoring from a backup, most commonly during a transition\nfrom one database to another.\n\nThe fix is to set the latest migrated version as a comment. To start, search through your previous\nmigrations and find the last time you ran an Oban migration. Once you've found the latest version,\ne.g. `version: 10`, then you can set that as a comment on the `oban_jobs` table:\n\n```sql\nCOMMENT ON TABLE public.oban_jobs IS '10'\"\n```\n\nOnce the comment is in place only the migrations from that version onward will\nrun.","ref":"troubleshooting.html#unexpectedly-re-running-all-migrations"},{"type":"extras","title":"Release Configuration","doc":"# Release Configuration\n\nWhile having the same Oban configuration for every environment might be fine,\nthere are certainly times you might want to make changes for a specific\nenvironment. For example, you may want to increase or decrease a queue's\nconcurrency.","ref":"release_configuration.html"},{"type":"extras","title":"Using Config Providers - Release Configuration","doc":"If you are using Elixir Releases, this is straight forward to do using [Config \nProviders][mcp]:\n\n\n```elixir\ndefmodule MyApp.ConfigProvider do\n  @moduledoc \"\"\"\n  Provide release configuration for Oban Queue Concurrency\n  \"\"\"\n\n  @behaviour Config.Provider\n\n  def init(path) when is_binary(path), do: path\n\n  def load(config, path) do\n    case parse_json(path) do\n      nil ->\n        config\n\n      queues ->\n        Config.Reader.merge(config, ingestion: [{Oban, [queues: queues]}])\n    end\n  end\n\n  defp parse_json(path) do\n    {:ok, _} = Application.ensure_all_started(:jason)\n\n    if File.exists?(path) do\n      path\n      |> File.read!()\n      |> Jason.decode!()\n      |> Map.fetch!(\"queues\")\n      |> Keyword.new(fn {key, value} -> {String.to_atom(key), value} end)\n    end\n  end\nend\n```\n\nOur config provider ensures that the `Jason` app is loaded so that we can parse\na `JSON` configuration file. Once the JSON is loaded we must extract the\n`queues` map and convert it to a keyword list where all of the keys are atoms.\nThe use of `String.to_atom/1` is safe because all of our queues names are\nalready defined.\n\nThen you include this in your `mix.exs` file, where your release is configured:\n\n```elixir\nreleases: [\n  umbrella_app: [\n    version: \"0.0.1\",\n    applications: [\n      child_app: :permanent\n    ],\n    config_providers: [{Path.To.ConfigProvider, \"/etc/config.json\"}]\n  ]\n]\n```\n\nThen when you release your app, you ensure that you have a JSON file mounted at\nwhatever path you specified above and that it contains all of your desired queues:\n\n```json\n{\"queues\": {\"special\": 1, \"default\": 10, \"events\": 20}}\n```\n\n[mcp]: https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-config-providers","ref":"release_configuration.html#using-config-providers"},{"type":"extras","title":"Writing Plugins","doc":"# Writing Plugins\n\nOban supports the use of plugins to extend its base functionality. A plugin is\nany module that begins a process and exposes a `start_link/1` function. That\nmeans a plugin may be a `GenServer`, an `Agent`, a `Task`, or any other `OTP`\nbehaviour that manages a process. Realistically you'll want a long lived process\nto complement Oban's behaviour, which makes a `GenServer` or `GenStateMachine`\nideal.\n\nUpon startup, Oban dynamically injects each plugin into its supervision tree and\npasses a few base options along with any custom configuration for the plugin.\n\nEvery plugin receives these base options:\n\n* `:conf` — An `Oban.Config` struct, which contains all of the options provided\n  to `Oban.start_link/1` in a validated and normalized form.\n* `:name` — A unique via tuple scoped to the current supervision tree. The name\n  may be used later to support helper/interface functions.","ref":"writing_plugins.html"},{"type":"extras","title":"Example Plugin - Writing Plugins","doc":"Let's look at a tiny example plugin to get a feel for how options are passed in\nand how they run. Our plugin will periodically generate a table of counts for\neach `queue` / `state` combination and then print it out. It isn't an amazingly\nuseful plugin, but it demonstrates how to handle options, work with the\n`Oban.Config` struct, and periodically interact with the `oban_jobs` table.\n\n```elixir\ndefmodule MyApp.Plugins.Breakdown do\n  @behaviour Oban.Plugin\n\n  use GenServer\n\n  import Ecto.Query, only: [group_by: 3, select: 3]\n\n  @impl Oban.Plugin\n  def start_link(opts) do\n    name = Keyword.get(opts, :name, __MODULE__)\n\n    GenServer.start_link(__MODULE__, opts, name: name)\n  end\n\n  @impl Oban.Plugin\n  def validate(opts) do\n    Oban.Validation.validate_schema(opts,\n      conf: :any,\n      name: :any,\n      interval: :pos_integer\n    )\n  end\n\n  @impl GenServer\n  def init(opts) do\n    state = Map.new(opts)\n\n    {:ok, schedule_poll(state)}\n  end\n\n  @impl GenServer\n  def handle_info(:poll, %{conf: conf} = state) do\n    breakdown =\n      Oban.Repo.all(\n        conf,\n        Oban.Job\n        |> group_by([j], [j.queue, j.state])\n        |> select([j], {j.queue, j.state, count(j.id)})\n      )\n\n    IO.inspect(breakdown)\n\n    {:noreply, schedule_poll(state)}\n  end\n\n  defp schedule_poll(%{interval: interval} = state) do\n    Process.send_after(self(), :poll, interval)\n\n    state\n  end\nend\n```\n\nThe plugin's `start_link/1` function expects a keyword list with `:name`,\n`:conf`, and `:interval` values. After extracting the `:name` for process\nregistration, it passes the options through to `init/1`. The `init` function\nconverts the keyword list of options into a map for easier access and then\nbegins a polling loop.\n\nEach iteration of the loop will query the `oban_jobs` table and print out a list\nof `{queue, state, count}` tuples like this:\n\n```\n[\n  {\"default\", \"executing\", 8},\n  {\"default\", \"retryable\", 1},\n  {\"default\", \"completed\", 3114},\n  {\"default\", \"discarded\", 1},\n  {\"events\", \"scheduled\", 3},\n  {\"events\", \"executing\", 21},\n  {\"events\", \"retryable\", 2},\n  {\"events\", \"completed\", 1783},\n  {\"events\", \"discarded\", 1},\n]\n```\n\nNow all that is left is to adding the `Breakdown` module to Oban's plugin list:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    {MyApp.Plugins.Breakdown, interval: :timer.seconds(10)}\n  ]\n  ...\n```\n\nIn the configuration we're only providing the `:interval` value. Oban injects\nthe `:name` and `:conf` automatically.","ref":"writing_plugins.html#example-plugin"},{"type":"extras","title":"Calling Interface Functions - Writing Plugins","doc":"Plugins are named dynamically using `via` tuples, which is an effective way to\nmanage process registration for multiple unique Oban instances. However, it\nmakes writing interface functions for plugins a little more complicated. The\nsolution is to make use of the `Oban.Registry` for process discovery.\n\nImagine adding a `pause` interface function to the `Breakdown` plugin we built\nabove:\n\n```elixir\nalias Oban.Registry\n\ndef pause(oban_name \\\\ Oban) do\n  oban_name\n  |> Registry.whereis({:plugin, __MODULE__})\n  |> GenServer.call(:pause)\nend\n```\n\nThe function accepts an `oban_name` argument with a default of `Oban`, which is\nthe default name for an Oban supervision tree. It then calls `whereis/2` with a\n`{:plugin, plugin_name}` tuple and uses the returned `pid` to call the plugin\nprocess.\n\nYou can then call `pause/1` from elsewhere in the application:\n\n```elixir\nMyApp.Plugins.Breakdown.pause()\n\n# or\n\nMyApp.Plugins.Breakdown.pause(MyApp.OtherOban)\n```","ref":"writing_plugins.html#calling-interface-functions"},{"type":"extras","title":"Caveats - Writing Plugins","doc":"Plugins run directly within Oban's supervision tree. A badly behaving plugin,\ne.g. a plugin that crashes repeatedly, may bring down the entire supervision\ntree. Be sure that your plugin has safety mechanisms in place to prevent\nrepeated crashes during startup.","ref":"writing_plugins.html#caveats"},{"type":"extras","title":"Recursive Jobs","doc":"# Recursive Jobs\n\nRecursive jobs, like recursive functions, call themselves after they have\nexecuted. Except unlike recursive functions, where recursion happens in a tight\nloop, a recursive job enqueues a new version of itself and may add a slight\ndelay to alleviate pressure on the queue.\n\nRecursive jobs are a great way to backfill large amounts of data where a\ndatabase migration or a mix task may not be suitable. Here are a few reasons\nthat a recursive job may be better suited for backfilling data:\n\n* Data can't be backfilled with a database migration, it may require talking to\n  an external service\n* A task may fail partway through execution; resuming the task would mean\n  starting over again, or tracking progress manually to resume where the failure\n  occurred\n* A task may be computationally intensive or put heavy pressure on the database\n* A task may run for too long and would be interrupted by code releases or other\n  node restarts\n* A task may interface with an external service and require some rate limiting\n* A job can be used directly for new records _and_ to backfill existing records\n\nLet's explore recursive jobs with a use case that builds on several of those\nreasons.","ref":"recursive-jobs.html"},{"type":"extras","title":"Use Case: Backfilling Timezone Data - Recursive Jobs","doc":"Consider a worker that queries an external service to determine what timezone a\nuser resides in. The external service has a rate limit and the response time is\nunpredictable. We have a lot of users in our database missing timezone\ninformation, and we need to backfill.\n\nOur application has an existing `TimezoneWorker` that accepts a user's `id`,\nmakes an external request and then updates the user's timezone. We can modify\nthe worker to handle backfilling by adding a new clause to `perform/1`. The new\nclause explicitly checks for a `backfill` argument and will enqueue the next job\nafter it executes:\n\n```elixir\ndefmodule MyApp.Workers.TimezoneWorker do\n  use Oban.Worker\n\n  import Ecto.Query\n\n  alias MyApp.{Repo, User}\n\n  @backfill_delay 1\n\n  @impl true\n  def perform(%{args: %{\"id\" => id, \"backfill\" => true}}) do\n    with :ok <- perform(%{args: %{\"id\" => id}}) do\n      case fetch_next(id) do\n        next_id when is_integer(next_id) ->\n          %{id: next_id, backfill: true}\n          |> new(schedule_in: @backfill_delay)\n          |> Oban.insert()\n\n        nil ->\n          :ok\n      end\n    end\n  end\n\n  def perform(%{args: %{\"id\" => id}}) do\n    update_timezone(id)\n  end\n\n  defp fetch_next(current_id) do\n    User\n    |> where([u], is_nil(u.timezone))\n    |> where([u], u.id > ^current_id)\n    |> order_by(asc: :id)\n    |> limit(1)\n    |> select([u], u.id)\n    |> Repo.one()\n  end\n\n  defp update_timezone(_id), do: Enum.random([:ok, {:error, :reason}])\nend\n```\n\nThere is a lot happening in the worker module, so let's unpack it a little bit.\n\n1. There are two clauses for `perform/1`, the first only matches when a job is\n   marked as `\"backfill\" => true`, the second does the actual work of updating the\n   timezone.\n2. The backfill clause checks that the timezone update succeeds and then uses\n   `fetch_next/1` to look for the id of the next user without a timezone.\n3. When another user needing a backfill is available it enqueues a new backfill\n   job with a one second delay.\n\nWith the new `perform/1` clause in place and our code deployed we can kick off\nthe recursive backfill. Assuming the `id` of the first user is `1`, you can\nstart the job from an `iex` console:\n\n```elixir\niex> %{id: 1, backfill: true} |> MyApp.Workers.TimezoneWorker.new() |> Oban.insert()\n```\n\nNow the jobs will chug along at a steady rate of one per second until the\nbackfill is complete (or something fails). If there are any errors the backfill\nwill pause until the failing job completes: especially useful for jobs relying\non flaky external services. Finally, when there aren't any more user's without a\ntimezone, the backfill is complete and recursion will stop.","ref":"recursive-jobs.html#use-case-backfilling-timezone-data"},{"type":"extras","title":"Building On Recursive Jobs - Recursive Jobs","doc":"This was a relatively simple example, and hopefully it illustrates the power and\nflexibility of recursive jobs. Recursive jobs are a general pattern and aren't\nspecific to Oban. In fact, aside from the `use Oban.Worker` directive there\nisn't anything specific to Oban in the recipe!","ref":"recursive-jobs.html#building-on-recursive-jobs"},{"type":"extras","title":"Reliable Scheduled Jobs","doc":"# Reliable Scheduled Jobs\n\nA common variant of recursive jobs are \"scheduled jobs\", where the goal is for a\njob to repeat indefinitely with a fixed amount of time between executions. The\npart that makes it \"reliable\" is the guarantee that we'll keep retrying the\njob's business logic when the job retries, but we'll **only schedule the next\noccurrence once**. In order to achieve this guarantee we'll make use of the\n`perform` function to receive a complete `Oban.Job` struct.\n\nTime for illustrative example!","ref":"reliable-scheduling.html"},{"type":"extras","title":"Use Case: Delivering Daily Digest Emails - Reliable Scheduled Jobs","doc":"When a new user signs up to use our site we need to start sending them daily\ndigest emails. We want to deliver the emails around the same time a user signed\nup, repeating every 24 hours. It is important that we don't spam them with\nduplicate emails, so we ensure that the next email is only scheduled on our\nfirst attempt.\n\n```elixir\ndefmodule MyApp.Workers.ScheduledWorker do\n  use Oban.Worker, queue: :scheduled, max_attempts: 10\n\n  alias MyApp.Mailer\n\n  @one_day 60 * 60 * 24\n\n  @impl true\n  def perform(%{args: %{\"email\" => email} = args, attempt: 1}) do\n    args\n    |> new(schedule_in: @one_day)\n    |> Oban.insert!()\n\n    Mailer.deliver_email(email)\n  end\n\n  def perform(%{args: %{\"email\" => email}}) do\n    Mailer.deliver_email(email)\n  end\nend\n```\n\nYou'll notice that the first `perform/1` clause only matches a job struct on the\nfirst attempt. When it matches, the first clause schedules the next iteration\nimmediately, _before_ attempting to deliver the email. Any subsequent retries\nfall through to the second `perform/1` clause, which only attempts to deliver\nthe email again. Combined, the clauses get us close to **at-most-once semantics\nfor scheduling**, and **at-least-once semantics for delivery**.","ref":"reliable-scheduling.html#use-case-delivering-daily-digest-emails"},{"type":"extras","title":"More Flexible Than CRON Scheduling - Reliable Scheduled Jobs","doc":"Delivering around the same time using cron-style scheduling would need extra\nbook-keeping to check when a user signed up, and then only deliver to those\nusers that signed up within that window of time. The recursive scheduling\napproach is more accurate and entirely self contained—when and if the digest\ninterval changes the scheduling will pick it up automatically once our code\ndeploys.\n\n_An [extensive discussion][oi27] on the Oban issue tracker prompted this example\nalong with the underlying feature that made it possible._\n\n[oi27]: https://github.com/sorentwo/oban/issues/27","ref":"reliable-scheduling.html#more-flexible-than-cron-scheduling"},{"type":"extras","title":"Considerations for Scheduling Jobs in the Very-Near-Future - Reliable Scheduled Jobs","doc":"If you use the `schedule_in` or `scheduled_at` options with a value that will\nresolve to the very-near-future, for example:\n\n```elixir\n# 1 second from now\n%{}\n|> new(schedule_in: 1)\n|> Oban.insert()\n\n# 500 milliseconds from now\nvery_soon = DateTime.utc_now() |> DateTime.add(500, :millisecond)\n\n%{}\n|> new(scheduled_at: very_soon)\n|> Oban.insert()\n```\n\nyour workers may not be aware of/attempt to perform the job until the next tick\nas specific by the [Oban Stager][stag] `:interval` option. By default this is\nset to `1_000ms`.\n\n**Be aware:** Configuring the `:interval` option below the recommended default\ncan have a considerable impact on database performance! It is not advised to\nlower this value and should only be done as a last resort after considering\nother ways to achieve your desired outcome.\n\n[stag]: Oban.Plugins.Stager.html","ref":"reliable-scheduling.html#considerations-for-scheduling-jobs-in-the-very-near-future"},{"type":"extras","title":"Reporting Job Progress","doc":"# Reporting Job Progress\n\nMost applications provide some way to generate an artifact—something that may\ntake the server a long time to accomplish. If it takes several minutes to render\na video, crunch some numbers or generate an export, users may be left wondering\nwhether your application is working. Providing periodic updates to end users\nassures them that the work is being done and keeps the application feeling\nresponsive.\n\nReporting progress is something that any background job processor with\n_unlimited execution time_ can do! Naturally, we'll look at an example built on\nOban.","ref":"reporting-progress.html"},{"type":"extras","title":"Use Case: Exporting a Large Zip File - Reporting Job Progress","doc":"Users of our site can export a zip of all the files they have uploaded. A _zip_\nfile (no, not a tar, our users don't have neck-beards) is generated on the fly,\nwhen the user requests it. Lazily generating archives is great for our server's\nutilization, but it means that users may wait a while when there are many files.\nFortunately, we know how many files will be included in the zip and we can use\nthat information to send progress reports! We will compute the archive's percent\ncomplete as each file is added and push a message to the user.","ref":"reporting-progress.html#use-case-exporting-a-large-zip-file"},{"type":"extras","title":"Before We Start - Reporting Job Progress","doc":"In the [forum question that prompted this guide][guide] the work was done\nexternally by a port process. Working with ports is well outside the scope of\nthis guide, so I've modified it for the sake of simplicity. The result is\nslightly contrived as it puts both processes within the same module, which isn't\nnecessary if the only goal is to broadcast progress. This guide is ultimately\nabout coordinating processes to report progress from a background job, so that's\nwhat we'll focus on (everything else will be rather [hand-wavy][wavy]).","ref":"reporting-progress.html#before-we-start"},{"type":"extras","title":"Coordinating Processes - Reporting Job Progress","doc":"Our worker, the creatively titled `ZippingWorker`, handles both building the\narchive and reporting progress to the client. Showing the entire module at once\nfelt distracting, so we'll start with only the module definition and the\n`perform/1` function:\n\n```elixir\ndefmodule MyApp.Workers.ZippingWorker do\n  use Oban.Worker, queue: :exports, max_attempts: 1\n\n  alias MyApp.{Endpoint, Zipper}\n\n  def perform(%_{args: %{\"channel\" => channel, \"paths\" => paths}}) do\n    build_zip(paths)\n    await_zip(channel)\n  end\n\n  # ...\nend\n```\n\nThe function accepts an Oban Job with a channel name and a list of file paths,\nwhich it immediately passes on to the private `build_zip/1`:\n\n```elixir\n  defp build_zip(paths) do\n    job_pid = self()\n\n    Task.async(fn ->\n      zip_path = Zipper.new()\n\n      paths\n      |> Enum.with_index(1)\n      |> Enum.each(fn {path, index} ->\n        :ok = Zipper.add_file(zip_path, path)\n        send(job_pid, {:progress, trunc(index / length(paths) * 100)})\n      end)\n\n      send(job_pid, {:complete, zip_path})\n    end)\n  end\n```\n\nThe function grabs the current pid, which belongs to the job, and kicks off an\nasynchronous task to handle the zipping. With a few calls to a fictional\n`Zipper` module the task works through each file path, adding it to the zip.\nAfter adding a file the task sends a `:progress` message with the percent\ncomplete back to the job. Finally, when the zip finishes, the task sends a\n`:complete` message with a path to the archive.\n\nThe asynchronous call spawns a separate process and returns immediately. In\norder for the task to finish building the zip we need to wait on it. Typically\nwe'd use `Task.await/2`, but we'll use a custom receive loop to track the task's\nprogress:\n\n```elixir\n  defp await_zip(channel) do\n    receive do\n      {:progress, percent} ->\n        Endpoint.broadcast(channel, \"zip:progress\", percent)\n        await_zip(channel)\n\n      {:complete, zip_path} ->\n        Endpoint.broadcast(channel, \"zip:complete\", zip_path)\n    after\n      30_000 ->\n        Endpoint.broadcast(channel, \"zip:failed\", \"zipping failed\")\n        raise RuntimeError, \"no progress after 30s\"\n    end\n  end\n```\n\nThe receive loop blocks execution while it waits for `:progress` or `:complete`\nmessages. When a message comes in it broadcasts to the provided channel and the\nclient receives an update (this example uses [Phoenix Channels][chan], but any\nother PubSub type mechanism would work). As a safety mechanism we have an\n`after` clause that will timeout after 30 seconds of inactivity. If the receive\nblock times out we notify the client and raise an error, failing the job.","ref":"reporting-progress.html#coordinating-processes"},{"type":"extras","title":"Made Possible by Unlimited Execution - Reporting Job Progress","doc":"Reporting progress asynchronously works in Oban because anything that blocks a\nworker's `perform/1` function will keep the job executing. Jobs aren't executed\ninside of a transaction, which alleviates any limitations on how long a job can\nrun.\n\nThis technique is suitable for any _single_ long running job where an end user\nis waiting on the results. You can read the \"Batching Jobs for Monitoring\" guide\nif you need to combine _multiple_ jobs into a single output by creating **batch\njobs**.\n\n[guide]: https://elixirforum.com/t/oban-reliable-and-observable-job-processing/22449/52\n[chan]: https://hexdocs.pm/phoenix/channels.html#content\n[wavy]: https://www.quora.com/When-someone-says-this-explanation-was-hand-wavy-what-does-that-mean","ref":"reporting-progress.html#made-possible-by-unlimited-execution"},{"type":"extras","title":"Handling Expected Failures","doc":"# Handling Expected Failures\n\nReporting job errors by sending notifications to an external service is\nessential to maintaining application health. While reporting is essential, noisy\nreports for flaky jobs can become a distraction that gets ignored. Sometimes we\n_expect_ that a job will error a few times. That could be because the job relies\non an external service that is flaky, because it is prone to race conditions, or\nbecause the world is a crazy place. Regardless of _why_ a job fails, reporting\nevery failure may be undesirable.","ref":"expected-failures.html"},{"type":"extras","title":"Use Case: Silencing Initial Notifications for Flaky Services - Handling Expected Failures","doc":"One solution for reducing noisy error notifications is to start reporting only\nafter a job has failed several times. Oban uses [Telemetry][tele] to make\nreporting errors and exceptions a simple matter of attaching a handler function.\nIn this example we will extend [Honeybadger][hb] reporting from the\n`Oban.Telemetry` documentation, but account for the number of processing attempts.\n\nTo start, we'll define a `Reportable` [protocol][pro] with a single\n`reportable?/2` function:\n\n```elixir\ndefprotocol MyApp.Reportable do\n  @fallback_to_any true\n  def reportable?(worker, attempt)\nend\n\ndefimpl MyApp.Reportable, for: Any do\n  def reportable?(_worker, _attempt), do: true\nend\n```\n\nThe `Reportable` protocol has a default implementation which always returns\n`true`, meaning it reports all errors. Our application has a `FlakyWorker`\nthat's known to fail a few times before succeeding. We don't want to see a\nreport until after a job has failed three times, so we'll add an implementation\nof `Reportable` within the worker module:\n\n```elixir\ndefmodule MyApp.Workers.FlakyWorker do\n  use Oban.Worker\n\n  defstruct []\n\n  defimpl MyApp.Reportable do\n    @threshold 3\n\n    def reportable?(_worker, attempt), do: attempt > @threshold\n  end\n\n  @impl true\n  def perform(%{args: %{\"email\" => email}}) do\n    MyApp.ExternalService.deliver(email)\n  end\nend\n```\n\nNote that we've also used `defstruct []` to make our worker a viable struct.\nThis is necessary for our protocol to dispatch correctly, as protocols consider\nall modules to be a plain `atom`.\n\nThe final step is to call `reportable?/2` from our application's error reporter,\npassing in the worker module and the attempt number:\n\n```elixir\ndefmodule MyApp.ErrorReporter do\n  alias MyApp.Reportable\n\n  def handle_event(_, _, meta, _) do\n    worker_struct = maybe_get_worker_struct(meta.job.worker)\n\n    if Reportable.reportable?(worker_struct, meta.job.attempt) do\n      context = Map.take(meta.job, [:id, :args, :queue, :worker])\n\n      Honeybadger.notify(meta.reason, context, meta.stacktrace)\n    end\n  end\n\n  def maybe_get_worker_struct(worker) do\n    try do\n      {:ok, module} = Oban.Worker.from_string(worker)\n\n      struct(module)\n    rescue\n      UndefinedFunctionError -> worker\n    end\n  end\nend\n```\n\nAttach the failure handler somewhere in your `application.ex` module:\n\n```elixir\n:telemetry.attach(\"oban-errors\", [:oban, :job, :exception], &ErrorReporter.handle_event/4, nil)\n```\n\nWith the failure handler attached you will start getting error reports **only\nafter the third error**.","ref":"expected-failures.html#use-case-silencing-initial-notifications-for-flaky-services"},{"type":"extras","title":"Giving Time to Recover - Handling Expected Failures","doc":"If a service is especially flaky you may find that Oban's default backoff\nstrategy is too fast. By defining a custom `backoff` function on the\n`FlakyWorker` we can set a linear delay before retries:\n\n```elixir\n# inside of MyApp.Workers.FlakyWorker\n\n@impl true\ndef backoff(%Job{attempt: attempt}) do\n  attempt * 60\nend\n```\n\nNow the first retry is scheduled `60s` later, the second `120s` later, and so on.","ref":"expected-failures.html#giving-time-to-recover"},{"type":"extras","title":"Building Blocks - Handling Expected Failures","doc":"Elixir's powerful primitives of behaviours, protocols and event handling make\nflexible error reporting seamless and extensible. While our `Reportable`\nprotocol only considered the number of attempts, this same mechanism is suitable\nfor filtering by any other `meta` value.\n\n[tele]: https://github.com/beam-telemetry/telemetry\n[hb]: https://www.honeybadger.io/\n[pro]: https://hexdocs.pm/elixir/Protocol.html","ref":"expected-failures.html#building-blocks"},{"type":"extras","title":"Splitting Queues Between Nodes","doc":"# Splitting Queues Between Nodes\n\nRunning every job queue on every node isn't always ideal. Imagine that your\napplication has some CPU intensive jobs that you'd prefer not to run on nodes\nthat serve web requests. Perhaps you start temporary nodes that are only meant\nto _insert_ jobs but should never _execute_ any. Fortunately, we can control\nthis by configuring certain node types, or even single nodes, to **run only a\nsubset of queues**.","ref":"splitting-queues.html"},{"type":"extras","title":"Use Case: Isolating Video Processing Intensive Jobs - Splitting Queues Between Nodes","doc":"One notorious type of CPU intensive work is video processing. When our\napplication is transcoding multiple videos simultaneously it is a _major_ drain\non system resources and may impact response times. To avoid this we can run\ndedicated worker nodes that don't serve any web requests and handle all of the\ntranscoding.\n\nWhile it's possible to separate our system into `web` and `worker` apps within\nan umbrella, that wouldn't allow us to dynamically change queues at runtime.\nLet's look at an **environment variable based method** for dynamically\nconfiguring queues at runtime.\n\nWithin `config.exs` our application is configured to run three queues:\n`default`, `media` and `events`:\n\n```elixir\nconfig :my_app, Oban,\n  repo: MyApp.Repo,\n  queues: [default: 15, media: 10, events: 25]\n```\n\nWe will use an `OBAN_QUEUES` environment variable to override the queues at\nruntime. For illustration purposes the queue parsing all happens within the\napplication module, but it would work equally well in `releases.exs` (or `runtime.exs`).\n\n```elixir\ndefmodule MyApp.Application do\n  @moduledoc false\n\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      MyApp.Repo,\n      MyApp.Endpoint,\n      {Oban, oban_opts()}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n  end\n\n  defp oban_opts do\n    env_queues = System.get_env(\"OBAN_QUEUES\")\n\n    :my_app\n    |> Application.get_env(Oban)\n    |> Keyword.update(:queues, [], &queues(env_queues, &1))\n  end\n\n  defp queues(\"*\", defaults), do: defaults\n  defp queues(nil, defaults), do: defaults\n  defp queues(_, false), do: false\n\n  defp queues(values, _defaults) when is_binary(values) do\n    values\n    |> String.split(\" \", trim: true)\n    |> Enum.map(&String.split(&1, \",\", trim: true))\n    |> Keyword.new(fn [queue, limit] ->\n      {String.to_existing_atom(queue), String.to_integer(limit)}\n    end)\n  end\nend\n```\n\nThe `queues` function's first three clauses ensure that we can fall back to the\nqueues specified in our configuration (or `false`, for testing). The fourth\nclause is much more involved, and that is where the environment parsing happens.\nIt expects the `OBAN_QUEUES` value to be a string formatted as `queue,limit`\npairs and separated by spaces. For example, to run only the `default` and\n`media` queues with a limit of 5 and 10 respectively, you would pass the string\n`default,5 media,10`.\n\nNote that the parsing clause has a couple of safety mechanisms to ensure that\nonly real queues are specified:\n\n1. It automatically trims while splitting values, so extra whitespace like won't\n   break parsing (i.e. ` default,3 `)\n2. It only converts the `queue` string to _an existing atom_, hopefully\n   preventing typos that would start a random queue (i.e. `default`)","ref":"splitting-queues.html#use-case-isolating-video-processing-intensive-jobs"},{"type":"extras","title":"Usage Examples - Splitting Queues Between Nodes","doc":"In development (or when using `mix` rather than releases) we can specify the\nenvironment variable inline:\n\n```bash\nOBAN_QUEUES=\"default,10 media,5\" mix phx.server # default: 10, media: 5\n```\n\nWe can also explicitly opt in to running all of the configured queues:\n\n```bash\nOBAN_QUEUES=\"*\" mix phx.server # default: 15, media: 10, events: 25\n```\n\nFinally, without `OBAN_QUEUES` set at all it will implicitly fall back to the\nconfigured queues:\n\n```bash\nmix phx.server # default: 15, media: 10, events: 25\n```","ref":"splitting-queues.html#usage-examples"},{"type":"extras","title":"Flexible Across all Environments - Splitting Queues Between Nodes","doc":"This environment variable based solution is more flexible than running separate\numbrella apps because we can reconfigure at any time. In a limited environment,\nlike staging, we can run _all_ the queues on a single node using the exact same\ncode we use in production. In the future, if other workers start to utilize too\nmuch CPU or RAM we can shift them to the worker node **without any code\nchanges**.\n\n_This guide was [prompted by an inquiry][oi82] on the Oban issue tracker._\n\n[oi82]: https://github.com/sorentwo/oban/issues/82","ref":"splitting-queues.html#flexible-across-all-environments"},{"type":"extras","title":"Migrating from Other Languages","doc":"# Migrating from Other Languages\n\nMigrating background jobs to Elixir is easy with Oban because everything lives in your PostgreSQL\ndatabase. Oban relies on a structured `oban_jobs` table as its job queue, and purposefully uses\nJSON as a portable data structures for serialization. That makes enqueueing jobs into Oban simple for\nany language with a PostgreSQL adapter—no Oban client necessary.","ref":"migrating-from-other-languages.html"},{"type":"extras","title":"Use Case: Inserting Jobs from Rails - Migrating from Other Languages","doc":"It's no secret that Ruby to Elixir is a common migration path for developers and existing\napplications alike. Let's explore how to write an adapter for inserting Oban jobs from a Rails\napplication.\n\nTo start, define a skeletal `ActiveRecord` model with a few conveniences for scheduling jobs:\n\n```ruby\nclass Oban::Job < ApplicationRecord\n  # This column is in use, but not used for the insert workflow.\n  self.ignored_columns = %w[errors]\n\n  # A simple wrapper around `create` that ensures the job is scheduled immediately.\n  def self.insert(worker:, args: {}, queue: \"default\", scheduled_at: nil)\n    create(\n      worker: worker,\n      queue: queue,\n      args: args,\n      scheduled_at: scheduled_at || Time.now.utc,\n      state: scheduled_at ? \"scheduled\" : \"available\"\n    )\n  end\nend\n```\n\nThe `insert` class method is a convenience that uses named arguments to force passing a `worker`\nwhile providing some defaults. The only semi-magical thing within `insert` is determining the\ncorrect state for scheduled jobs. In Oban, jobs that are ready to execute have an `available`\nstate, while jobs slated for the future are `scheduled`.\n\nTo insert a single job using the `insert` class method:\n\n```ruby\nOban::Job.insert(worker: \"MyWorker\", args: {id: 1}, queue: \"default\")\n```\n\nProvided your Elixir application has a worker named `MyWorker` and the `default` queue is running,\nOban will pick up and execute the job immediately. To schedule the job to run a minute in the\nfuture instead, pass a `scheduled_at` timestamp:\n\n```ruby\nOban::Job.insert(worker: \"MyWorker\", args: {id: 1}, scheduled_at: 1.minute.from_now.utc)\n```\n\nNow, if you're using Rails 6+, you can also use `insert_all` to batch insert jobs:\n\n```ruby\nOban::Job.insert_all([\n  {worker: \"MyWorker\", args: {id: 1}, queue: \"default\"},\n  {worker: \"MyWorker\", args: {id: 2}, queue: \"default\"},\n  {worker: \"MyWorker\", args: {id: 3}, queue: \"default\"},\n])\n```","ref":"migrating-from-other-languages.html#use-case-inserting-jobs-from-rails"},{"type":"extras","title":"Safety Guaranteed - Migrating from Other Languages","doc":"Most columns in `oban_jobs` have sensible defaults, so only the `worker` and `args` are typically\nrequired. For integrity, all required columns are marked as `NON NULL`, and several have `CHECK`\nconstraints as well for extra enforcement.\n\nThat's all you need to start migrating background jobs from Rails to Elixir (if you're using Oban,\nthat is). Naturally, the same pattern would work for Python, Node, PHP, or any other language with\na Postgres adapter.","ref":"migrating-from-other-languages.html#safety-guaranteed"},{"type":"extras","title":"Introduction to Testing","doc":"# Introduction to Testing\n\nAutomated testing is an essential component of building reliable, long-lived\napplications. Oban orchestrates your application's background tasks, so naturally,\ntesting Oban is highly recommended.","ref":"testing.html"},{"type":"extras","title":"Setup Application Config - Introduction to Testing","doc":"Ensure your app is configured for testing before you begin running tests.\n\nThere are two testing modes available:\n\n* `:inline`—jobs execute immediately within the calling process and without\n  touching the database. This mode is simple and may not be suitable for apps\n  with complex jobs.\n* `:manual`—jobs are inserted into the database where they can be verified and\n  executed when desired. This mode is more advanced and trades simplicity for\n  flexibility.\n\nIf you're just starting out, `:inline` mode is recommended:\n\n```elixir\nconfig :my_app, Oban, testing: :inline\n```\n\nFor more complex applications, or if you'd like complete control over when jobs\nrun, then use `:manual` mode instead:\n\n```elixir\nconfig :my_app, Oban, testing: :manual\n```\n\nBoth testing modes prevent Oban from running any database queries in the\nbackground. This simultaneously prevents Sandbox errors from plugin queries and\nprevents queues from executing jobs unexpectedly.","ref":"testing.html#setup-application-config"},{"type":"extras","title":"Changing Testing Modes - Introduction to Testing","doc":"Once the application starts in a particular testing mode it can't be changed.\nThat's inconvenient if you're running in `:inline` mode and don't want a\nparticular job to execute inline! `Oban.Testing` provides a helper to\ntemporarily change testing modes within the context of a function.\n\nFor example, to switch to `:manual` mode when Oban is configured for `:inline`\ntesting:\n\n```elixir\nOban.Testing.with_testing_mode(:manual, fn ->\n  Oban.insert(MyWorker.new(%{id: 123}))\n\n  assert_enqueued worker: MyWorker, args: %{id: 123}\nend)\n```\n\nOr vice-versa, switch to `:inline` mode when the application is configured for\n`:manual` mode:\n\n```elixir\nOban.Testing.with_testing_mode(:inline, fn ->\n  {:ok, %Job{state: \"completed\"}} = Oban.insert(MyWorker.new(%{id: 123}))\nend)\n```","ref":"testing.html#changing-testing-modes"},{"type":"extras","title":"Setup Testing Helpers - Introduction to Testing","doc":"Oban provides helpers to facilitate manual testing. These helpers handle the\nboilerplate of making assertions on which jobs are enqueued.\n\nThe most convenient way to use the helpers is to `use` the module within your\ntest case:\n\n```elixir\ndefmodule MyApp.Case do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      use Oban.Testing, repo: MyApp.Repo\n    end\n  end\nend\n```\n\nAlternatively, you can `use` the testing module in individual tests if you'd\nprefer not to include helpers in _every_ test.\n\n```elixir\ndefmodule MyApp.WorkerTest do\n  use MyApp.Case, async: true\n\n  use Oban.Testing, repo: MyApp.Repo\nend\n```\n\nWhichever way you choose, using `use Oban.Testing` requires the `repo` option\nbecause it's injected into many of the generated helpers.\n\nIf you are using isolation with namespaces through PostgreSQL schemas (Ecto\n\"prefixes\"), you can also specify this prefix when using `Oban.Testing`:\n\n```elixir\nuse Oban.Testing, repo: MyApp.Repo, prefix: \"private\"\n```\n\nWith Oban configured for testing and helpers in the appropriate places, you're\nready for testing. Learn about unit testing with [Testing Workers][tw],\nintegration testing with [Testing Queues][tq], or prepare for production with\n[Testing Config][tc].\n\n[tw]: testing_workers.html\n[tq]: testing_queues.html\n[tc]: testing_config.html","ref":"testing.html#setup-testing-helpers"},{"type":"extras","title":"Testing Workers","doc":"# Testing Workers\n\nWorker modules are the primary \"unit\" of an Oban system. You can (and should)\ntest a worker's callback functions locally, in-process, without touching the\ndatabase.\n\nMost worker callback functions take a single argument: an `Oban.Job` struct. A\njob encapsulates arguments, metadata, and other options. Creating jobs, and\nverifying that they're built correctly, requires some boilerplate...that's where\n`Oban.Testing.perform_job/3` comes in!","ref":"testing_workers.html"},{"type":"extras","title":"Testing Perform - Testing Workers","doc":"The `perform_job/3` helper reduces boilerplate when constructing jobs for unit\ntests and checks for common pitfalls. For example, it automatically converts\n`args` to string keys before calling `perform/1`, ensuring that perform clauses\naren't erroneously trying to match on atom keys.\n\nLet's work through test-driving a worker to demonstrate.\n\nStart by defining a test that creates a user and then use `perform_job` to\nmanually call an account activation worker. In this context \"activation\" could\nmean sending an email, notifying administrators, or any number of\nbusiness-critical functions—what's important is how we're testing it.\n\n```elixir\ndefmodule MyApp.ActivationWorkerTest do\n  use MyApp.Case, async: true\n\n  test \"activating a new user\" do\n    user = MyApp.User.create(email: \"parker@example.com\")\n\n    {:ok, _user} = perform_job(MyApp.ActivationWorker, %{id: user.id})\n  end\nend\n```\n\nRunning the test at this point will raise an error that explains the module\ndoesn't implement the `Oban.Worker` behaviour.\n\n```text\n1) test activating a new account (MyApp.ActivationWorkerTest)\n\n   Expected worker to be a module that implements the Oban.Worker behaviour, got:\n\n   MyApp.ActivationWorker\n\n   code: {:ok, user} = perform_job(MyApp.ActivationWorker, %{id: user.id})\n```\n\nTo fix it, define a worker module with the appropriate signature and return\nvalue:\n\n```elixir\ndefmodule MyApp.ActivationWorker do\n  use Oban.Worker\n\n  @impl Worker\n  def perform(%Job{args: %{\"id\" => user_id}}) do\n    MyApp.Account.activate(user_id)\n  end\nend\n```\n\nThe `perform_job/3` helper's errors will guide you through implementing a\ncomplete worker with the following assertions:\n\n* That the worker implements the `Oban.Worker` behaviour\n* That `args` is encodable/decodable to JSON and always has string keys\n* That the options provided build a valid job\n* That the return value is expected, e.g. `:ok`, `{:ok, value}`, `{:error, value}` etc.\n* That a custom `new/1,2` callback works properly\n\nIf all of the assertions pass, then you'll get the result of `perform/1` for you\nto make additional assertions on.","ref":"testing_workers.html#testing-perform"},{"type":"extras","title":"Testing Other Callbacks - Testing Workers","doc":"You may wish to test less-frequently used worker callbacks such as `backoff/1`\nand `timeout/1`, but those callbacks don't have dedicated testing helpers.\nNever fear, it's adequate to build a job struct and test callbacks directly!\n\nHere's a sample test that asserts the backoff value is simply two-times the\njob's `attempt`:\n\n```elixir\ntest \"calculating custom backoff as a multiple of job attempts\" do\n  assert 2 == MyWorker.backoff(%Oban.Job{attempt: 1})\n  assert 4 == MyWorker.backoff(%Oban.Job{attempt: 2})\n  assert 6 == MyWorker.backoff(%Oban.Job{attempt: 3})\nend\n```\n\nSimilarly, here's a sample that verifies a `timeout/1` callback always returns\nsome number of milliseconds:\n\n```elixir\ntest \"allowing a multiple of the attempt as job timeout\" do\n  assert 1000 == MyWorker.timeout(%Oban.Job{attempt: 1})\n  assert 2000 == MyWorker.timeout(%Oban.Job{attempt: 2})\nend\n```\n\nJobs are Ecto schemas, and therefore structs. There isn't anything magical about\nthem! Explore the `Oban.Job` documentation to see all of the types and fields\navailable for testing.","ref":"testing_workers.html#testing-other-callbacks"},{"type":"extras","title":"Testing Queues","doc":"# Testing Queues\n\nWhere workers are the primary \"unit\" of an Oban system, queues are the\n\"integration\" point between the database and your application. That means to\ntest queues and the jobs within them, your tests will have to interact with the\ndatabase. To simplify that interaction, reduce boilerplate, and make assertions\nmore expressive `Oban.Testing` provides a variety of helpers.","ref":"testing_queues.html"},{"type":"extras","title":"Asserting Enqueued Jobs - Testing Queues","doc":"During test runs you don't typically want to execute jobs. Rather, you need\nto verify that the job was enqueued properly. With the recommended test setup\nqueues and plugins are disabled, and jobs won't execute at all. The\n`Oban.Testing.assert_enqueued/2` and `Oban.Testing.refute_enqueued/2` helpers\nsimplify running queries to check for those `available` or `scheduled` jobs\nsitting in the database.\n\nLet's look at an example where we want to check that an activation job is\nenqueued after a user signs up:\n\n```elixir\ntest \"scheduling activation upon sign up\" do\n  {:ok, account} = MyApp.Account.sign_up(email: \"parker@example.com\")\n\n  assert_enqueued worker: MyApp.ActivationWorker, args: %{id: account.id}, queue: :default\nend\n```\n\nLikewise, we can also refute that a job was enqueued. The `refute_enqueued`\nhelper takes the same arguments as `assert_enqueued`, though you should take\ncare to be as unspecific as possible.\n\nBuilding on the example above, let's refute that a job is enqueued when account\nsign up fails:\n\n```elixir\ntest \"bypassing activation when sign up fails\" do\n  {:error, _reason} = MyApp.Account.sign_up(email: \"parker@example.com\")\n\n  refute_enqueued worker: MyApp.ActivationWorker\nend\n```","ref":"testing_queues.html#asserting-enqueued-jobs"},{"type":"extras","title":"Asserting Multiple Jobs - Testing Queues","doc":"Asserting and refuting about a single job isn't always enough. Sometimes you\nneed to check for multiple jobs at once, or perform more complex assertions on\nthe jobs themselves. In that situation, you can use `all_enqueued` instead.\n\nThe first example we'll look at asserts that multiple jobs from the same worker are\nenqueued all at once:\n\n```elixir\ntest \"enqueuing one job for each child record\" do\n  :ok = MyApp.Account.notify_owners(account())\n\n  assert jobs = all_enqueued(worker: MyApp.NotificationWorker)\n  assert 3 == length(jobs)\nend\n```\n\nThe `enqueued` helpers all build dynamic queries to check for jobs within the\ndatabase. Dynamic queries don't work for complex objects with nested values or a\npartial set of keys. In that case, you can use `all_enqueued` to pull jobs into\nyour tests and use the full power of pattern matching for assertions.\n\n```elixir\ntest \"enqueued jobs have args that match a particular pattern\" do\n  :ok = MyApp.Account.notify_owners(account())\n\n  for job <- all_enqueued(queue: :default) do\n    assert %{\"email\" => _, \"avatar\" => %{\"url\" => _}} = job.args\n  end\nend\n```","ref":"testing_queues.html#asserting-multiple-jobs"},{"type":"extras","title":"Integration Testing Queues - Testing Queues","doc":"During integration tests it may be necessary to run jobs because they do work\nessential for the test to complete, i.e. sending an email, processing media,\netc. You can execute all available jobs in a particular queue by calling\n`Oban.drain_queue/1,2` directly from your tests.\n\nFor example, to process all pending jobs in the \"mailer\" queue while testing\nsome business logic:\n\n```elixir\ndefmodule MyApp.BusinessTest do\n  use MyApp.DataCase, async: true\n\n  alias MyApp.{Business, Worker}\n\n  test \"we stay in the business of doing business\" do\n    :ok = Business.schedule_a_meeting(%{email: \"monty@brewster.com\"})\n\n    assert %{success: 1, failure: 0} = Oban.drain_queue(queue: :mailer)\n\n    # Now, make an assertion about the email delivery\n  end\nend\n```\n\nSee `Oban.drain_queue/1,2` for a myriad of options and additional details.","ref":"testing_queues.html#integration-testing-queues"},{"type":"extras","title":"Testing Config","doc":"# Testing Config\n\nAn Oban instance's config is built from options passed to `Oban.start_link`\nduring initialization and it determines the supervision tree that Oban builds.\nThe instance config is encapsulated by an `Oban.Config` struct, which is then\nreferenced by plugins, queues, and most public functions.","ref":"testing_config.html"},{"type":"extras","title":"Testing Dynamic Config - Testing Config","doc":"Oban automatically normalizes and validates config options on initialization.\nThat covers any static configuration, but it won't help when config is generated\ndynamically, at runtime.\n\nThe `Oban.Config.validate/1` helper is used internally when the config\ninitializes. Along with each top level option, like the `:engine` or `:repo`, it\nrecursively verifies all queue and plugin options.\n\nYou can use `validate/1` to verify dynamic configuration, e.g. options specified\nin `runtime.exs`:\n\n```elixir\ntest \"production oban config is valid\" do\n  config =\n    \"config/config.exs\"\n    |> Config.Reader.read!(env: :prod)\n    |> get_in([:my_app, Oban])\n\n  assert :ok = Oban.Config.validate(config)\nend\n```\n\nWhen the configuration contains **any** invalid options, like an invalid engine,\nyou'll see the test fail with an error like this:\n\n```elixir\n{:error, \"expected :engine to be an Oban.Queue.Engine, got: MyApp.Repo\"}\n```","ref":"testing_config.html#testing-dynamic-config"},{"type":"extras","title":"Testing Dynamic Plugin Config - Testing Config","doc":"Validation is especially helpful for plugins that have complex configuration,\ne.g. `Cron` or the `Dynamic*` plugins from Oban Pro. As of Oban v2.12.0 all\nplugins implement the `c:Oban.Plugin.validate/1` callback and we can test them\nin isolation as well as through the top level config.\n\nSuppose you have a helper function that returns a crontab config at runtime:\n\n```elixir\ndefmodule MyApp.Oban do\n  def cron_config do\n    [crontab: [{\"0 24 * * *\", MyApp.Worker}]]\n  end\nend\n```\n\nYou can call that function within a test and then assert that it is valid with\n`c:Oban.Plugin.validate/1`:\n\n```elixir\ntest \"testing cron plugin configuration\" do\n  config = MyApp.Oban.cron_config()\n\n  assert :ok = Oban.Plugins.Cron.validate(config)\nend\n```\n\nRunning this test will return an error tuple, showing that the cron expression\nisn't valid.\n\n```elixir\n{:error, %ArgumentError{message: \"expression field 24 is out of range 0..23\"}}\n```\n\nSwitch the expression from `0 24 * * *` to `0 23 * * *`, run the tests again,\nand everything passes!","ref":"testing_config.html#testing-dynamic-plugin-config"},{"type":"extras","title":"Upgrading to v2.0","doc":"# Upgrading to v2.0\n\n> This information is extracted and expanded from the [CHANGELOG](changelog.html).\n\nThis is a big release with a _ton_ of new features and fixes, but also a few\nbreaking changes. This guide will walk you through the upgrade process.","ref":"v2-0.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.0","doc":"Update Oban to the latest version:\n\n```elixir\n  defp deps do\n    [\n      {:oban, \"~> 2.0.0\"}\n      ...\n    ]\n  end\n```\n\nIf you're an Oban Web+Pro user you'll also need to bump Oban.Web and add Oban.Pro:\n\n```elixir\n  defp deps do\n    [\n      {:oban, \"~> 2.0.0\"},\n      {:oban_web, \"~> 2.0.0\", organization: \"oban\"},\n      {:oban_pro, \"~> 0.3.0\", organization: \"oban\"}\n      ...\n    ]\n  end\n```","ref":"v2-0.html#bump-your-deps"},{"type":"extras","title":"Oban.Worker Changes - Upgrading to v2.0","doc":"The `perform/2` callback is replaced with `perform/1`, where the only argument\nis an Oban.Job struct. This unifies the interface for all Oban.Worker callbacks\nand helps to eliminate confusion around pattern matching on arguments.\n\nChange all worker definitions from accepting `args` and a `job` to only accept a\njob and match on the nested `args` key:\n\n```diff\n- def perform(%{\"id\" => id}, _job)\n+ def perform(%Job{args: %{\"id\" => id}})\n```\n\nFor finer control of job backoff the `backoff/1` callback now expects a job\nstruct instead of an integer. Change any workers that expect an attempt number\nto match on the full job struct:\n\n```diff\n- def backoff(attempt)\n+ def backoff(%Job{attempt: attempt})\n```","ref":"v2-0.html#oban-worker-changes"},{"type":"extras","title":"Update Your Config - Upgrading to v2.0","doc":"The `:verbose` setting was renamed to `:log` to better align with values\naccepted by `Ecto.Repo`.\n\n```diff\nconfig :my_app, Oban,\n- verbose: false,\n+ log: false\n  ...\n```\n\nPruning is now handled by the new plugin system. Replace `:prune` in your config\nand pass a `:max_age` value to the plugin:\n\n```diff\nconfig :my_app, Oban,\n- prune: {:max_age, 60},\n- prune_interval: 30_000,\n- prune_limit: 5000,\n+ plugins: [{Oban.Plugins.Pruner, max_age: 60}]\n  ...\n```\n\nIn test mode you can disable pruning by setting `plugins` to false instead:\n\n```diff\nconfig :my_app, Oban,\n-  prune: false,\n+  plugins: false\n  ...\n```\n\n🌟 Oban.Pro users may opt to use the [DynamicPruner](dynamic_pruning.html)\ninstead for finer control. For example, to set per-state retention periods:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [{\n    Oban.Pro.Plugins.DynamicPruner,\n    state_overrides: [\n      completed: {:max_age, {1, :day}},\n      discarded: {:max_age, {1, :month}}\n    ]\n  }]\n  ...\n```\n\nPulse tracking and orphaned job rescue are removed from base Oban. This change\nmeans that you will need to manually rescue any jobs left executing state after\na crash or forced shutdown.\n\nRemove any `:beats_maxage`, `:rescue_after` or `:rescue_interval` settings from\nyour config:\n\n```diff\nconfig :my_app, Oban,\n-  beats_maxage: 10,\n-  rescue_after: 30_000,\n-  rescue_interval: 5_000\n  ...\n```\n\n🌟 Oban.Pro users may use the [Lifeline](lifeline.html) plugin to retain\nautomatic orphaned job rescue with lightweight heartbeat recording:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [Oban.Pro.Plugins.Lifeline]\n  ...\n```","ref":"v2-0.html#update-your-config"},{"type":"extras","title":"Update Your Tests - Upgrading to v2.0","doc":"The new `perform_job` test helper automates validating, normalizing and\nperform jobs in unit tests.\n\nTo update your tests replace any calls to `perform` with the new helper:\n\n```diff\n- assert :ok = MyApp.Worker.perform(%{id: 1}, %Oban.Job{})\n+ assert :ok = perform_job(MyApp.Worker, %{id: 1})\n```\n\nThe `perform_job` helper will verify the worker, the arguments and any\nprovided options. It will then verify that your worker returns a valid result\nand return the value for you to assert on.\n\nWithin integration tests replace `drain_queue/3` with `drain_queue/2`:\n\n```diff\n- Oban.drain_queue(:myqueue, with_safety: false)\n+ Oban.drain_queue(queue: :myqueue, with_safety: false)\n```","ref":"v2-0.html#update-your-tests"},{"type":"extras","title":"Update Telemetry - Upgrading to v2.0","doc":"Telemetry event names have changed, along with some of the metadata and timing\nunits. The new event names are consistent and align with the now-standard\n`telemetry:span` conventions.\n\nUpdate handlers to and match on the new event names:\n\n```diff\n- def handle_event([:oban, :failure], measure, meta, _) do\n+ def handle_event([:oban, :job, :exception], measure, meta, _) do\n```\n\nAnd update the `attach` calls:\n\n```diff\n- :telemetry.attach(\"oban\", [[:oban, :failure]], &handle_event/4, %{})\n+ :telemetry.attach(\"oban\", [[:oban, :job, :exception]], &handle_event/4, %{})\n```\n\nFinally, replace references to `meta.stack` with `meta.stacktrace`:\n\n```diff\n- Sentry.capture_exception(meta.error, stacktrace: meta.stack, extra: extra)\n+ Sentry.capture_exception(meta.error, stacktrace: meta.stacktrace, extra: extra)\n```\n\nHere is a conversion chart for any other handlers you may have:\n\n```text\n[:oban, :started] -> [:oban, :job, :start]\n[:oban, :success] -> [:oban, :job, :stop]\n[:oban, :failure] -> [:oban, :job, :exception]\n[:oban, :trip_circuit] -> [:oban, :circuit, :trip]\n[:oban, :open_circuit] -> [:oban, :circuit, :open]\n```","ref":"v2-0.html#update-telemetry"},{"type":"extras","title":"Update Oban.Web (Optional) - Upgrading to v2.0","doc":"Oban.Web is streamlined to share configuration with Oban and use the new plugin\nsystem. Remove any Oban.Web specific configuration:\n\n```diff\n- config :my_app, Oban.Web, repo: MyApp.Repo\n```\n\nNext, add the necessary plugins to the Oban config:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    Oban.Pro.Plugins.DynamicPruner,\n    Oban.Pro.Plugins.Lifeline,\n    Oban.Web.Plugins.Stats\n  ],\n  ...\n```\n\nNow remove ObanWeb from your application's supervision tree:\n\n```diff\nchildren = [\n  MyApp.Repo,\n  MyApp.Endpoint,\n  {Oban, oban_opts},\n- {ObanWeb, oban_web_opts}\n]\n```\n\nAnd delete any existing `ObanWeb` migrations:\n\n```diff\n- defdelegate up, to: ObanWeb.Migrations\n- defdelegate down, to: ObanWeb.Migrations\n```\n\nFinally, use the `oban_dashboard` macro to easily mount in your router:\n\n```diff\n+ import Oban.Web.Router\n\nscope \"/\" do\n  pipe_through :browser\n\n-  live \"/oban\", ObanWeb.DashboardLive, layout: {ObanWeb.LayoutView, \"app.html\"}\n+  oban_dashboard(\"/oban\")\nend\n```","ref":"v2-0.html#update-oban-web-optional"},{"type":"extras","title":"Upgrading to v2.6","doc":"# Upgrading to v2.6\n\nFor Oban OSS users the v2.6 upgrade is a drop in replacement—there isn't\nanything to do! However, Web+Pro users will need to make some changes to unlock\nthe goodness of engines.","ref":"v2-6.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.6","doc":"Update Oban, Web, and Pro to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.6\"},\n  {:oban_web, \"~> 2.6\", repo: \"oban\"},\n  {:oban_pro, \"~> 0.7\", repo: \"oban\"}\n  ...\n]\n```\n\nBe sure to specify **both `:oban_web` and `:oban_pro`** if you use them both.\nThere aren't any dependencies between Web and Pro now. That means you're free to\nuse Pro for workers and only include Web for Phoenix servers, etc.","ref":"v2-6.html#bump-your-deps"},{"type":"extras","title":"Switch to the SmartEngine - Upgrading to v2.6","doc":"The `SmartEngine` uses centralized records to track and exchange state globally,\nenabling features such as global concurrency.\n\nFirst, create a migration to add the new `oban_producers` table:\n\n```bash\n$ mix ecto.gen.migration add_oban_producers\n```\n\nWithin the migration module:\n\n```elixir\nuse Ecto.Migration\n\ndefdelegate change, to: Oban.Pro.Migrations.Producers\n```\n\nIf you have multiple Oban instances or use prefixes, you can specify the prefix\nand create multiple tables in one migration:\n\n```elixir\nuse Ecto.Migration\n\ndef change do\n  Oban.Pro.Migrations.Producers.change()\n  Oban.Pro.Migrations.Producers.change(prefix: \"special\")\n  Oban.Pro.Migrations.Producers.change(prefix: \"private\")\nend\n```\n\nNext, update your config to use the `SmartEngine`:\n\n```elixir\nconfig :my_app, Oban,\n  engine: Oban.Pro.Queue.SmartEngine,\n  ...\n```\n\nIf you have multiple Oban instances you need to configure each one to use the\n`SmartEngine`, otherwise they'll default to the `Basic` engine.","ref":"v2-6.html#switch-to-the-smartengine"},{"type":"extras","title":"Start Gossiping - Upgrading to v2.6","doc":"Oban Pro no longer writes heartbeat records to `oban_beats`. Instead, any Oban\ninstance that runs queues must use the `Gossip` plugin to broadcast status via\nPubSub.\n\nTo start, include the `Gossip` plugin in your Oban config:\n\n```elixir\nconfig :my_app, Oban,\n  plugins: [\n    Oban.Plugins.Gossip\n    ...\n```\n\nWith the default configuration the plugin will broadcast every 1 second. If that\nis too frequent you can configure the interval:\n\n```elixir\n  plugins: [\n    {Oban.Plugins.Gossip, interval: :timer.seconds(5)}\n    ...\n```","ref":"v2-6.html#start-gossiping"},{"type":"extras","title":"Remove the Workflow Manager - Upgrading to v2.6","doc":"Due to an improvement in how configuration is passed to workers the\n`WorkflowManager` plugin is no longer needed. You can remove it from your list\nof plugins:\n\n```diff\nplugins: [\n  Oban.Pro.Plugins.Lifeline,\n- Oban.Pro.Plugins.WorkflowManager,\n  ...\n```","ref":"v2-6.html#remove-the-workflow-manager"},{"type":"extras","title":"Remove Extra Lifeline Options - Upgrading to v2.6","doc":"The `Lifeline` plugin is simplified and doesn't accept as many configuration\noptions. If you previously configured the `record_interval` or `delete_interval`\nyou can remove them:\n\n```diff\nplugins: [{\n  Oban.Pro.Plugins.Lifeline,\n- delete_interval: :timer.minutes(10),\n- record_interval: :timer.seconds(10),\n  rescue_interval: :timer.minutes(5)\n}]\n```","ref":"v2-6.html#remove-extra-lifeline-options"},{"type":"extras","title":"Drop the Beats Table - Upgrading to v2.6","doc":"Once you've rolled out the switch to producer records, the smart engine and the\ngossip plugin you are free to remove the `oban_beats` table at your discretion\n(preferably in a follow up release, to prevent errors):\n\n```bash\n$ mix ecto.gen.migration drop_oban_beats\n```\n\nWithin the generated migration module:\n\n```elixir\nuse Ecto.Migration\n\ndef up do\n  drop_if_exists table(\"oban_beats\")\n  drop_if_exists table(\"oban_beats\", prefix: \"private\") # If you have any prefixes:\nend\n\ndef down do\n  # No going back!\nend\n```","ref":"v2-6.html#drop-the-beats-table"},{"type":"extras","title":"Upgrading to v2.11","doc":"# Upgrading to v2.11\n\nThis Oban release includes a required migration and a couple of optional, but\nrecommended, changes.","ref":"v2-11.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.11","doc":"Update Oban, Web, and Pro to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.11\"},\n  {:oban_pro, \"~> 0.10\", repo: \"oban\"},\n  {:oban_web, \"~> 2.9\", repo: \"oban\"}\n]\n```","ref":"v2-11.html#bump-your-deps"},{"type":"extras","title":"Run Oban.Migrations for v11 - Upgrading to v2.11","doc":"Oban's new leadership mechanism uses an unlogged table to track state globally.\nThe v11 migration creates a new `oban_peers` table, and is required for\nleadership—without it many plugins won't run.\n\nTo get started, create a migration to create the table:\n\n```bash\n$ mix ecto.gen.migration create_oban_peers\n```\n\nWith in the generated migration module:\n\n```elixir\nuse Ecto.Migration\n\ndef up, do: Oban.Migrations.up(version: 11)\n\ndef down, do: Oban.Migrations.down(version: 11)\n```\n\nIf you have multiple Oban instances or use an alternate prefix you'll need to\nrun the migration for each prefix.\n\nThe `Oban.Peer` module will safely handle a missing `oban_peers` table and log a\nwarning.","ref":"v2-11.html#run-oban-migrations-for-v11"},{"type":"extras","title":"Update Notifier Names - Upgrading to v2.11","doc":"Now that we've pulled the `PG` notifier in from Oban Pro there are a few\nnaming changes you should make.\n\n1. If your config explicitly declares `Oban.PostgresNotifier`, change it to the\n   new namespaced version: `Oban.Notifiers.Postgres`:\n\n```diff\n-notifier: Oban.PostgresNotifier\n+notifier: Oban.Notifiers.Postgres\n```\n\n2. If you are using Pro's `PG` notifier, change it to the new namespaced version\n   from Oban:\n\n```diff\n-notifier: Oban.Pro.Notifiers.PG\n+notifier: Oban.Notifiers.PG\n```","ref":"v2-11.html#update-notifier-names"},{"type":"extras","title":"Check Configuration for Multi-Node Setups - Upgrading to v2.11","doc":"This release introduces centralized leadership through the `Oban.Peer`\nbehaviour. To prevent duplicate plugin work across nodes, only one Oban instance\nwithin a cluster may be the leader. Unfortunately, if a node that doesn't run\nplugins becomes the leader then jobs may get stuck as `available` and plugins\nlike `Cron` or `Pruner` won't run.\n\nThe simplest solution is avoid `plugins: false` altogether:\n\n```diff\n-plugins: false\n+plugins: []\n```\n\nSee the [Troubleshooting guide](./troubleshooting.md) for more context.","ref":"v2-11.html#check-configuration-for-multi-node-setups"},{"type":"extras","title":"Swap the Compound Index (Optional, but Recommended) - Upgrading to v2.11","doc":"Oban uses a single compound index for most queries. The index is comprised of\njob `state`, `queue`, `priority`, `scheduled_at`, and `id`. That single index is\nflexible enough to power most of Oban's queries. However, the column order is\nimportant, and the order created by Oban's migrations isn't optimal in all\nsituations.\n\nIf you're experiencing slow plugin queries, e.g. the `Stager`, then you may\nbenefit from swapping the indexes. To do so, create a migration:\n\n```bash\n$ mix ecto.gen.migration swap_primary_oban_indexes\n```\n\nWithin the generated migration module:\n\n```elixir\n@disable_ddl_transaction true\n@disable_migration_lock true\n\ndef change do\n  create_if_not_exists index(\n    :oban_jobs,\n    [:state, :queue, :priority, :scheduled_at, :id],\n    concurrently: true,\n    prefix: \"public\"\n  )\n\n  drop_if_exists index(\n    :oban_jobs,\n    [:queue, :state, :priority, :scheduled_at, :id],\n    concurrently: true,\n    prefix: \"public\"\n  )\nend\n```\n\nBe sure to reference the correct prefix if your `oban_jobs` table uses a prefix\nother than `public`.","ref":"v2-11.html#swap-the-compound-index-optional-but-recommended"},{"type":"extras","title":"Upgrading to v2.12","doc":"# Upgrading to v2.12\n\nThis Oban release includes a couple of optional configuration changes to aid in\ntesting and development.","ref":"v2-12.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.12","doc":"Update Oban (and optionally Pro) to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.12\"},\n  {:oban_pro, \"~> 0.11\", repo: \"oban\"}\n]\n```","ref":"v2-12.html#bump-your-deps"},{"type":"extras","title":"Modify Configuration for Testing - Upgrading to v2.12","doc":"The new `:testing` option automates configuring an Oban instance for testing.\nMake the following change to your `test.exs` to opt into `:manual` testing mode:\n\n```diff\n# test.exs\n- config :my_app, Oban, queues: false, plugins: false\n+ config :my_app, Oban, testing: :manual\n```\n\nIf you'd prefer to run jobs inline as they're inserted, without involving the\ndatabase, then you can use `:inline` mode instead:\n\n```elixir\nconfig :my_app, Oban, testing: :inline\n```\n\nSee the [testing guide](testing.html) to learn more about test configuration.","ref":"v2-12.html#modify-configuration-for-testing"},{"type":"extras","title":"Upgrading to v2.14","doc":"# Upgrading to v2.14\n\nThis Oban release includes a number of configuration changes and deprecations for redundant\nfunctionality.","ref":"v2-14.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.14","doc":"Update Oban (and optionally Pro) to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.14\"},\n  {:oban_pro, \"~> 0.13\", repo: \"oban\"}\n]\n```","ref":"v2-14.html#bump-your-deps"},{"type":"extras","title":"Remove Repeater and Stager Plugins - Upgrading to v2.14","doc":"The `Repeater` plugin is no longer necessary as the new `Stager` falls back to polling mode\nautomatically. Remove the `Repeater` from your plugins:\n\n```diff\n plugins: [\n   Oban.Plugins.Lifeline,\n   Oban.Plugins.Pruner,\n-  Oban.Plugins.Repeater\n```\n\nThe `Stager` is no longer a plugin because it's essential for queue operation. If you've\noverridden the staging interval:\n\n1. Reconsider whether that's necessary, staging is optimized to be a light-weight operation.\n2. If you're set on using a different interval, move it to `:stage_interval`\n\n```diff\n plugins: [\n   Oban.Plugins.Lifeline,\n   Oban.Plugins.Pruner,\n-  {Oban.Plugins.Stager, interval: 5_000}\n ],\n+ stage_interval: 5_000\n```","ref":"v2-14.html#remove-repeater-and-stager-plugins"},{"type":"extras","title":"Ensure Configuration for Testing - Upgrading to v2.14","doc":"Now that `Stager` isn't a plugin, it isn't disabled by `plugins: false`. Be sure\nto use the `:testing` option [introduced in v2.12][v212] to automate configuration:\n\n```diff\n# test.exs\n- config :my_app, Oban, queues: false, plugins: false\n+ config :my_app, Oban, testing: :manual\n```\n\nWithout this change you may see a flurry of `DBConnection.OwnershipError` errors\nduring test runs.\n\n[v212]: v2-12.html#modify-configuration-for-testing","ref":"v2-14.html#ensure-configuration-for-testing"},{"type":"extras","title":"Upgrading to v2.17","doc":"# Upgrading to v2.17\n\nThis Oban release includes an optional, but recommended migration.\n\n> #### Prevent Duplicate Insert Notifications {: .warning}\n>\n> You must either [run the v12 migrations](#run-oban-migrations-for-v12-optional) or [disable\n> insert triggers](#disable-insert-notifications-optional) in your Oban configuration, otherwise\n> you'll receive duplicate insert notifications for each job.","ref":"v2-17.html"},{"type":"extras","title":"Bump Your Deps - Upgrading to v2.17","doc":"Update Oban (and optionally Pro) to the latest versions:\n\n```elixir\n[\n  {:oban, \"~> 2.17\"},\n  {:oban_pro, \"~> 1.2\", repo: \"oban\"}\n]\n```","ref":"v2-17.html#bump-your-deps"},{"type":"extras","title":"Run Oban.Migrations for v12 (Optional) - Upgrading to v2.17","doc":"The v12 migration removes insert triggers and relaxes the `priority` column's check constraint to\nallow values in the new range of `0..9`.\n\nTo get started, create a migration to create the table:\n\n```bash\n$ mix ecto.gen.migration upgrade_oban_jobs_to_v12\n```\n\nWithin the generated migration module:\n\n```elixir\nuse Ecto.Migration\n\ndef up, do: Oban.Migrations.up(version: 12)\n\ndef down, do: Oban.Migrations.down(version: 12)\n```\n\nIf you have multiple Oban instances, or use an alternate prefix, you'll need to run the migration\nfor each prefix.","ref":"v2-17.html#run-oban-migrations-for-v12-optional"},{"type":"extras","title":"Disable Insert Notifications (Optional) - Upgrading to v2.17","doc":"If you opt not to run the v12 migration to disable Postgres triggers, then you should disable\ninsert notifications in your configuration:\n\n```diff\n config :my_app, Oban,\n+  insert_trigger: false,\n   ...\n```","ref":"v2-17.html#disable-insert-notifications-optional"},{"type":"extras","title":"Remove the Gossip Plugin - Upgrading to v2.17","doc":"The Gossip plugin is no longer used by Oban Web and now useless. You can safely remove it from\nyour configuration:\n\n```diff\n config :my_app, Oban,\n   plugins: [\n-    Oban.Plugins.Gossip,\n   ]\n```","ref":"v2-17.html#remove-the-gossip-plugin"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}